/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : modec                                                         *
 * Module  : MandelbrotCell                                                *
 * Package : MandelbrotCellModule                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:20:44 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: MandelbrotCell.cpp,v $
 *   Revision 1.1  2005/11/25 09:20:44  bhadorn
 *   Created
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <examples/mandelbrot/MandelbrotCellModule/MandelbrotCell.h> 
#include <examples/mandelbrot/Interfaces/IMandelbrotFrame.hpp>
#include <zeusbase/System/Interfaces/IXMLNode.hpp>
#include <zeusbase/System/LoggerManager.h>

USING_NAMESPACE_Zeus


/****************************************************************************/
/*! Creates a mandelbrot cell
*/
/****************************************************************************/
TMandelbrotCell::TMandelbrotCell(IXMLNode* node) : TAbstractCell(node)
{
}
    
/****************************************************************************/
/*! Destroys the mandelbrot cell
*/
/****************************************************************************/
TMandelbrotCell::~TMandelbrotCell()
{

}


/****************************************************************************/
/*! Receives an object message and starts the calculating routine
*/
/****************************************************************************/
void TMandelbrotCell::performObjectMessage(IObjectMessage* msg, unsigned long long timer)
{
  if (msg != NULL && m_pCommunication != NULL)
  {
    ZLOGGER.printfln(LOGMODE_INFO, "Message received...");
    IMandelbrotFrame* pFrame = NULL;
    if (msg->getObject((ISerializable**)&pFrame) == RET_NOERROR)
    {
      TString strSenderCell;
      TString strSenderPipe;
      msg->getSenderCellName(&strSenderCell);
      msg->getSenderPipeName(&strSenderPipe);
      
      if (calcData(pFrame) == RET_NOERROR)
      {
        msg->setTargetCellName(&strSenderCell);
        msg->setTargetPipeName(&strSenderPipe);
        long lMaxCount = 3;
        while(lMaxCount > 0 && m_pCommunication->sendMessage(msg, 0) != RET_NOERROR)
        {
          lMaxCount--;
        }  
      }
      pFrame->release();
    }
  }
}


/****************************************************************************/
/*! Calculates all requested values and adds the result to the frame
*/
/****************************************************************************/
unsigned long TMandelbrotCell::calcData(IMandelbrotFrame* frame)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  long lX1 = frame->getX1();
  long lY1 = frame->getY1();
  long lX2 = frame->getX2();
  long lY2 = frame->getY2();
  long lIt = frame->getIterations();
  long lRange = frame->getMaxRange();
  double dScale = frame->getScale();
  double dCenterX = frame->getCenterX();
  double dCenterY = frame->getCenterY();
  IList<double>* pList = NULL;
  frame->getData(&pList);
  for(int i = lY1; i < lY2; i++)
  {
    for(int j = lX1; j < lX2; j++)
    {
/*      pList->add(getValue(j/dScale - dCenterX, 
                      i/dScale - dCenterY, lIt, lRange));*/
      pList->add(getValue(j/dScale - dCenterX, 
                      i/dScale - dCenterY, lIt, lRange));    
    }
  }
  
  if (pList->getCount() > 0)
  {
   ulRetval = RET_NOERROR;
  }
  return ulRetval;
}
    
/****************************************************************************/
/*! Returns one value of the mandelbrot set
*/
/****************************************************************************/
double TMandelbrotCell::getValue(double x, double y, long it, long range)
{
  long double dVal = 0;
  
  long double dZre = 0;
  long double dZim = 0;
  long double dZre_temp = 0;
  long double dSqareZre = 0;
  long double dSqareZim = 0;
    
  while(dVal <= range && it > 0)
  {
    //Calculate new z
    // f(z) = z^2 + u
    dZre_temp = dZre;
    dZre = dSqareZre - dSqareZim + x;
    dZim = 2*dZre_temp*dZim + y;
    
    dSqareZre = dZre*dZre;
    dSqareZim = dZim*dZim;
    dVal = fabsl(sqrtl(dSqareZre + dSqareZim));
    it--;
  }
  return it;
}

/****************************************************************************/
/*! Test routings
*/
/****************************************************************************/
double TMandelbrotCell::getValueTest(double x, double y, long it, long range)
{
  long double dVal = 0;
  
  long double dZre = 0;
  long double dZim = 0;
  long double dZre_temp = 0;
  long double dSqareZre = 0;
  long double dSqareZim = 0;
  long double dCubicZre = 0;
  long double dCubicZim = 0;
    
  while(dVal <= range && it > 0)
  {
    //Calculate new z
    // f(z) = z^3 + u
    dZre_temp = dZre;
    dZre = dCubicZre -  3*dZre*dSqareZim + x;
    dZim = -dCubicZim + 3*dZim*dSqareZre + y;
    
    dSqareZre = dZre*dZre;
    dSqareZim = dZim*dZim;
    
    dCubicZre = dZre*dSqareZre;
    dCubicZim = dZim*dSqareZim;
    dVal = fabsl(sqrtl(dSqareZre + dSqareZim));
    it--;
  }
  return it;
}


/****************************************************************************/
/*! See TAbstractCell
*/
/****************************************************************************/
void TMandelbrotCell::onInitialLife()
{
}

/****************************************************************************/
/*! See TAbstractCell
*/
/****************************************************************************/
void TMandelbrotCell::onEndLife()
{}

