/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FrameLoader                                                   *
 * Package : zeusbase                                                      *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:17:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: AbstractFrameLoader.cpp,v $
 *   Revision 1.4  2005/11/25 09:17:53  bhadorn
 *   Register RelayCell class
 *
 *   Revision 1.3  2005/11/16 11:25:54  bhadorn
 *   Removed cell registry from project
 *
 *   Revision 1.2  2005/11/10 11:03:37  bhadorn
 *   Checking of the property values. Registered a bunch of new classes.
 *
 *   Revision 1.1  2005/10/28 12:19:16  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/AbstractFrameLoader.h>
#include <zeusbase/System/CriticalSection.h>
#include <zeusbase/System/LibraryManager.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/SingletonManager.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/System/XRootObject.h>
#include <zeusbase/System/XLoaderObject.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/Security/FingerPrint.h>
#include <zeusbase/Security/SecurityManager.h>
#include <zeusbase/Messaging/XMLMessage.h>
#include <zeusbase/Messaging/SystemMessage.h>
#include <zeusbase/Messaging/BinaryMessage.h>
#include <zeusbase/Messaging/ObjectMessage.h>
#include <zeusbase/MOM/Module.h>
#include <zeusbase/MOM/SystemManager.h>
#include <zeusbase/Remote/RemoteMethodCall.h>
#include <zeusbase/Remote/RemoteReference.h>
#include <zeusbase/Remote/Naming.h>
#include <zeusbase/CellModel/CommPipe_Stub.h>
#include <zeusbase/CellModel/LocalPipeRegistry.h>
#include <zeusbase/CellModel/CellEnvironment.h>
#include <zeusbase/CellModel/CellSurvey.h>
#include <zeusbase/CellModel/Cell.h>
#include <zeusbase/CellModel/RelayCell.h>
 
 
USING_NAMESPACE_Zeus 
 
/***************************************************************************/
/*! Creates a FrameLoader 
*/
/***************************************************************************/
TAbstractFrameLoader::TAbstractFrameLoader(TString base_name, int /*argc*/, char *argv[]) : TZObject()
{
  m_bStartedUp = false;
  m_bInitialized = false;
  m_strBaseName = base_name;
  m_pMainDir = NULL;
  m_pRootObject = NULL;
  m_pSystemSettings = NULL;
  TFile ModuleFile(argv[0]);
  m_strPropertyFileName = ModuleFile.extractFileNameBase() + L".properties";
  m_strConfigFileName = ModuleFile.extractFileNameBase() + L".xml";

  if (TDirectory::getCurrentDirectory(&m_pMainDir) == RET_NOERROR)
  {
    ZLOGGER.printfln(LOGMODE_INFO, "Working Path: %s", m_pMainDir->getAbsoluteName().c_str());
    
    m_strPropertyFileName = m_pMainDir->getAbsoluteName() + m_strPropertyFileName;
    m_strConfigFileName = m_pMainDir->getAbsoluteName() + m_strConfigFileName;
  }  
  
  //Register all classes
  registerClasses();
}

/***************************************************************************/
/*! Destroys the FrameLoader 
*/
/***************************************************************************/
TAbstractFrameLoader::~TAbstractFrameLoader()
{
  shutdown();
    
  if (m_pMainDir != NULL)
  {
    m_pMainDir->release();
    m_pMainDir = NULL;
  }
  
  if (m_pSystemSettings != NULL)
  {
    m_pSystemSettings->release();
    m_pSystemSettings = NULL;
  }
  
  //Remove logger from the singleton manager
  TString strSingName = L"ILogger";
  TSingletonManager::getInstance().removeSingleton(&strSingName);
  
  TLoggerManager::getInstance().uninit();
  TLibraryManager::getInstance().removeAll();  
  TSingletonManager::getInstance().removeAll(); /*MUST BE AT THE END*/
}


/***************************************************************************/
/*! Initializes the frameloader
*/
/***************************************************************************/
void TAbstractFrameLoader::initialize()
{
  if (!m_bInitialized)
  {
    TTime::initRandom();
    if (initProperties() != RET_NOERROR)
    {
      ZLOGGER.printfln(LOGMODE_ERROR, "Initialisation of properties failed.");
    }
    if (initSecurity() != RET_NOERROR)
    {
      ZLOGGER.printfln(LOGMODE_ERROR, "Initialisation of security failed.");
    }
    if (initSystemLibraries() != RET_NOERROR)
    {
      ZLOGGER.printfln(LOGMODE_ERROR, "Initialisation of system libraries failed.");
    }
    
    m_bInitialized = true;
  }
}


/***************************************************************************/
/*! Initialises the security of the framework. This method can be 
    overwritten, so that customized code may be added for security 
    extendsions. The security data base can be set in the property file
    using the [BASE].SecurityManager.FileName=[securityfilename] property.
    If none is set the file [./security.db] is the standard value.
*/
/***************************************************************************/
unsigned long TAbstractFrameLoader::initSecurity()
{
  TString strPropName = m_strBaseName + L".SecurityManager.FileName";
  TString strSecurityDB;
  
  if (m_pSystemSettings->getPropertyValue(&strPropName, &strSecurityDB) != RET_NOERROR || strSecurityDB == L"")
  {
    strSecurityDB = L"./security.db";
  }
  
  //Initialize the security manager
  TSecurityManager::getInstance().loadRegistry(strSecurityDB);
  return RET_NOERROR;
}

/***************************************************************************/
/*! Load the property file 
    \return Error code of TPropertyFile::load()
*/
/***************************************************************************/
unsigned long TAbstractFrameLoader::initProperties()
{
  m_pSystemSettings = new TPropertyFile(m_strPropertyFileName);
  return m_pSystemSettings->load();
}

/***************************************************************************/
/*! Initializes the System Libraries.
    \retval RET_NOERROR : System libs initialized
    \retval RET_REQUEST_FAILED : No system libs initialized
*/
/***************************************************************************/
unsigned long TAbstractFrameLoader::initSystemLibraries()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  TLibraryManager& LibManager = TLibraryManager::getInstance();
  
  if (m_pSystemSettings != NULL)
  {
    //---------------------------------------------------------------------------
    //Read the LibraryManager settings    
    //List of code modules to load
    TString strPropName = m_strBaseName + L".LibraryManager";
    TString strValue;
    m_pSystemSettings->getPropertyValue(&strPropName, &strValue);
    
    
    long lCount = strValue.getSize();
    long lStartIndex = 0;
    long lEndIndex = 0;
    for(int i = 0; i < lCount; i++)
    {
      if (strValue.getChar(i) == L',' || i == lCount- 1)
      {
        if (i == lCount - 1)
        {
          lEndIndex = i;
        }
        else
        {
          lEndIndex = i-1;
        }
        
        //Get substring and load module
        TString strName = strValue.getSubString(lStartIndex, lEndIndex);
        if (strName != L"")
        {
          TString strPropLib = strPropName + TString(L".") + strName;
          TString strFileName;
          if (m_pSystemSettings->getPropertyValue(&strPropLib, &strFileName) == RET_NOERROR)
          {
            LibManager.addCodeModule(strName, strFileName);
          }
          else
          {
            ZLOGGER.printfln(LOGMODE_WARN, "Property [%s] not configured.", strPropLib.c_str());
          }
        }
        lStartIndex = i+1;
      }
    }
    
    //---------------------------------------------------------------------------
    //Initialize logger
    strValue = L"";
    strPropName = m_strBaseName + L".LoggerManager";
    if (m_pSystemSettings->getPropertyValue(&strPropName, &strValue) == RET_NOERROR)
    {
      TLoggerManager::getInstance().initLogger(strValue);
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_WARN, "Property [%s] not configured.", strPropName.c_str());
    }
    
    //External logger availabe
    if (ZLOGGER != NULL)
    {
      TString strSingName = L"ILogger";
      TSingletonManager::getInstance().removeSingleton(&strSingName);
      TSingletonManager::getInstance().addSingleton(&strSingName, ZLOGGER);
      strValue = L""; 
      strPropName = m_strBaseName + L".LoggerManager.ConfigFile";
      if (m_pSystemSettings->getPropertyValue(&strPropName, &strValue) == RET_NOERROR)
      {
        if (strValue == L"this")
        {
          ZLOGGER->loadProperties(m_pSystemSettings);
        }
        else
        {
          ZLOGGER->loadConfiguration(&strValue);
        }
      }
    }
    
    //---------------------------------------------------------------------------
    //Set security settings for XObjectFactory
    TString strSecurityPropName = m_strBaseName + L".LibraryManager.Security";
    TString strUseSecurity;
    m_pSystemSettings->getPropertyValue(&strSecurityPropName, &strUseSecurity);
    
    if (strUseSecurity == L"1")
    {
      LibManager.setSecurity(TLibraryManager::slUseSecurity);
    }
    else
    {
      LibManager.setSecurity(TLibraryManager::slNoSecurity);
    }
    
    
    ulRetval = RET_NOERROR;
  }
  
  return ulRetval;
}

/***************************************************************************/
/*! This releases the x root object
*/
/***************************************************************************/
void TAbstractFrameLoader::releaseXRootObject()
{
  if (m_pRootObject != NULL)
  {
    m_pRootObject->lockTree();
    m_pRootObject->freeze();
    m_pRootObject->unlockTree();
    m_pRootObject->release();
    m_pRootObject = NULL;
  }
}


/***************************************************************************/
/*! Creates a XObject tree out of a file
*/
/***************************************************************************/
unsigned long TAbstractFrameLoader::buildXObjectTree(TString filename /*= L""*/)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  TFile TreeFile(filename);
  if (TreeFile.exists())
  {
    if (TXObjectFactory::getInstance().createObjectFromFile(&filename, &m_pRootObject, false, false) == RET_NOERROR)
    {
      //Set the tree lock
      TCriticalSection* pTreeLock = new TCriticalSection(TCriticalSection::ltRecursive);
      m_pRootObject->setTreeLock(pTreeLock);
      pTreeLock->release();
      
      m_pRootObject->lockTree();
      m_pRootObject->unfreeze();
      m_pRootObject->unlockTree();
      
      ulRetval = RET_NOERROR;
    }
  }
  return ulRetval;
}


/***************************************************************************/
/*! Returns the root object of the framework
    \param root : return parameter of the Root 
    \retval RET_NOERROR : Root returned
    \retval RET_INVALID_PARAMETER : Invalid parameter received
    \retval RET_REQUEST_FAILED : No root available
*/
/***************************************************************************/
unsigned long TAbstractFrameLoader::getRootObject(IXObject** root)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (root == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    if (m_pRootObject != NULL)
    {
      *root = m_pRootObject;
      m_pRootObject->addRef();
      ulRetval = RET_NOERROR;
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! Checks if the framewaork should be started or not
*/
/***************************************************************************/
bool TAbstractFrameLoader::canStartup()
{
  return true;
}


/***************************************************************************/
/*! Starts the Frame
*/
/***************************************************************************/
unsigned long TAbstractFrameLoader::startup()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  if (!m_bInitialized)
  {
    ZLOGGER.printfln(LOGMODE_ERROR, "Frameloader not initialized.");   
  }
  else if(!m_bStartedUp)
  {    
    ulRetval = buildXObjectTree(m_strConfigFileName);
    m_bStartedUp = true;
  }
  return ulRetval;
}

/***************************************************************************/
/*! Closes the Frame
*/
/***************************************************************************/
unsigned long TAbstractFrameLoader::shutdown()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  releaseXRootObject();
    
  if (m_bStartedUp)
  {
    ulRetval = RET_NOERROR;
    m_bStartedUp = false;
  }
  return ulRetval;
}

/***************************************************************************/
/*! Register all classes of the Framework
*/
/***************************************************************************/
void TAbstractFrameLoader::registerClasses()
{
  ///////////////////////////////////////////////////
  XOBJECTFACTORY_REGISTER_ROOT(TXObjectFactory);
  XOBJECTFACTORY_REGISTER_ROOT(TSystemManager);
  XOBJECTFACTORY_REGISTER_ROOT(TXMLMessage);
  
  XOBJECTFACTORY_REGISTER_SUB(TXObjectFactory);
  XOBJECTFACTORY_REGISTER_SUB(TXLoaderObject);
  XOBJECTFACTORY_REGISTER_SUB(TModule);
  XOBJECTFACTORY_REGISTER_SUB(TCellEnvironment);
  XOBJECTFACTORY_REGISTER_SUB(TCellSurvey);
  XOBJECTFACTORY_REGISTER_SUB(TCell);
  XOBJECTFACTORY_REGISTER_SUB(TRelayCell);
  XOBJECTFACTORY_REGISTER_SUB(TLocalPipeRegistry);
  ///////////////////////////////////////////////////

  OBJECTFACTORY_REGISTER_CLASS(TBinaryMessage);
  OBJECTFACTORY_REGISTER_CLASS(TXMLMessage);
  OBJECTFACTORY_REGISTER_CLASS(TSystemMessage);
  OBJECTFACTORY_REGISTER_CLASS(TObjectMessage);
  OBJECTFACTORY_REGISTER_CLASS(TFingerPrint);
  OBJECTFACTORY_REGISTER_CLASS(TRemoteMethodCall);
  OBJECTFACTORY_REGISTER_CLASS(TRemoteReference);
  OBJECTFACTORY_REGISTER_CLASS(TZVariant);
  
  OBJECTFACTORY_REGISTER_CLASS(TNaming_Stub);
  OBJECTFACTORY_REGISTER_CLASS(TCommPipe_Stub);
  
  TSingletonManager::getInstance().registerCommonSingletons();
}
