/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Cell                                                          *
 * Package : Cell Computing                                                *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:16:13 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Cell.cpp,v $
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/CellModel/Cell.h>
#include <zeusbase/CellModel/Interfaces/ICellKernel.hpp>
#include <zeusbase/System/LoggerManager.h>
    
USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a new Cell
*/
/***************************************************************************/
TCell::TCell(IXMLNode* node) : TXObject(node)
{
  m_pCommunication = NULL;
  m_pCellKernel = NULL;
  m_dLifePower = 100.0;
  m_dAddingPower = 1.0;
  m_dRemovingPower = 1.0;
  m_bMessageEaten = false;
  m_bPing = false;
}
    
/***************************************************************************/
/*! Destroys the cell
*/
/***************************************************************************/
TCell::~TCell()
{
  releaseInstances();
}

/***************************************************************************/
/*! Release all instances
*/
/***************************************************************************/
void TCell::releaseInstances()
{
  if (m_pCommunication != NULL)
  {
    m_pCommunication->release();
    m_pCommunication = NULL;
  }
  if (m_pCellKernel != NULL)
  {
    m_pCellKernel->release();
    m_pCellKernel = NULL;
  }
}

/***************************************************************************/
/*! See ICell
*/
/***************************************************************************/
double MQUALIFIER TCell::getLifePower()
{
  return m_dLifePower;
}


/***************************************************************************/
/*! See ICell
*/
/***************************************************************************/
void MQUALIFIER TCell::resetPing()
{
  m_bPing = false;
}


/***************************************************************************/
/*! See ICell
*/
/***************************************************************************/
bool MQUALIFIER TCell::getPingResponse()
{
  return m_bPing;
}


/***************************************************************************/
/*! See ICell
*/
/***************************************************************************/
void MQUALIFIER TCell::recalculateLife()
{
  if (m_bMessageEaten)
  {
    m_bMessageEaten = false;
    m_dLifePower +=  m_dAddingPower;
  }
  else
  {
    m_dLifePower -=  m_dRemovingPower;
  }
  
  
  //Check boundaries
  if (m_dLifePower > 100)
  {
    m_dLifePower = 100;
  }
  else if (m_dLifePower < 0)
  {
    m_dLifePower = 0;
  } 
}


/***************************************************************************/
/*! See ICell
*/
/***************************************************************************/
unsigned long MQUALIFIER TCell::clone(ICell** cell)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (cell == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    //////////////////////////////////
    this->lockTree();
    IXObject* pParent = NULL;
    IXObject* pChild = NULL;
    this->getParent(&pParent);
    
    IXMLNode* pNode = NULL;
    if (m_pNode->clone(true, &pNode) == RET_NOERROR)
    {
      ulRetval = TXObjectFactory::getInstance().addNewChildObjects(pNode, pParent, &pChild, true);
      pNode->release();
    }
        
    if (pChild != NULL)
    {
      ulRetval = pChild->askForInterface(INTERFACE_ICell, (IZUnknown**)cell);
      pChild->release();
      pChild = NULL;
    }
    
    if (pParent != NULL)
    {
      pParent->release();
    }
    this->unlockTree();
    //////////////////////////////////
  }
  return ulRetval;
}


/***************************************************************************/
/*! See ICell
*/
/***************************************************************************/
unsigned long MQUALIFIER TCell::kill()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  //////////////////////////////////
  this->lockTree();
  IXObject* pParent = NULL;
  if (this->getParent(&pParent) == RET_NOERROR)
  {
    freeze();
    pParent->removeChild(this);
    pParent->release();
    ulRetval = RET_NOERROR;
  }
  this->unlockTree();
  //////////////////////////////////
  
  return ulRetval;
}


/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
unsigned long MQUALIFIER TCell::getInputMessage(IZUnknown* receiver, 
                                         IMessage** msg, 
                                         unsigned long long* proctime)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pCommunication != NULL)
  {
    ulRetval = m_pCommunication->getInputMessage(receiver, msg, proctime);
    
    //On Systemmessages do not live from those messages
    if (ulRetval == RET_NOERROR && (*msg)->getInterfaceID() != INTERFACE_ISystemMessage)
    {
      m_bMessageEaten = true;
    }
  }
  m_bPing = true;
  return ulRetval;
}

/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
unsigned long MQUALIFIER TCell::sendMessage(IMessage* message, 
                                     unsigned long long inittime /*= 0*/)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pCommunication != NULL)
  {
    ulRetval = m_pCommunication->sendMessage(message, inittime);
  }
  m_bPing = true;
  return ulRetval;
}

/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
unsigned long MQUALIFIER TCell::sendControlMessage(IMessage* message, 
                                                        unsigned long long inittime /*= 0*/)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pCommunication != NULL)
  {
    ulRetval = m_pCommunication->sendControlMessage(message, inittime);
  }
  return ulRetval;
}


/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
long MQUALIFIER TCell::getJobCount()
{
  long lRetval = 0;
  if (m_pCommunication != NULL)
  {
    lRetval = m_pCommunication->getJobCount();
  }
  return lRetval;
}

/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
void MQUALIFIER TCell::flushAllPipes()
{
  if (m_pCommunication != NULL)
  {
    m_pCommunication->flushAllPipes();
  }
}
    
/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
void MQUALIFIER TCell::updatePipes()
{
  if (m_pCommunication != NULL)
  {
    m_pCommunication->updatePipes();
  }
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TCell::freeze()
{
  if (m_pCellKernel != NULL)
  {
    ZLOGGER.printfln(LOGMODE_INFO, "Stop cell life...");
    m_pCellKernel->uninitialize();
    ZLOGGER.printfln(LOGMODE_INFO, "...done");
  }
  
  bool bRetval = TXObject::freeze();
  if (bRetval)
  {
    releaseInstances();
  }
  return bRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TCell::unfreeze()
{
  IXObject* pParentObject = NULL;
  if (isFrozen() && this->getParent(&pParentObject) == RET_NOERROR)
  {
    if (pParentObject->askForInterface(INTERFACE_ICellComm, (IZUnknown**)&m_pCommunication) != RET_NOERROR)
    {
      TString strName;
      this->getName(&strName);
      ZLOGGER.printfln(LOGMODE_ERROR, "Cell [Name=%s], Could not get the communication layer.", strName.c_str());  
    }
    pParentObject->release();
    
    IXObject* pChild = NULL;
    TString strObjName = L"Kernel";
    if (this->getChildByName(&strObjName, &pChild) == RET_NOERROR)
    {
      if (pChild->askForInterface(INTERFACE_ICellKernel, (IZUnknown**)&m_pCellKernel) == RET_NOERROR)
      {
        ZLOGGER.printfln(LOGMODE_INFO, "Start cell life...");
        m_pCellKernel->initialize((ICellComm*)this);
        ZLOGGER.printfln(LOGMODE_INFO, "...done.");
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_ERROR, "Cell Kernel does not export interface [INTERFACE_ICellKernel].");
      }
      pChild->release();
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_ERROR, "No Cell Kernel found [Name='Kernel'].");
    }
  }
  return TXObject::unfreeze();
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TCell);
  INTERFACE_CAST(ICellComm, INTERFACE_ICellComm);
  INTERFACE_CAST(ICell, INTERFACE_ICell);
MEMORY_MANAGER_IMPL_PARENT_END(TXObject);

