/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CellEnvironment                                               *
 * Package : Cell Computing                                                *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/10/28 12:16:13 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CellEnvironment.cpp,v $
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <zeusbase/CellModel/Interfaces/ICellSurvey.hpp>
#include <zeusbase/CellModel/Interfaces/ILocalPipeRegistry.hpp>
#include <zeusbase/CellModel/CellEnvironment.h>
#include <zeusbase/CellModel/OutputPipeManager.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/Time.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a cell environment
*/
/***************************************************************************/
TCellEnvironment::TCellEnvironment(IXMLNode* node) : TModule(node)
{
  m_pRegistry = NULL;
  m_pCellSurvey = NULL;
  m_pOutputPipes = new TOutputPipeManager();
  m_pInputPipeLock = new TCriticalSection();
}


/***************************************************************************/
/*! Destroys the cell environment
*/
/***************************************************************************/
TCellEnvironment::~TCellEnvironment()
{
  m_pOutputPipes->release();
  m_pOutputPipes = NULL;
  
  m_pInputPipeLock->release();
}


/***************************************************************************/
/*! Remove all instances
*/
/***************************************************************************/
void TCellEnvironment::releaseAllInstances()
{
  if(m_pCellSurvey != NULL)
  {
    m_pCellSurvey->uninitialize();
    m_pCellSurvey->unregisterAllCells();
    m_pCellSurvey->release();
    m_pCellSurvey = NULL;
  }
  
  //--------------------------------------------------------------------
  // Removes all input pipes
  
  //////////////////////////
  //INPUT PIPE LOCK
  m_pInputPipeLock->enter();
  while(m_lstInputPipes.getCount() > 0)
  {
    IOwnCommPipe* pPipe = m_lstInputPipes[0];
    pPipe->lock();
    
    if (m_pRegistry != NULL)
    {
      m_pRegistry->unregisterPipe(pPipe);
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_INFO, "Unable to unregister pipe. [no registry]");
    }
    pPipe->release();
    m_lstInputPipes.deleteItem(0);
  }
  m_pInputPipeLock->leave();
  //////////////////////////
  
  ZLOGGER.printfln(LOGMODE_INFO, "Input pipes removed");
    
  //--------------------------------------------------------------------
  // Remove Registry
  if(m_pRegistry != NULL)
  {
    m_pRegistry->release();
    m_pRegistry = NULL;
  }
}

/******************************************************************************/
/*! unlock all input pipes. This method is called at the beginning, so no
    other thread accesses the list.
*/
/******************************************************************************/
void TCellEnvironment::createInputPipes()
{
  ZLOGGER.printfln(LOGMODE_INFO, "Try to create a pipes...");  
  ZLOGGER.indent();
  if (m_pRegistry != NULL)
  {
    TString strDebugOut = L"Pipe created: ";
    
    //Create standard in pipe.
    TString strPipeName;
    TString strModulName;
    this->getName(&strModulName);
  
    //High priority pipe for Cell Survey
    IOwnCommPipe* pControlPipe = NULL;
    strPipeName = PIPENAME_CONTROL;
    if (m_pRegistry->createPipe(&strModulName, &strPipeName, PIPEPRIORITY_HIGH, false, &pControlPipe) == RET_NOERROR)
    {
      TString strOut = strDebugOut + strPipeName;
      ZLOGGER.printfln(LOGMODE_INFO, strOut.c_str());  
      //m_pRegistry->registerPipe(pControlPipe);
      m_lstInputPipes.add(pControlPipe);
    }

    //Regular pipe for work messages
    IOwnCommPipe* pStdPipe = NULL;
    strPipeName = PIPENAME_STDIN;
    if (m_pRegistry != NULL && m_pRegistry->createPipe(&strModulName, &strPipeName, PIPEPRIORITY_NORMAL,true, &pStdPipe) == RET_NOERROR)
    {
      TString strOut = strDebugOut + strPipeName;
      ZLOGGER.printfln(LOGMODE_INFO, strOut.c_str());
      m_pRegistry->registerPipe(pStdPipe);
      m_lstInputPipes.add(pStdPipe);
    }
  
    //Creates pipe out of the XML Node
    if (m_pNode != NULL)
    {
      TString strNodeName = L"InputPipe";
      m_pNode->resetNodeIterator(&strNodeName);
      IXMLNode* pPipeNode = NULL;
      while(m_pNode->getNextNode(&pPipeNode) == RET_NOERROR)
      {
        TString strAttPipeName;
        TString strAttPipePrio;
        TString strAttRemote;
        TString strAttName = L"Name";
        pPipeNode->getAttributeValue(&strAttName, &strAttPipeName);
        strAttName = L"Priority";
        pPipeNode->getAttributeValue(&strAttName, &strAttPipePrio);
        strAttName = L"Remote";
        pPipeNode->getAttributeValue(&strAttName, &strAttRemote);
        
        IOwnCommPipe* pXPipe = NULL;
        if (m_pRegistry->createPipe(&strModulName, 
                                    &strAttPipeName, 
                                    strAttPipePrio.toLong(PIPEPRIORITY_NORMAL),
                                    strAttRemote.toLong(0), 
                                    &pXPipe) == RET_NOERROR)
        {
          TString strOut = strDebugOut + strAttPipeName;
          ZLOGGER.printfln(LOGMODE_INFO, strOut.c_str());  
          m_pRegistry->registerPipe(pXPipe);
          m_lstInputPipes.add(pXPipe);
        }      
        pPipeNode->release();
      }
    }
    
    ZLOGGER.unindent();
    ZLOGGER.printfln(LOGMODE_INFO, "...done.");  
 
    
    //---------------------------------------------------------------------- 
    //Unlock all input pipes
    ZLOGGER.printf(LOGMODE_INFO, "Unlock all input pipes...");  
    //////////////////////////
    //INPUT PIPE LOCK
    m_pInputPipeLock->enter();
    IListIterator<IOwnCommPipe*>* pIterator = m_lstInputPipes.getIterator();
    IOwnCommPipe* pPipe = pIterator->getNextItem();
    while(pPipe != NULL)
    {
      pPipe->unlock();
      pPipe = pIterator->getNextItem();
    }
    m_lstInputPipes.releaseIterator(pIterator);
    m_pInputPipeLock->leave();
    //////////////////////////
    ZLOGGER.println(LOGMODE_INFO, "...done.");  
  
  }
  else
  {
    ZLOGGER.printfln(LOGMODE_INFO, "Unable to register pipe. [no registry]");  
  }
}


/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TCellEnvironment::freeze()
{
  bool bRetval = TModule::freeze();
  if (bRetval)
  {
    releaseAllInstances();
  }
  return bRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TCellEnvironment::unfreeze()
{
  if (isFrozen())
  {
  
    //-----------------------------------------------------------------------
    // Try to get the local pipe registry of this process
    ZLOGGER.printfln(LOGMODE_INFO, "Get local pipe registry...");
    ZLOGGER.indent();
    IXObject* pRegObj = NULL;
    TString strPath = L"/LocalPipeRegistry";
    if (this->getObject(&strPath, &pRegObj) == RET_NOERROR)
    {
      if (pRegObj->askForInterface(INTERFACE_ILocalPipeRegistry, (IZUnknown**)&m_pRegistry) == RET_NOERROR)
      {
        m_pOutputPipes->setRegistry(m_pRegistry);
        ZLOGGER.unindent();
        ZLOGGER.printfln(LOGMODE_INFO, "...done.");
      }
      else
      {
        ZLOGGER.unindent();
        ZLOGGER.printfln(LOGMODE_INFO, "...failed.");
      }
      pRegObj->release();
    }
    else
    {
      ZLOGGER.unindent();
      ZLOGGER.printfln(LOGMODE_INFO, "...failed.");
    } 
    
    
    //-----------------------------------------------------------------------
    //Create input pipes out of the XML
    createInputPipes();
    
    
    //-----------------------------------------------------------------------
    //Register all static configured cells
    ZLOGGER.printf(LOGMODE_INFO, "Get survey cell...");
    IXObject* pSurveyObj = NULL;
    TString strSurveyName = L"Survey";
    if (this->getChildByName(&strSurveyName, &pSurveyObj) == RET_NOERROR)
    {
      pSurveyObj->askForInterface(INTERFACE_ICellSurvey, (IZUnknown**)&m_pCellSurvey);
      pSurveyObj->release();
      ZLOGGER.printfln(LOGMODE_INFO, "...done.");
    }
    
    ZLOGGER.printfln(LOGMODE_INFO, "Register all cells at the Cellsurvey...");
    if (m_pCellSurvey != NULL)
    {
      long lCellCount = this->getChildCount();
      for(int i = 0; i < lCellCount; i++)
      {
        IXObject* pChild = NULL;
        if (this->getChild(i, &pChild) == RET_NOERROR)
        {
          tryToRegisterCell(pChild);
          pChild->release();
        }
      }
      ZLOGGER.printfln(LOGMODE_INFO, "...done.");
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_INFO, "...failed.");
    }
  }
  
  /////////////////////////////////////
  //IMPORTANT: Children have to be unfrozen after the environment has been initialized
  bool bRetval = TModule::unfreeze();
  /////////////////////////////////////
  
  if (m_pCellSurvey != NULL)
  {
    ZLOGGER.printfln(LOGMODE_INFO, "Add Control pipe...");
    ICommPipe* pPipe = NULL;
    if (m_pCellSurvey->getInputPipe(&pPipe) == RET_NOERROR)
    {
      m_pOutputPipes->addPipe(pPipe);
      ZLOGGER.printfln(LOGMODE_INFO, "..done");
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_INFO, "...failed");
    }
    ZLOGGER.printfln(LOGMODE_INFO, "Start Control cell...");
    m_pCellSurvey->initialize((ICellComm*)this);
    ZLOGGER.printfln(LOGMODE_INFO, "...done.");
  }
  return bRetval; 
}

/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
unsigned long MQUALIFIER TCellEnvironment::getInputMessage(IZUnknown* /*receiver*/, IMessage** msg, unsigned long long* proctime)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  static unsigned long long ulCounter = 0;

  if (msg == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    ulCounter = TTime::getTickCount();
    bool bFound = false;
    int i = 0;
    
    //////////////////////////
    //INPUT PIPE LOCK
    m_pInputPipeLock->enter();
    IListIterator<IOwnCommPipe*>* pIterator = m_lstInputPipes.getIterator();
    IOwnCommPipe* pPipe = pIterator->getNextItem();
    while(pPipe != NULL && !bFound)
    {
      if (i == 0 && ulCounter%100 < 85 && !pPipe->isEmpty())
      {
        ulRetval = pPipe->getMessage(msg,proctime);
        bFound = true;
      }
      else if (i != 0 && !pPipe->isEmpty())
      {
        ulRetval = pPipe->getMessage(msg,proctime);
        bFound = true;
      }
      else
      {
        pPipe = pIterator->getNextItem();
      }
      i++;
    }
    m_lstInputPipes.releaseIterator(pIterator);
    m_pInputPipeLock->leave();
    //////////////////////////
    
    if (bFound)
    {
      ulRetval = RET_NOERROR;
    }
  }
  
  return ulRetval;
}

/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
unsigned long MQUALIFIER TCellEnvironment::sendMessage(IMessage* message, unsigned long long inittime /*= 0*/)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    ICommPipe* pPipe = NULL;
    TString strTargetCellName;
    TString strTargetPipeName;
    TString strSenderCellName;
    this->getName(&strSenderCellName);

    message->getTargetCellName(&strTargetCellName);
    message->getTargetPipeName(&strTargetPipeName);
    message->setSenderCellName(&strSenderCellName);
        
    if (!isFrozen() && m_pOutputPipes->findPipe(&strTargetCellName, &strTargetPipeName, &pPipe) == RET_NOERROR)
    {
      ulRetval = pPipe->sendMessage(message, inittime);
      pPipe->release();
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! See ICellControl
*/
/***************************************************************************/
unsigned long MQUALIFIER TCellEnvironment::sendControlMessage(IMessage* message, unsigned long long inittime /*= 0*/)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strPipeName = PIPENAME_CONTROL;
    ZLOGGER.printf(LOGMODE_INFO, "Try to find pipe %s...", strPipeName.c_str());  
    //////////////////////////
    //INPUT PIPE LOCK
    m_pInputPipeLock->enter();
    IListIterator<IOwnCommPipe*>* pIterator = m_lstInputPipes.getIterator();
    IOwnCommPipe* pPipe = pIterator->getNextItem();
    bool bFound = false;
    while(pPipe != NULL && !bFound)
    {
      TString strName;
      pPipe->getPipeName(&strName);
      if (strName.equalsStr(&strPipeName))
      {
        bFound = true;
        ulRetval = pPipe->sendMessage(message, inittime);
      }
      else
      {
        pPipe = pIterator->getNextItem();
      }
    }
    m_lstInputPipes.releaseIterator(pIterator);
    m_pInputPipeLock->leave();
    //////////////////////////
    
    if (bFound)
    {
      ZLOGGER.println(LOGMODE_INFO, "...done.");  
    }
    else
    {
      ZLOGGER.println(LOGMODE_INFO, "...failed.");  
    }
  }
  return ulRetval;
}


/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
long MQUALIFIER TCellEnvironment::getJobCount()
{
  long lMessageCount = 0;
  
  ////////////////////////// 
  //INPUT PIPE LOCK
  m_pInputPipeLock->enter();
  IListIterator<IOwnCommPipe*>* pIterator = m_lstInputPipes.getIterator();
  IOwnCommPipe* pPipe = pIterator->getNextItem();
  while(pPipe != NULL)
  {
    lMessageCount += pPipe->getCount();
    pPipe = pIterator->getNextItem();
  }
  m_lstInputPipes.releaseIterator(pIterator);
  m_pInputPipeLock->leave();
  //////////////////////////
  
  return lMessageCount;
}


/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
void MQUALIFIER TCellEnvironment::flushAllPipes()
{
  ////////////////////////// 
  //INPUT PIPE LOCK
  m_pInputPipeLock->enter();
  IListIterator<IOwnCommPipe*>* pIterator = m_lstInputPipes.getIterator();
  IOwnCommPipe* pPipe = pIterator->getNextItem();
  while(pPipe != NULL)
  {
    pPipe->flush();
    pPipe = pIterator->getNextItem();
  }
  m_lstInputPipes.releaseIterator(pIterator);
  m_pInputPipeLock->leave();
  //////////////////////////
}


/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
void MQUALIFIER TCellEnvironment::updatePipes()
{
  m_pOutputPipes->refreshPipes();
}


/******************************************************************************/
/*! Registeres a cell object for the cell survey (multithreaded)
    \pre obj must be a valid pointer
*/
/******************************************************************************/
void TCellEnvironment::tryToRegisterCell(IXObject* obj)
{
  ICell* pCell = NULL;
  if (obj->askForInterface(INTERFACE_ICell, (IZUnknown**)&pCell) == RET_NOERROR)
  {
    ZLOGGER.indent();
    m_pCellSurvey->registerCell(pCell);
    ZLOGGER.unindent();
    pCell->release();
  }
}

/******************************************************************************/
/*! Unregisteres a cell object for the cell survey
    \pre obj must be a valid pointer
*/
/******************************************************************************/
void TCellEnvironment::tryToUnregisterCell(IXObject* obj)
{
  ICell* pCell = NULL;
  if (obj->askForInterface(INTERFACE_ICell, (IZUnknown**)&pCell) == RET_NOERROR)
  {
    ZLOGGER.indent();
    m_pCellSurvey->unregisterCell(pCell);
    ZLOGGER.unindent();
    pCell->release();
  }
}

/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
unsigned long MQUALIFIER TCellEnvironment::addChild(IXObject* child)
{
  unsigned long ulRetval = TModule::addChild(child);
  tryToRegisterCell(child);
  return ulRetval;
}

/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
unsigned long MQUALIFIER TCellEnvironment::deleteChild(long index)
{
  IXObject* pObj = NULL;
  if (this->getChild(index, &pObj) == RET_NOERROR)
  {
    tryToUnregisterCell(pObj);
    pObj->release();
  }
  return TModule::deleteChild(index);
}

/***************************************************************************/
/*! See ICellComm
*/
/***************************************************************************/
unsigned long MQUALIFIER TCellEnvironment::removeChild(IXObject* child)
{
  tryToUnregisterCell(child);
  return TModule::removeChild(child);
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TCellEnvironment);
  INTERFACE_CAST(ICellComm, INTERFACE_ICellComm);
MEMORY_MANAGER_IMPL_PARENT_END(TModule);

