/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CellSurvey                                                    *
 * Package : CellModel                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:36:05 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CellSurvey.cpp,v $
 *   Revision 1.2  2005/11/10 10:36:05  bhadorn
 *   Checking of updatetime. Using absolute difference.
 *
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

 
#include <zeusbase/CellModel/CellSurvey.h>
#include <zeusbase/CellModel/CommPipe.h>
#include <zeusbase/CellModel/GrowthCalc.h>
#include <zeusbase/Messaging/SystemMessage.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/Stack.hpp>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/CriticalSection.h>
#include <zeusbase/System/Time.h>
#include <math.h>

/***************************************************************************/
/*! creates a cell survey
*/
/***************************************************************************/
TCellSurvey::TCellSurvey(IXMLNode* node)  : TAbstractCell(node)
{
  m_pCellLock = new TCriticalSection();
  m_lActCellListIndex = 0;
  
  TString cellname;
  m_dMaxSleepTime = 1.0;   //[sec]
  m_dMinSleepTime = 0.1;    //[sec]
  m_dSurveySleepTime = 1.0; //[sec]
  m_dUpdatePipeRate = 10.0;  //[sec]
  m_dTimeStamp = 0.0;
  m_dLastPipeUpdate = 0.0;

  unsigned long ulJobLimit = 20;
  unsigned long ulReactionLimit = 10;
  if(m_pNode != NULL)
  {
    TString strAttValue;
    TString strAttName = L"MaxSleep";
    m_pNode->getAttributeValue(&strAttName,&strAttValue);
    m_dMaxSleepTime = strAttValue.toDouble();

    strAttName = L"MinSleep";
    m_pNode->getAttributeValue(&strAttName,&strAttValue);
    m_dMinSleepTime = strAttValue.toDouble();

    strAttName = L"JobLimit";
    m_pNode->getAttributeValue(&strAttName,&strAttValue);
    ulJobLimit = strAttValue.toLong(ulJobLimit);
  
    strAttName = L"ReactionLimit";
    m_pNode->getAttributeValue(&strAttName,&strAttValue);
    ulReactionLimit = strAttValue.toLong(ulReactionLimit);
    
    strAttName = L"PipeRefresh";
    m_pNode->getAttributeValue(&strAttName,&strAttValue);
    m_dUpdatePipeRate = strAttValue.toDouble(m_dUpdatePipeRate);
  }


  m_pCalc = new TGrowthCalc();
  m_pCalc->setBoundary(ulJobLimit);
  m_pCalc->setReactionLimit(ulReactionLimit);
  
  m_pInputPipe = NULL;
  
  m_dSurveySleepTime = m_dMinSleepTime; //[sec]
  m_bPingSent = false;
}

/***************************************************************************/
/*! destroys the cell survey
*/
/***************************************************************************/
TCellSurvey::~TCellSurvey()
{
  releaseInstances();
  m_pCalc->release();
  m_pCellLock->release();
  
  ZLOGGER.printfln(LOGMODE_INFO, "Survey killed.");
}


/***************************************************************************/
/*! Remove all instances
*/
/***************************************************************************/
void TCellSurvey::releaseInstances()
{
  unregisterAllCells();
  if (m_pInputPipe != NULL)
  {
    m_pInputPipe->lock();
    m_pInputPipe->release();
    m_pInputPipe = NULL;
  }
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TCellSurvey::freeze()
{
  bool bRetval = TAbstractCell::freeze();
  if (bRetval)
  {
    releaseInstances();
  }
  return bRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TCellSurvey::unfreeze()
{
  bool bRetval = TAbstractCell::unfreeze();
  if (bRetval)
  {
    IXObject* pParent = NULL;
    if (this->getParent(&pParent) == RET_NOERROR)
    {
      pParent->getName(&m_strCellName);
      m_pInputPipe = new TCommPipe(PIPEPRIORITY_NORMAL,PIPENAME_SURVEY,m_strCellName);
      m_pInputPipe->unlock();
      
      pParent->release();
    }
  }
  return bRetval;
}


/***************************************************************************/
/*! checks and adjustes the sleeping time for its maximum
*/
/***************************************************************************/
void TCellSurvey::checkMaxSleepTime()
{
  ///Maximum reached
  if (m_dSurveySleepTime >= m_dMaxSleepTime)
  {
    //Try to recreate a cell
  }
  else
  {
    m_dSurveySleepTime *= 1.5;
    if (m_dSurveySleepTime > m_dMaxSleepTime)
    {
      m_dSurveySleepTime = m_dMaxSleepTime;
    }
    ZLOGGER.printfln(LOGMODE_INFO, "Survey: Sleep increased : %f",m_dSurveySleepTime);
  }
}

/***************************************************************************/
/*! checks and adjustes the sleeping time for its minimum
*/
/***************************************************************************/
void TCellSurvey::checkMinSleepTime()
{
  double dTimeDiff = (TTime::getTimeStamp() - m_dTimeStamp);
  //Sleeping time is too big
  if (m_dSurveySleepTime > m_dMinSleepTime && dTimeDiff < m_dSurveySleepTime/1.5)
  {
    
    m_dSurveySleepTime /= 1.5;
    if (m_dSurveySleepTime < m_dMinSleepTime)
    {
      m_dSurveySleepTime = m_dMinSleepTime;
    }
    ZLOGGER.printfln(LOGMODE_INFO, "Survey: Sleep decreased : %f", m_dSurveySleepTime);
  }
}

/***************************************************************************/
/*! Calculates the probability to remove the cell
*/
/***************************************************************************/
double TCellSurvey::getCellRemoveProb(double x)
{
  return 100*exp(-x/7.0);
}

/***************************************************************************/
/*! Checks if all cells are ok
*/
/***************************************************************************/
unsigned long TCellSurvey::controlAndKill()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  //At least one cell has to live
  TStack<ICell*> lstCellsToRemove;
  /////////////////////////
  // CELL LOCK
  m_pCellLock->enter();
  if (!this->isFrozen() && m_lstCells.getCount() > 1)
  {
    long i = 0;
    TString strOut = L";CellList={";
    while(i < m_lstCells.getCount())
    {
      ICell* pCell = m_lstCells[i];
      
      //-----------------------------------------------------------
      //Control the life power
      pCell->recalculateLife();
      double dLifePower = pCell->getLifePower();
      
                  
      //-----------------------------------------------------------
      //Calculate the probability to remove the cell
      double dP = this->getCellRemoveProb(dLifePower);
      double dR = TTime::random(1, 100);
      strOut = strOut + TString(L"Cell") + TString(i) + TString(L"[Power=") + TString((long)dLifePower) + 
               TString(L",p=") + TString((long)dP) + TString(L"],");
      
      if (dR < dP && m_lstCells.getCount() > 1)
      {
        lstCellsToRemove.push(pCell);
        m_lstCells.deleteItem(i); 
      }
      else
      {
        i++;
      }
    }
    
    strOut = strOut + TString(L"}\n");
    ZLOGGER.out(strOut.c_str());
    ulRetval = RET_NOERROR;
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  
  m_pCellLock->leave();
  /////////////////////////
  
  //------------------------------------------------------------------      
  //Remove all cells witch are deprecated.,
  while(!lstCellsToRemove.isEmpty())
  {
    ICell* pCell = lstCellsToRemove.pop();
    ZLOGGER.printfln(LOGMODE_INFO, "Stop cell life...");
    pCell->kill();
    ZLOGGER.printfln(LOGMODE_INFO, "...done"); 
    pCell->release();
    pCell = NULL;
  }
  //------------------------------------------------------------------      
  return ulRetval;
}


/***************************************************************************/
/*! See ICellKernel
*/
/***************************************************************************/
void TCellSurvey::sendLifeSignal()
{  
  //Sleeping before checking the response data
  TTime::sleep(m_dSurveySleepTime);
  
  if (m_pCommunication != NULL && m_pInputPipe != NULL)
  {
    //---------------------------------------------------------------------
    //A ping has been sent
    if (m_bPingSent)
    {
      IMessage* pInMessage = NULL;
      unsigned long long ulTimer = 0;
      
      //Receive the response message
      if (m_pInputPipe->getMessage(&pInMessage, &ulTimer) == RET_NOERROR)
      {
        dispatchMessage(pInMessage, ulTimer);
        pInMessage->release();
        
        checkMinSleepTime(); //Adjust the minimum sleeping time
        m_bPingSent = false;
      }
      //No message received. The sleeping time is propably no short or the cell is dead.
      else
      {
        checkMaxSleepTime(); //Adjust the maximum sleeping time
      }
    }
    
    //---------------------------------------------------------------------
    //Response has been received
    if (!m_bPingSent)
    {
      ZLOGGER.printfln(LOGMODE_INFO, "Survey: Send survey ping...");
      TSystemMessage* pMessage = NULL;
      
      //Send the message throu the high priority pipe "CONTROL"
      if (TSystemMessage::createMessage(L"MessageObject", m_strCellName, PIPENAME_CONTROL, &pMessage) == RET_NOERROR)
      {
        TString strSPipeName = PIPENAME_SURVEY;
        pMessage->setMessageType(MSG_PING_REQUEST);
        pMessage->setSenderPipeName(&strSPipeName);  //if not set it will take STDIN as default pipe
        pMessage->setSenderCellName(&m_strCellName);
        
        if (m_pCommunication->sendControlMessage(pMessage, 0) == RET_NOERROR)
        {
          ZLOGGER.printfln(LOGMODE_INFO, "...done.");
          m_bPingSent = true;
          m_dTimeStamp = TTime::getTimeStamp();
        }
        else
        {
          ZLOGGER.printfln(LOGMODE_INFO, "...failed.");
        }
        pMessage->release();
      }
    }
    
    //Check if the update of the pipes must be performed
    if (m_dUpdatePipeRate > 0 && abs(TTime::getTimeStamp() - m_dLastPipeUpdate) > m_dUpdatePipeRate)
    {
      m_pCommunication->updatePipes();
      m_dLastPipeUpdate = TTime::getTimeStamp();
    }
    else
    {
      //ZLOGGER.out("UpdateRate=%f; Time=%f\n", m_dUpdatePipeRate, TTime::getTimeStamp() - m_dLastPipeUpdate);
    }
  }
}

/***************************************************************************/
/*! See TAbstractCell
*/
/***************************************************************************/
void TCellSurvey::performSystemMessage(ISystemMessage* msg, unsigned long long timer)
{
  if (msg != NULL)
  {
    TString strSenderCellName;
    TString strSenderPipeName;
    msg->getSenderCellName(&strSenderCellName);
    msg->getSenderPipeName(&strSenderPipeName);
    
    switch(msg->getMessageType())
    {
      //-----------------------------------------------------------------------------
      //Response received
      case MSG_PING_RESPONSE :
      {
        /////////////////////////
        m_pCellLock->enter();
        long lJobCount = m_pCommunication->getJobCount();
        long lCellCount = m_lstCells.getCount();
        if (m_lActCellListIndex >= lCellCount-1)
        {
          m_lActCellListIndex = 0;
          if (lCellCount == 0)
          {
            m_lActCellListIndex = -1;
          }
        }
        else
        {
          m_lActCellListIndex++;
        }
        ICell* pCellToClone = m_lstCells[m_lActCellListIndex];
        pCellToClone->addRef();
        m_pCellLock->leave();
        /////////////////////////

                
        //Timer is in micro seconds
        double dTemp = timer/(double)1000000.0;
        double dTime = dTemp - m_dSurveySleepTime;
        if (dTime <= 0)
        {
          dTime = 0.000001;
        }
        
        TString strOut = TString(L"Survey: Response received. Time=") + TString(dTime) + TString(L"; Jobs=") + lJobCount;
        ZLOGGER.printfln(LOGMODE_DEBUG, strOut.c_str());
        
        
        
        m_pCalc->addData(dTime,lJobCount);
        
        //Check the grows boundaries
        if (m_pCalc->isExceeded())
        {
          ZLOGGER.indent();
          ZLOGGER.printfln(LOGMODE_INFO, "Survey: Max reached -> Clone cell...");
          ICell* pNewCell = NULL;
          if (pCellToClone->clone(&pNewCell) == RET_NOERROR)
          {
            pNewCell->release();
          //  m_lstCells.add(pNewCell);
            ZLOGGER.printfln(LOGMODE_INFO, "...done");
          }
          else
          {
            ZLOGGER.printfln(LOGMODE_INFO, "...failed");
          }
          ZLOGGER.unindent();
        }
        //Data print out
        strOut = TString(L"Time=") + TString(m_dSurveySleepTime)+
                         TString(L";Jobs=") + TString(lJobCount) + 
                         TString(L";Cells=") + TString(lCellCount);
        //ZLOGGER.out(strOut.c_str());
        //
        this->controlAndKill();
        //ZLOGGER.out("\n");
        
        pCellToClone->release();
      }
      break;
      
      default:
      ;
    }
  }
}

/***************************************************************************/
/*! See ICellSurvey
*/
/***************************************************************************/
void MQUALIFIER TCellSurvey::registerCell(ICell* cell)
{
  if (cell != NULL)
  {
    m_pCellLock->enter();
    cell->addRef();
    m_lstCells.add(cell);
    m_pCellLock->leave();
    ZLOGGER.printfln(LOGMODE_INFO, "Cell registered.");
  }
}

/***************************************************************************/
/*! See ICellSurvey
*/
/***************************************************************************/
void MQUALIFIER TCellSurvey::unregisterCell(ICell* cell)
{
  if (cell != NULL)
  {
    m_pCellLock->enter();
    long lIndex = m_lstCells.indexOf(cell); 
    if (lIndex >= 0)
    {
      m_lstCells[lIndex]->release();
      m_lstCells.deleteItem(lIndex);
      ZLOGGER.printfln(LOGMODE_INFO, "Cell unregistered.");
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_INFO, "Cell not unregistered [unknown cell].");
    }
    m_pCellLock->leave();
  }
}

/***************************************************************************/
/*! See ICellSurvey
*/
/***************************************************************************/
void MQUALIFIER TCellSurvey::unregisterAllCells()
{
  m_pCellLock->enter();
  while(!m_lstCells.isEmpty())
  {
    m_lstCells[0]->release();
    m_lstCells.deleteItem(0);
  }
  m_pCellLock->leave();
}

/***************************************************************************/
/*! See ICellSurvey
*/
/***************************************************************************/
unsigned long MQUALIFIER TCellSurvey::getInputPipe(ICommPipe** pipe)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (pipe == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pInputPipe != NULL)
  {
    *pipe = m_pInputPipe;
    m_pInputPipe->addRef();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See ICellKernel
*/
/***************************************************************************/
void MQUALIFIER TCellSurvey::initialize(ICellComm* cell_interface)
{
  TAbstractCell::initialize(cell_interface);
  m_dLastPipeUpdate = TTime::getTimeStamp();
}

/***************************************************************************/
/*! See ICellKernel
*/
/***************************************************************************/
void MQUALIFIER TCellSurvey::uninitialize()
{
  TAbstractCell::uninitialize();
}

/***************************************************************************/
/*! See ICellKernel
*/
/***************************************************************************/
bool MQUALIFIER TCellSurvey::isInitialized()
{
  return TAbstractCell::isInitialized();
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TCellSurvey);
  INTERFACE_CAST(ICellSurvey, INTERFACE_ICellSurvey);
MEMORY_MANAGER_IMPL_PARENT_END(TAbstractCell);

