/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CommPipe                                                      *
 * Package : CellModul                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:22:50 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CommPipe.cpp,v $
 *   Revision 1.2  2005/11/16 11:22:50  bhadorn
 *   Communication improved by using RMI for C++ and the Naming service
 *
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//---------------------------------------------------------------------------

#include <zeusbase/CellModel/CommPipe.h>
#include <zeusbase/CellModel/CommPipe_Skel.h>
#include <zeusbase/System/LoggerManager.h>

//---------------------------------------------------------------------------
//
//                __      __      __
//  HeadElement: |  | <- |  | <- |  | TailElement
//
//
//-----------------------------------------------------------------------------

/*****************************************************************************/
/*! Creates a new communication pipe
    \param name : Name of the pipe
*/
/*****************************************************************************/
TCommPipe::TCommPipe(TString name)
{
  m_ulPriority = PIPEPRIORITY_NORMAL;
  m_pLock = new TCriticalSection();
  m_bLockFlag= true;
  m_pHeadElement = NULL;
  m_pTailElement = NULL;
  m_strName = name;
  m_strCellName = L"";
  m_lSize = 0;
}


/*****************************************************************************/
/*! Erstellen der FIFO Pipe.
    \param priority : Prioritaet der FIFO
    \param name : Name der Pipe
    \param ownername : Name des Besitzers
*/
/*****************************************************************************/
TCommPipe::TCommPipe(unsigned long priority, TString name, TString ownername) : TAbstractRemoteObject()
{
  m_ulPriority = priority;
  m_pLock = new TCriticalSection();
  m_bLockFlag= true;
  m_pHeadElement = NULL;
  m_pTailElement = NULL;
  m_strName = name;
  m_strCellName = ownername;
  m_lSize = 0;
}
//-----------------------------------------------------------------------------


/*****************************************************************************/
/*! Zerstoeren der Pipe
*/
/*****************************************************************************/
TCommPipe::~TCommPipe()
{
  flush();
  
  m_pLock->release();
  m_pLock = NULL;
  TString strOut = TString(L"Pipe killed. ") + (long)this;
  ZLOGGER.printfln(LOGMODE_INFO, strOut.c_str());  
}
//-----------------------------------------------------------------------------


/*****************************************************************************/
/*! Checks if the remote access of a pipe is available
*/
/*****************************************************************************/
bool MQUALIFIER TCommPipe::hasRemoteAccess() const
{
  return (m_pSkeleton != NULL);
}

/*****************************************************************************/
/*! see ICommpipe
*/
/*****************************************************************************/
void MQUALIFIER TCommPipe::lock()
{
  m_bLockFlag = true;
}

//-----------------------------------------------------------------------------

/*****************************************************************************/
/*! see ICommpipe
*/
/*****************************************************************************/
void MQUALIFIER TCommPipe::unlock()
{
  m_bLockFlag = false;
}
//-----------------------------------------------------------------------------

/*****************************************************************************/
/*! see ICommPipe
*/
/*****************************************************************************/
bool MQUALIFIER TCommPipe::isLocked() const
{
  return m_bLockFlag;
}


/*****************************************************************************/
/*! Checks if the pipe is empty
    \retval true: Pipe is empty
    \retval false : Pipe is not empty
*/
/*****************************************************************************/
bool MQUALIFIER TCommPipe::isEmpty() const
{
  return (m_pHeadElement == NULL);
}
//-----------------------------------------------------------------------------

/*****************************************************************************/
/*! Flushes the pipe
*/
/*****************************************************************************/
void MQUALIFIER TCommPipe::flush()
{
  m_pLock->enter();
  TString strOut = TString(L"Flush pipe: Elements remained :") + m_lSize;
  ZLOGGER.printfln(LOGMODE_INFO, strOut.c_str());
  while (m_pTailElement != NULL)
  {
    TPipeContainer* pWasteElement = m_pTailElement;
    m_pTailElement = pWasteElement->getPrev();
    delete pWasteElement;
  }
  m_pHeadElement = NULL;

  m_lSize = 0;
  m_pLock->leave();
}
//-----------------------------------------------------------------------------


/*****************************************************************************/
/*! Returns the size of the pipe
    \return size
*/
/*****************************************************************************/
long MQUALIFIER TCommPipe::getCount() const
{ return m_lSize; }
//-----------------------------------------------------------------------------


/*****************************************************************************/
/*! Get the top message of the pipe, without removing it
    \param msg : Return parameter of the message
    \param proctime : Return value of the time delay
    \retval RET_NOERROR : No error occured returning the message
    \retval RET_REQUEST_FAILED : No Message in the queue
    \retval RET_INVALID_PARAMETER : Invalid parameter received (=NULL)
*/
/*****************************************************************************/
unsigned long MQUALIFIER TCommPipe::peekMessage(IMessage** msg, unsigned long long* proctime)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_bLockFlag)
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  else if (msg == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pTailElement == NULL)
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  else
  {
    m_pLock->enter();

    ulRetval = m_pTailElement->getData(msg);
    if (proctime != NULL) *proctime = m_pTailElement->getTime();
    m_pLock->leave();

  }
  return ulRetval;
}
//-----------------------------------------------------------------------------


/*****************************************************************************/
/*! Get the top message of the pipe, and removes it
    \param msg : Return parameter of the message
    \param proctime : Return value of the time delay
    \retval RET_NOERROR : No error occured returning the message
    \retval RET_REQUEST_FAILED : No Message in the queue
    \retval RET_INVALID_PARAMETER : Invalid parameter received (=NULL)
*/
/*****************************************************************************/
unsigned long MQUALIFIER TCommPipe::getMessage(IMessage** msg, unsigned long long* proctime)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;

  if (m_bLockFlag)
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  else if (msg == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pTailElement == NULL)
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  else
  {
    m_pLock->enter();
    TPipeContainer* pWasteElement = m_pTailElement;

    ulRetval = pWasteElement->getData(msg);
    m_pTailElement = pWasteElement->getPrev();

    if (proctime != NULL) *proctime = pWasteElement->getTime();
    delete pWasteElement;

    //Das letzte Element wurde entfernt
    if (m_pTailElement == NULL) m_pHeadElement = NULL;

    m_lSize--;
    m_pLock->leave();
  }

  return ulRetval;
}
//-----------------------------------------------------------------------------


/*****************************************************************************/
/*! See ICommPipe
*/
/*****************************************************************************/
unsigned long MQUALIFIER TCommPipe::getPipeName(IString* name) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    name->assignStr(&m_strName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/*****************************************************************************/
/*! See ICommPipe
*/
/*****************************************************************************/
unsigned long MQUALIFIER TCommPipe::getCellName(IString* name) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    name->assignStr(&m_strCellName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/*****************************************************************************/
/*! see ICommPipe
*/
/*****************************************************************************/
unsigned long MQUALIFIER TCommPipe::getPriority() const
{ return m_ulPriority; }
//-----------------------------------------------------------------------------


/*****************************************************************************/
/*! see ICommPipe
*/
/*****************************************************************************/
unsigned long MQUALIFIER TCommPipe::sendMessage(IMessage* msg, unsigned long long inittime)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (msg == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_bLockFlag)
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  else
  {
    m_pLock->enter();
    TPipeContainer* pNewElement = new TPipeContainer(msg, inittime);
    if (m_pHeadElement != NULL) m_pHeadElement->setPrev(pNewElement);
    m_pHeadElement = pNewElement;

    //Noch kein Tail vorhanden
    if (m_pTailElement == NULL)
    {
      m_pTailElement = m_pHeadElement;
    }
    m_lSize++;
    m_pLock->leave();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}
//-----------------------------------------------------------------------------

REMOTE_OBJECT_IMPL(TCommPipe, L"TCommPipe", L"", TCommPipe_Skel);

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL_LOCK(TCommPipe, m_pLock);
  INTERFACE_CAST(ICommPipe, INTERFACE_ICommPipe);
MEMORY_MANAGER_IMPL_PARENT_END(TAbstractRemoteObject);

