/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CommPipe_Stub                                                 *
 * Package : CellModul                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:16:56 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CommPipe_Stub.cpp,v $
 *   Revision 1.4  2005/11/25 09:16:56  bhadorn
 *   Using readArray to get messages from the socket. Safe reading.
 *
 *   Revision 1.3  2005/11/16 11:22:50  bhadorn
 *   Communication improved by using RMI for C++ and the Naming service
 *
 *   Revision 1.2  2005/11/10 11:00:46  bhadorn
 *   Apply CCTP changes of response data type. New is TByteArray
 *
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 


#include <zeusbase/CellModel/CommPipe_Stub.h>
#include <zeusbase/System/LoggerManager.h> 
#include <zeusbase/System/ByteArray.hpp>
#include <zeusbase/System/CriticalSection.h> 
#include <zeusbase/System/Protocols/CCTPRequest.h> 
#include <zeusbase/System/Protocols/CCTPResponse.h> 
#include <zeusbase/System/Socket.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a cell registry query object
*/
/***************************************************************************/
TCommPipe_Stub::TCommPipe_Stub() : TAbstractStub()
{
  m_bLocked = false;
  m_pLock = new TCriticalSection();
}


/***************************************************************************/
/*! Destroys a cell registry query object
*/
/***************************************************************************/
TCommPipe_Stub::~TCommPipe_Stub()
{
  m_pLock->release();
}


/***************************************************************************/
/*! Locks the pipe if an error occurs
*/
/***************************************************************************/
void TCommPipe_Stub::lock()
{
  m_bLocked = true;
}

/***************************************************************************/
/*! Unlocks the pipe if the connection has been reset
*/
/***************************************************************************/
void TCommPipe_Stub::unlock()
{
  m_bLocked = false;
}

/***************************************************************************/
/*! connects to the remote object
    \param address : The Address of the pipe host
    \param port : Port of the pipe host
*/
/***************************************************************************/
unsigned long MQUALIFIER TCommPipe_Stub::connect(const IString* address, unsigned long port)
{
  ZLOGGER.printf(LOGMODE_INFO, "Try to connect to remote pipe...");
  
  unsigned long ulRetval = TAbstractStub::connect(address, port);
    
  if (ulRetval == RET_NOERROR)
  {
    unlock();
    ZLOGGER.println(LOGMODE_INFO, "done.");
    ulRetval = RET_NOERROR;
  }
  else
  {
    lock();
    ZLOGGER.println(LOGMODE_INFO, "failed.");
    ulRetval = RET_REQUEST_FAILED;  
  }  
  return ulRetval;
}

/***************************************************************************/
/*! see ICommPipe
*/
/***************************************************************************/
unsigned long MQUALIFIER TCommPipe_Stub::sendMessage(IMessage* msg, unsigned long long inittime /*= 0*/)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (msg != NULL)
  {
    //TODO: Choose the correct transport protocoll
    m_pLock->enter();
    ulRetval = sendMessageAsBinary(msg, inittime);
    m_pLock->leave();
  }
  else
  {
    ulRetval = RET_INVALID_PARAMETER;
  }

  return ulRetval;
}

/***************************************************************************/
/*! sends a MessageObject to a remote system using the Cell
    Response Protocol. This method must be protected by a lock before 
    calling. Only one thread has access to this method
*/
/***************************************************************************/
unsigned long TCommPipe_Stub::sendMessageAsBinary(IMessage* msg, unsigned long long /*inittime*/)
{
  TByteArray aBytes;
  ZLOGGER.printfln(LOGMODE_INFO, "Sending a simple message to a remote system");
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pSocket == NULL)
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  else if(msg->serialize(&aBytes) == RET_NOERROR)
  {
    TCCTPRequest Request;
    Request.setData(aBytes);
    Request.setMethod(CCTP_METHOD_POST);
    Request.setObjectType(INTERFACE_ICommPipe);
    
    TByteArray aOutData;
    Request.toStream(&aOutData);
    if (m_pSocket->writeArray(&aOutData) == RET_NOERROR)
    {
      //Lesen des Inputs der Response
      TByteArray aData;
      TCCTPResponse Response;
      if (m_pSocket->readArray(&aData) == RET_NOERROR)
      {
        aData.add('\0');
        Response.parse(aData);
        
        if (Response.getState() == CCTP_STATUS_OK)
        {
          ulRetval = RET_NOERROR;
          ZLOGGER.printfln(LOGMODE_INFO, "done");      
        }
        else
        {
          ulRetval = RET_REMOTECALL_ERROR;
          ZLOGGER.printfln(LOGMODE_ERROR, "failed. [Protocol returned error]. Error code=%d: Message=%s", (int)Response.getState(), TString::transcode(Response.getData()).c_str());
        }
      }
      else
      {
        ulRetval = RET_INTERNAL_ERROR;
        ZLOGGER.printfln(LOGMODE_ERROR, "failed. [Socket returned error]");
        lock();
      }
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_ERROR, "failed writing. Pipe locked.");
      lock();
    }
  }
  else
  {
    ZLOGGER.printfln(LOGMODE_ERROR, "failed. [Could not stream or no socket]");
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}

/*****************************************************************************/
/*! See ICommPipe
*/
/*****************************************************************************/
STUB_DELEGATION("getPipeName", unsigned long MQUALIFIER TCommPipe_Stub::getPipeName(IString* name) const)
  STUB_RETURN_START;
    STUB_RETPARAM_STRING("name", name)
  STUB_RETURN_END(ulRetval);

/*****************************************************************************/
/*! See ICommPipe
*/
/*****************************************************************************/
STUB_DELEGATION("getCellName", unsigned long MQUALIFIER TCommPipe_Stub::getCellName(IString* name) const)
  STUB_RETURN_START;
    STUB_RETPARAM_STRING("name", name)
  STUB_RETURN_END(ulRetval);


/***************************************************************************/
/*! see ICommPipe
*/
/***************************************************************************/
STUB_DELEGATION("getPriority", unsigned long MQUALIFIER TCommPipe_Stub::getPriority() const)
  long lValue = PIPEPRIORITY_NORMAL;
  STUB_RETURN_START;
    STUB_RETVALUE_LONG(lValue)
  STUB_RETURN_END(lValue);

/***************************************************************************/
/* method stub for isLocked()
*/
/***************************************************************************/
bool MQUALIFIER TCommPipe_Stub::isLocked() const
{
  return m_bLocked;
}
// STUB_DELEGATION("isLocked", bool MQUALIFIER TCommPipe_Stub::isLocked() const)
//   long lValue = 0;
//   STUB_RETURN_START;
//     STUB_RETVALUE_LONG(lValue)
//   STUB_RETURN_END((lValue || m_bLocked));

/***************************************************************************/
/*! see ICommPipe
*/
/***************************************************************************/
bool MQUALIFIER TCommPipe_Stub::hasRemoteAccess() const
{ return false; }


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL_LOCK(TCommPipe_Stub, m_pLock);
  INTERFACE_CAST(ICommPipe, INTERFACE_ICommPipe);
MEMORY_MANAGER_IMPL_PARENT_END(TAbstractStub);
