/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ILocalPipeRegistry                                            *
 * Package : MOM                                                           *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:25:23 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ILocalPipeRegistry.hpp,v $
 *   Revision 1.2  2005/11/16 11:25:23  bhadorn
 *   New communication interface for pipes created for naming service
 *
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
 
#ifndef ILocalPipeRegistryHPP
#define ILocalPipeRegistryHPP

#include <zeusbase/System/Interfaces/IZUnknown.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/CellModel/Interfaces/IOwnCommPipe.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! Interface describes a local pipe registry to register and unregister
    cell pipes
*/
/***************************************************************************/
class ILocalPipeRegistry : public IZUnknown
{
  public:
    /***********************************************************************/
    /*! Checks is the local pipe registry is correctly connected to the
        naming service
        \retval true : Connected
        \retval false : not connected
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER isConnected() const=0;
    
    /***********************************************************************/
    /*! Returns the port of the naming service connection
        \return tcp port
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getPort() const=0;
    
    /***********************************************************************/
    /*! Returns the address of the naming service
        \param addr : Return parameter of the ip address
        \retval RET_NOERROR : Address returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getAddress(IString* addr) const=0;
    
    /***********************************************************************/
    /*! Creates a pipe for a cell environment
        \param cellname : Name of the owner cell
        \param pipename : Name of the pipe
        \param priority : Priority of the pipe
        \param create_remote_access : Creates a remote access (skeleton for 
                                      this pipe. This will not register the
                                      pipe at the naming service. Call 
                                      registerPipe() to do so.
        \param pipe : return parameter 
        \retval RET_NOERROR : Pipe created and returned
        \retval RET_INVALID_PARAMETER : invalid parameter received
        \retval RET_REQUEST_FAILED : Could not create a pipe.
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER createPipe(const IString* cellname, 
                                                const IString* pipename, 
                                                unsigned long priority, 
                                                bool create_remote_access, 
                                                IOwnCommPipe** pipe)=0;
    
    /***********************************************************************/
    /*! This registeres a pipe. If the pipe has remote access it will 
        register the pipe at the naming service as well.
        \param pipe : Pipe to register.
        \retval RET_NOERROR : Pipe registered
        \retval RET_REQUEST_FAILED : Could not register the pipe
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER registerPipe(ICommPipe* pipe)=0;
    
    /***********************************************************************/
    /*! Unregisteres a pipe from the local pipe registry. If the pipe has
        remote access, the pipe will be unbound from the naming service
        \param pipe : Pipe instance to unregister
        \retval RET_NOERROR : Pipe unregistered
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED: Could not unregister pipe.
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER unregisterPipe(ICommPipe* pipe)=0;
    
    /***********************************************************************/
    /*! Returns a pipe from the registry. If the pipe has been registered 
        locally it will return this pipe, otherwise it will return a 
        new pipe from the naming service. In this case the pipe stub will
        be registered automatically.
        \param cellname : Name of the cell providing this pipe
        \param pipename : Name of the pipe
        \param pipe : Return parameter of the pipe instance
        \retval RET_NOERROR : Pipe found and returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not find a pipe 
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getPipe(const IString* cellname, const IString* pipename, ICommPipe** pipe)=0;
    
    /***********************************************************************/
    /*! Returns a list of pipes from the registry. This method will
        look internally first. If there's no pipe locally available, this
        method call will do a lookup from the naming service 
        (see updatePipes()). <BR>
        To update or synchronize pipes with the naming service use 
        the method updatePipes() directly.
        \param cellname : Name of the cell providing this pipe
        \param pipename : Name of the pipe
        \param list : Return parameter of the pipe list
        \retval RET_NOERROR : Pipe found and returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not find a pipe 
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getPipes(const IString* cellname, const IString* pipename, IList<ICommPipe*>* list)=0;
    
    /***********************************************************************/
    /*! This method synchronizes a list of pipes (identified by cellname and
        pipename) with the naming service. First it gets a list of all 
        registered pipes from the naming service. Then it add new pipes and
        removes wasted pipes from the local registry and the list given by 
        the caller. It will return the actual list of pipes to the caller.
        \param cellname : Name of the cell
        \param pipename : Name of the pipe
        \param list : In and out parameter containing all pipes 
        \retval RET_NOERROR : Pipe list updated
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : No such pipes are availale.
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER updatePipes(const IString* cellname, const IString* pipename, IList<ICommPipe*>* list)=0;

};
END_NAMESPACE_Zeus

#endif
