/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : LocalPipeRegistry                                             *
 * Package : MOM                                                           *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:15:57 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: LocalPipeRegistry.cpp,v $
 *   Revision 1.3  2005/11/25 09:15:57  bhadorn
 *   Changed lock mode to recursive
 *
 *   Revision 1.2  2005/11/16 11:22:50  bhadorn
 *   Communication improved by using RMI for C++ and the Naming service
 *
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
 
#include <zeusbase/CellModel/LocalPipeRegistry.h>
#include <zeusbase/CellModel/CommPipe.h>
#include <zeusbase/CellModel/CommPipe_Stub.h>
#include <zeusbase/System/CriticalSection.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/System/StringList.h>
#include <zeusbase/Remote/Naming.h>

USING_NAMESPACE_Zeus


/***************************************************************************/
/*! Creates a new local pipe registry
    \param node : Node containing the attributes
*/
/***************************************************************************/
TLocalPipeRegistry::TLocalPipeRegistry(IXMLNode* node) : TXObject(node)
{
  m_bConnected = false;
  TString strPort;
  this->readAttribute(L"RegistryPort", strPort);
  m_ulPort = strPort.toLong(0);
  this->readAttribute(L"RegistryAddress", m_strAddress);
  
  m_pDataLock = new TCriticalSection(TCriticalSection::ltRecursive);
  Naming.useRemote();
  m_bConnected = (Naming.connect(m_strAddress, m_ulPort) == RET_NOERROR);
  if (m_bConnected)
  {
    ZLOGGER.println(LOGMODE_INFO, "Connected to the naming service [%s:%d].", m_strAddress.c_str(), m_ulPort);    
  }
  else
  {
    ZLOGGER.println(LOGMODE_ERROR, "Could not connected to the naming service [%s:%d].", m_strAddress.c_str(), m_ulPort);    
  }
}

/***************************************************************************/
/*! Destroys local pipe registry
*/
/***************************************************************************/
TLocalPipeRegistry::~TLocalPipeRegistry()
{  
  IListIterator<ICommPipe*>* pIt = m_mapPipes.getIterator();
  while(pIt->hasNextItem())
  {
    ICommPipe* pPipe = pIt->getNextItem();
    
    //Removes the pipe from the registry if the pipe has remote access
    TString strPipeName = getPipeObjectName(pPipe);
    removeFromNaming(pPipe, strPipeName);
    
    pPipe->release();
  }
  m_mapPipes.releaseIterator(pIt);
  m_mapPipes.clear();
  
  m_pDataLock->release();
  m_pDataLock = NULL;
  Naming.useRemote(false);
}

/***************************************************************************/
/*! This removes a pipe from the registry
    \param pipe : Pipe to unbind from naming service
    \param objname : Objectname of the pipe.
*/
/***************************************************************************/
void TLocalPipeRegistry::removeFromNaming(ICommPipe* pipe, const TString& objname)
{
  if (pipe->hasRemoteAccess())
  {
    TString strCellName;
    pipe->getCellName(&strCellName);
    TString strPipeName;
    pipe->getPipeName(&strPipeName);
    strCellName = strCellName + TString(L"/") + strPipeName;
                            
    if (Naming.unbind2(&objname, &strCellName) == RET_NOERROR)
    {
      ZLOGGER.printfln(LOGMODE_INFO, "Unregistered at naming service...ok");
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_INFO, "Unregistered at naming service...failed");
    }
  }
}

/***************************************************************************/
/*! This static method is used to create an object name out of a 
    pipe
    \param pipe : Pipe instance
    \return generated pipe name
*/
/***************************************************************************/
TString TLocalPipeRegistry::getPipeObjectName(ICommPipe* pipe)
{
  TString strPipeAddress;
  TString strCellName;
  TString strPipeName;
  unsigned long ulPipePort= 0;
  pipe->getCellName(&strCellName);
  pipe->getPipeName(&strPipeName);
  IRemoteObject* pObject = NULL;
  
  if (pipe->hasRemoteAccess() && pipe->askForInterface(INTERFACE_IRemoteObject, (IZUnknown**)&pObject) == RET_NOERROR)
  {
    pObject->getConnectionParam(&strPipeAddress, &ulPipePort);
    pObject->release();
  }
  else
  {
    strPipeAddress = L"local";
  }
  
  return  TString(L"ccm://") + 
          strPipeAddress + 
          TString(L":") + 
          TString((long)ulPipePort) + 
          TString(L"/") + 
          strCellName +
          TString(L"/") + 
          strPipeName;
}

/***************************************************************************/
/*! See ILocalPipeRegistry
*/
/***************************************************************************/
unsigned long MQUALIFIER TLocalPipeRegistry::isConnected() const
{ return m_bConnected;}

/***************************************************************************/
/*! See ILocalPipeRegistry
*/
/***************************************************************************/
unsigned long MQUALIFIER TLocalPipeRegistry::getPort() const
{ return m_ulPort; }

/***************************************************************************/
/*! See ILocalPipeRegistry
*/
/***************************************************************************/
unsigned long MQUALIFIER TLocalPipeRegistry::getAddress(IString* addr) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (addr != NULL)
  {
    addr->assignStr(&m_strAddress);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! See ILocalPipeRegistry
*/
/***************************************************************************/
unsigned long MQUALIFIER TLocalPipeRegistry::createPipe(const IString* cellname, 
                                                        const IString* pipename, 
                                                        unsigned long priority, 
                                                        bool create_remote_access, 
                                                        IOwnCommPipe** pipe)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (cellname == NULL || pipename == NULL || pipe == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TCommPipe* pPipe = new TCommPipe(priority, TString(pipename), TString(cellname));

    //Create the remote access of a pipe
    if (create_remote_access)
    {
      pPipe->connect();
    }

    *pipe = pPipe;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! See ILocalPipeRegistry
*/
/***************************************************************************/
unsigned long MQUALIFIER TLocalPipeRegistry::registerPipe(ICommPipe* pipe)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (pipe == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strPipeObjName = getPipeObjectName(pipe);
    ulRetval = addPipe(strPipeObjName, pipe);
  }
  return ulRetval;
}


/***************************************************************************/
/*! Adds a pipe to the registry. Such a pipe could be a stub object 
    or a remote object. Stub objects wont be bound by the naming service.
    If the pipe with the key name already exists, this method fails.
    \param key : Key name of the pipe. The format of the key is 
                 ccm://address:port/cellname/pipename
    \param pipe : Pipe instance to be added
    \retval RET_REQUEST_FAILED : Key name already exists
    \retval RET_NOERROR : Pipe added
    \pre The parameter pipe must be valid.
*/
/***************************************************************************/
unsigned long TLocalPipeRegistry::addPipe(const TString& key, ICommPipe* pipe)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  ZLOGGER.printfln(LOGMODE_INFO, "Try to register pipe %s.", key.c_str());
  ZLOGGER.indent();
  
  //////////////////////////
  // DATA LOCK
  m_pDataLock->enter();
  
  if (!m_mapPipes.hasItem(key))
  {    
    //Register pipe at the naming service if the pipe has remote access
    IRemoteObject* pRemoteObject = NULL;
    if (pipe->hasRemoteAccess() && 
        pipe->askForInterface(INTERFACE_IRemoteObject, (IZUnknown**)&pRemoteObject) == RET_NOERROR)
    {
      TString strCellName;
      pipe->getCellName(&strCellName);
      TString strPipeName;
      pipe->getPipeName(&strPipeName);
      strCellName = strCellName + TString(L"/") + strPipeName;
      if (Naming.rebind2(&key, &strCellName, pRemoteObject) == RET_NOERROR)
      {
        ZLOGGER.printfln(LOGMODE_INFO, "Registered at naming service...ok");
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_INFO, "Registered at naming service...failed");
      }
      pRemoteObject->release();
    }

    m_mapPipes.setItem(key, pipe);
    pipe->addRef();
    ulRetval = RET_NOERROR;
    ZLOGGER.unindent();
    ZLOGGER.printfln(LOGMODE_INFO, "...done.");
    ulRetval = RET_NOERROR;
  }
  else
  {
    ZLOGGER.unindent();
    ZLOGGER.printfln(LOGMODE_INFO, "...failed. [Pipe already exists]");
  }
  m_pDataLock->leave();
  //////////////////////////
  
  return ulRetval;
}


/***************************************************************************/
/*! See ILocalPipeRegistry
*/
/***************************************************************************/
unsigned long MQUALIFIER TLocalPipeRegistry::unregisterPipe(ICommPipe* pipe)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (pipe == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strCellName;
    TString strPipeName;
    pipe->getCellName(&strCellName);
    pipe->getPipeName(&strPipeName);
    ZLOGGER.printfln(LOGMODE_INFO, "Try to unregister pipe %s/%s.", strCellName.c_str(), strPipeName.c_str());
    ZLOGGER.indent();
    
    //////////////////////////
    // DATA LOCK
    m_pDataLock->enter();
    
    IStringMapIterator<ICommPipe*>* pIt = m_mapPipes.getIterator();
    bool bFound = false;
    TString strKey;
    while(!bFound && pIt->hasNextItem())
    {
      ICommPipe* pPipe = NULL;
      
      pIt->getNextKey(&strKey, &pPipe);
      if (pPipe == pipe)
      {
        bFound = true;
        removeFromNaming(pipe, strKey);
        m_mapPipes.deleteItem(strKey);
        pPipe->release();
      }
    }
    m_mapPipes.releaseIterator(pIt);
    
    if (bFound)
    {   
      ZLOGGER.unindent();
      ZLOGGER.printfln(LOGMODE_INFO, "...done.[%s]", strKey.c_str());
      ulRetval = RET_NOERROR;
    }
    else
    {
      ZLOGGER.unindent();
      ZLOGGER.printfln(LOGMODE_INFO, "...failed. [unable to unregister]");
    }
    m_pDataLock->leave();
    //////////////////////////
  }
  return ulRetval;
}

/***************************************************************************/
/*! See ILocalPipeRegistry
*/
/***************************************************************************/
unsigned long MQUALIFIER TLocalPipeRegistry::getPipe(const IString* cellname, const IString* pipename, ICommPipe** pipe)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (cellname == NULL || pipename == NULL || pipe == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    ZLOGGER.printfln(LOGMODE_INFO, "Try to get pipe...");
    ZLOGGER.indent();
    
    //////////////////////////
    // DATA LOCK
    m_pDataLock->enter();
    bool bFound = false;
    IListIterator<ICommPipe*>* pIterator = m_mapPipes.getIterator();
    while(!bFound && pIterator->hasNextItem())
    {
      ICommPipe* pItem = pIterator->getNextItem();
      TString strCellName;
      pItem->getCellName(&strCellName);
      if (cellname->equalsStr(&strCellName))
      {
        TString strPipeName;
        pItem->getPipeName(&strPipeName);
        if (pipename->equalsStr(&strPipeName))
        {
          bFound = true;
          *pipe = pItem;
          pItem->addRef();
          ulRetval = RET_NOERROR;
        }
      }
    }
    m_mapPipes.releaseIterator(pIterator);
    m_pDataLock->leave();
    //////////////////////////

    if (bFound)
    {
      ZLOGGER.unindent();
      ZLOGGER.printfln(LOGMODE_INFO, "...done.[Found]");
    }
    
    //Pipe not found -> try to get pipe from the naming service
    else 
    {
      ZLOGGER.printfln(LOGMODE_INFO, "...not found. Get pipe from naming service...");
      IRemoteObject* pRemoteStub = NULL;
      TString strCat = TString(cellname) + TString(L"/") + TString(pipename);
      if (Naming.lookupAny(&strCat, &pRemoteStub) == RET_NOERROR)
      {
        ZLOGGER.printfln(LOGMODE_INFO, "...done.[Found]");
        ICommPipe* pPipe = NULL;
        if (pRemoteStub->askForInterface(INTERFACE_ICommPipe, (IZUnknown**)&pPipe) == RET_NOERROR)
        {
          *pipe = pPipe;
          ulRetval = RET_NOERROR;
        }
        else
        {
          ZLOGGER.printfln(LOGMODE_FATAL, "Internal error: Could not cast remote object to ICommPipe.");
        }
        pRemoteStub->release();
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_INFO, "...failed.[No pipe found]");
      }
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! See ILocalPipeRegistry
*/
/***************************************************************************/
unsigned long MQUALIFIER TLocalPipeRegistry::getPipes(const IString* cellname, const IString* pipename, IList<ICommPipe*>* list)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (cellname == NULL || pipename == NULL || list == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    ZLOGGER.printfln(LOGMODE_INFO, "Try to get pipe list...");
    ZLOGGER.indent();

    if (this->getPipesFromLocal(cellname, pipename, list) == RET_NOERROR)
    {
      ZLOGGER.unindent();
      ZLOGGER.printfln(LOGMODE_INFO, "...done.[Found=%d]", list->getCount());
      ulRetval = RET_NOERROR;
    }
    else if (this->getPipesFromRemote(cellname, pipename, list, 200) == RET_NOERROR)
    {
      ZLOGGER.unindent();
      ZLOGGER.printfln(LOGMODE_INFO, "...done. [remote found=%d]",list->getCount());
      ulRetval = RET_NOERROR;
    }
    else
    {
      ZLOGGER.unindent();
      ZLOGGER.printfln(LOGMODE_INFO, "...failed. [none found]");
    }
//    printf("GetPipes called. %d found\n", list->getCount());
  }
  return ulRetval;
}

/***************************************************************************/
/*! See ILocalPipeRegistry
*/
/***************************************************************************/
unsigned long MQUALIFIER TLocalPipeRegistry::updatePipes(const IString* cellname, const IString* pipename, IList<ICommPipe*>* list)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (cellname == NULL || pipename == NULL || list == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TArrayList<ICommPipe*> lstOldPipes;
    this->getPipesFromLocal(cellname, pipename, &lstOldPipes);
    
    TString strCat = TString(cellname) + TString(L"/") + TString(pipename);
    ZLOGGER.printfln(LOGMODE_INFO, "Try to get all pipes from registry [%s]...", strCat.c_str());
    TStringList lstPipeNames;
    if(Naming.getObjectNames(&strCat, &lstPipeNames) != RET_NOERROR)
    {
      ZLOGGER.printfln(LOGMODE_INFO, "...failed [No pipes available]");
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_INFO, "...done [%d received]", lstPipeNames.getCount());
      ZLOGGER.indent();
      IListIterator<IString*>* pIt = lstPipeNames.getIterator();
      while(pIt->hasNextItem())
      {
        ZLOGGER.printfln(LOGMODE_DEBUG, "**** Name is %s", pIt->getNextItem()->c_str());
        //printf("**** Pipe from remote: %s\n", pIt->getNextItem()->c_str());
      }
      lstPipeNames.releaseIterator(pIt);
      ZLOGGER.unindent();
      
    }
    
    //--------------------------------------------------------------------
    // CLEAR NAME LIST
    // Remove all pipes witch are already known to this registry
    // add them to the return list
    
    //////////////////////////
    // DATA LOCK
    m_pDataLock->enter();
    long lNameCount = lstPipeNames.getCount();
    for(int i = 0; i < lNameCount; i++)
    {
      TString strKey = lstPipeNames[i];
      if (m_mapPipes.hasItem(strKey))
      {
        //printf("**** Pipe is registered: %s\n", strKey.c_str());
        ICommPipe* pPipe = m_mapPipes.getItem(strKey);
        
        //Not in the caller list
        if (list->indexOf(pPipe) == -1)
        {
          list->add(pPipe);
          pPipe->addRef();
          //printf("  - Pipe not in caller list ->added [%s]\n", strKey.c_str());
        }
        
        //In the registry list
        if (lstOldPipes.indexOf(pPipe) != -1)
        {
          lstOldPipes.remove(pPipe);
          pPipe->release();
        }
        lstPipeNames.deleteItem(i);
        i--;
        lNameCount--;
      }
    }
    m_pDataLock->leave();
    //////////////////////////
    
    
    //--------------------------------------------------------------------
    // CLEAR EXISTING PIPE LIST
    // Remove all pipes witch are not available any more (locked)
    // If the pipes here are not locked, they might be only local pipes
    // ->add them to the return list as well
    // clear this list here
    long lOldCount = lstOldPipes.getCount();
    for(int i = 0; i < lOldCount; i++)
    {
      ICommPipe* pPipe = lstOldPipes[i];
      
      //If the pipe is locked 
      // -> remove pipe from registry and from the caller list
      if (pPipe->isLocked())
      {
        //printf("**** Pipe is locked -> unregistered.\n");
        this->unregisterPipe(pPipe);
        if (list->indexOf(pPipe) != -1)
        {
          list->remove(pPipe);
          pPipe->release();
          //printf("  - Pipe removed from the caller list.\n");
        }
      }
      
      //Pipe is registered and unlocked. If the pipe is not 
      //inside the caller list 
      // -> add pipe to caller list
      else
      {
        if (list->indexOf(pPipe) == -1)
        {
          //printf("  - New Pipe added to the caller list.\n");
          list->add(pPipe);
          pPipe->addRef();
        }
      }
      pPipe->release();
    }
    lstOldPipes.clear();
    
    
    //--------------------------------------------------------------------
    // ADD NEW PIPES FROM NAMING SERVICE
    // Add new pipes to the registry. All remained names are new pipes
    // to be added to the registry
    IListIterator<IString*>* pIt = lstPipeNames.getIterator();
    while(pIt->hasNextItem())
    {
      TString strName = pIt->getNextItem();
      IRemoteObject* pObject = NULL;
      if (Naming.lookup2(&strName, &strCat, &pObject) == RET_NOERROR)
      {
        ICommPipe* pPipe = NULL;
        if (pObject->askForInterface(INTERFACE_ICommPipe, (IZUnknown**)&pPipe) == RET_NOERROR)
        {
          if (this->addPipe(strName, pPipe) == RET_NOERROR)
          {
            list->add(pPipe);
            pPipe->addRef();
            //printf("  - New pipe created [%s].\n", strName.c_str());
            ZLOGGER.printfln(LOGMODE_INFO, "Update Pipes: pipe added [%s]", strName.c_str());
          }
          else
          {
            ZLOGGER.printfln(LOGMODE_ERROR, "Update Pipes: Could not register pipe [%s]", strName.c_str());
          }
          pPipe->release();
        }
        else
        {
          ZLOGGER.printfln(LOGMODE_ERROR, "Update Pipes: lookup failed [%s]", strName.c_str());
        }
        pObject->release();
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_ERROR, "Update Pipes: lookup failed [%s]", strName.c_str());
      }
    }
    lstPipeNames.releaseIterator(pIt);
    
    if (list->getCount() > 0)
    {
      ulRetval = RET_NOERROR;
    }
  }  
  //printf("============================================.\n");
  return ulRetval;
}


/***************************************************************************/
/*! this method tries to get the a list of pipes from a remote system. If this was done
    successful, a RemotePipe-Instance has been created and added to the
    local pipe registry
    \param cellname : Name of the cell
    \param pipename : Name of the pipe
    \param pipe : return value of the created remote pipe
    \retval RET_NOERROR : Pipe created
    \retval RET_REQUEST_FAILED : No pipe found
    \retval RET_INTERNAL_ERROR : Error occured getting the pipe
    \pre cellname, pipename and pipe must be valid parameters
*/
/***************************************************************************/
unsigned long TLocalPipeRegistry::getPipesFromRemote(const IString* cellname, const IString* pipename, IList<ICommPipe*>* list, long /*max_count*/)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  TStringList lstNames;
  ZLOGGER.printfln(LOGMODE_INFO, "Try to get all pipes from registry...");
  ZLOGGER.indent();
  TString strCat = TString(cellname) + TString(L"/") + pipename;
  if (Naming.getObjectNames(&strCat, &lstNames) == RET_NOERROR)
  {
    ZLOGGER.unindent();
    ZLOGGER.printfln(LOGMODE_INFO, "...done.");
    IListIterator<IString*>* pItNames = lstNames.getIterator();
    while(pItNames->hasNextItem())
    {
      TString strName = pItNames->getNextItem();
      IRemoteObject* pObject = NULL;
      if (Naming.lookup2(&strName, &strCat, &pObject) == RET_NOERROR)
      {
        //Convert to ICommPipe Interfaces
        ICommPipe* pPipe = NULL;
        if (pObject->askForInterface(INTERFACE_ICommPipe, (IZUnknown**)&pPipe) == RET_NOERROR)
        {
          if (this->addPipe(strName, pPipe) == RET_NOERROR)
          {
            list->add(pPipe);
          }
          else
          {
            pPipe->release();
          }
        }
        pObject->release();
      }
    }
    lstNames.releaseIterator(pItNames);
    
    //If pipes have been found
    if (list->getCount() > 0)
    {
      ZLOGGER.println(LOGMODE_INFO, "...done.");
      ulRetval = RET_NOERROR;
    }
    else
    {
      ZLOGGER.println(LOGMODE_INFO, "...failed.");
    }
  }
  else
  {
    ZLOGGER.unindent();
    ZLOGGER.printfln(LOGMODE_INFO, "...failed.");
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}

/***************************************************************************/
/*! this method tries to get the a list of pipes from the local registry. 
    \param cellname : Name of the cell
    \param pipename : Name of the pipe
    \param list : return list of the local pipes
    \retval RET_NOERROR : Pipes returned
    \retval RET_REQUEST_FAILED : No pipes found
    \retval RET_INTERNAL_ERROR : Error occured getting the pipes
    \pre cellname, pipename and pipe must be valid parameters
*/
/***************************************************************************/
unsigned long TLocalPipeRegistry::getPipesFromLocal(const IString* cellname, const IString* pipename, IList<ICommPipe*>* list)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  //////////////////////////
  // DATA LOCK
  m_pDataLock->enter();
  IListIterator<ICommPipe*>* pIterator = m_mapPipes.getIterator();
  bool bFound = false;
  while(pIterator->hasNextItem())
  {
    ICommPipe* pItem = pIterator->getNextItem();
    TString strCellName;
    pItem->getCellName(&strCellName);
    if (cellname->equalsStr(&strCellName))
    {
      TString strPipeName;
      pItem->getPipeName(&strPipeName);
      if (pipename->equalsStr(&strPipeName))
      {
        bFound = true;
        list->add(pItem);
        pItem->addRef();
        ulRetval = RET_NOERROR;
      }
    }
  }
  m_mapPipes.releaseIterator(pIterator);
  m_pDataLock->leave();
  //////////////////////////
  
  if (bFound)
  {
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}    

 
/**************************************************************************/
/*! see IXObject
*/
/**************************************************************************/
bool MQUALIFIER TLocalPipeRegistry::freeze()
{
  bool bRetval = TXObject::freeze();
  if (bRetval)
  {
    ///
  }
  return bRetval;
}

/**************************************************************************/
/*! see IXObject
*/
/**************************************************************************/
bool MQUALIFIER TLocalPipeRegistry::unfreeze()
{
  bool bRetval = TXObject::unfreeze();
  if (bRetval)
  {
  }
  return bRetval;
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TLocalPipeRegistry);
  INTERFACE_CAST(ILocalPipeRegistry, INTERFACE_ILocalPipeRegistry);
MEMORY_MANAGER_IMPL_PARENT_END(TXObject);

