/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : OutputPipeManager                                             *
 * Package : CellModel                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:24:32 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: OutputPipeManager.cpp,v $
 *   Revision 1.2  2005/11/16 11:24:32  bhadorn
 *   Error updating pipe list. Same pipe list was added multiple times.
 *
 *   Revision 1.1  2005/10/28 12:16:13  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/CellModel/OutputPipeManager.h> 
#include <zeusbase/CellModel/OutputPipeList.h> 

/******************************************************************************/
/*! Creates a new output pipe manager object
*/
/******************************************************************************/
TOutputPipeManager::TOutputPipeManager() : TZObject()
{
  m_pRegistry = NULL;
  m_pOutputPipeLock = new TCriticalSection(TCriticalSection::ltRecursive);
}


/******************************************************************************/
/*! Destroys the output pipe manager object
*/
/******************************************************************************/
TOutputPipeManager::~TOutputPipeManager()
{
  removeAllOutputPipes();
  if (m_pRegistry != NULL)
  {
    m_pRegistry->release();
    m_pRegistry = NULL;
  }
  
  m_pOutputPipeLock->release();
  m_pOutputPipeLock = NULL;
}


/******************************************************************************/
/*! Sets an instance of the local pipe registry.
    \param registry : Local pipe registry
*/
/******************************************************************************/
void TOutputPipeManager::setRegistry(ILocalPipeRegistry* registry)
{
  if (m_pRegistry != NULL)
  {
    m_pRegistry->release();
    m_pRegistry = NULL;
  }
  m_pRegistry = registry;
  if (m_pRegistry != NULL)
  {
    m_pRegistry->addRef();
  }
}

/******************************************************************************/
/*! Adds a pipe to the manager.
    \param pPipe : Pipe reference
*/
/******************************************************************************/
void TOutputPipeManager::addPipe(ICommPipe* pPipe)
{ 
  ZLOGGER.indent();
  if (pPipe != NULL)
  {
    TString strCellName;
    TString strPipeName;
    pPipe->getCellName(&strCellName);
    pPipe->getPipeName(&strPipeName);
    ZLOGGER.printfln(LOGMODE_INFO, "CellName=%s; PipeName=%s", strCellName.c_str(), strPipeName.c_str());
      
    //Add a special pipe. This pipe is not from the registry
    TOutputPipeList* pList = new TOutputPipeList(strCellName, strPipeName, false);
      
    //////////////////////////
    //OUTPUT PIPE LOCK
    m_pOutputPipeLock->enter();
    pList->add(pPipe);
    m_lstOutputPipes.add(pList);
    m_pOutputPipeLock->leave();
    //////////////////////////
  }
  ZLOGGER.unindent();
}


/******************************************************************************/
/*! Removes all output pipe references
*/
/******************************************************************************/
void TOutputPipeManager::removeAllOutputPipes()
{
  //////////////////////////
  //OUTPUT PIPE LOCK
  m_pOutputPipeLock->enter();
  while(m_lstOutputPipes.getCount() > 0)
  {
    TOutputPipeList* pPipeList = m_lstOutputPipes[0];
    unregisterPipes(pPipeList);
    pPipeList->releaseInstances();
    pPipeList->release();
    m_lstOutputPipes.deleteItem(0);
  }
  m_pOutputPipeLock->leave();
  //////////////////////////
  //ZLOGGER.printfln(LOGMODE_INFO, "Output pipes removed");
}

/******************************************************************************/
/*! Removes all output pipe references
*/
/******************************************************************************/
void TOutputPipeManager::removeLockedOutputPipes()
{
  ZLOGGER.printfln(LOGMODE_INFO, "Try to remove all locked pipes...");  
  //////////////////////////
  //OUTPUT PIPE LOCK
  m_pOutputPipeLock->enter();

  long lIndex = 0;
  
  
  TOutputPipeList* pPipeList = m_lstOutputPipes[lIndex];
  
  while(pPipeList != NULL)
  {
    TSingleLinkedList<ICommPipe*> lstLockedPipes;
    if (pPipeList->getAllLockedPipes(&lstLockedPipes) == RET_NOERROR)
    {
      while (lstLockedPipes.getCount() > 0)
      {
        //The pipe should also removed from the local pipe cach
        if(m_pRegistry != NULL)
        {
          m_pRegistry->unregisterPipe(lstLockedPipes[0]);
        }
        pPipeList->remove(lstLockedPipes[0]);
        lstLockedPipes[0]->release(); // Reference from pPipeList
        
        lstLockedPipes[0]->release(); // Reference from locked pipe list
        lstLockedPipes.deleteItem(0);
        ZLOGGER.printfln(LOGMODE_INFO, "output pipe released.");  
      }
      
      //No pipes are available any more
      if (pPipeList->getCount() == 0)
      {
        pPipeList->releaseInstances();
        pPipeList->release();
        m_lstOutputPipes.deleteItem(lIndex);
        ZLOGGER.printfln(LOGMODE_INFO, "output pipe list removed.");  
      }
      else
      {
        lIndex++;
      }
    }
    else
    {
      lIndex++;
    }
    pPipeList = NULL;
    pPipeList = m_lstOutputPipes[lIndex];
  }
  m_pOutputPipeLock->leave();
  //////////////////////////
  ZLOGGER.printfln(LOGMODE_INFO, "...done.");  
}

/***************************************************************************/
/*! Try to find a pipe. If the pipe has been found before, the lookup
    will be done in the OutputPipe-List. This method is not thread safe and
    therefor it must be used with the m_pOutputPipeLock.
*/
/***************************************************************************/
unsigned long TOutputPipeManager::findPipe(IString* cellname, IString* pipename, ICommPipe** pipe)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  if (cellname == NULL || pipename == NULL || pipe == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    //-------------------------------------------------------------------
    //Try to find a pipe in the output pipe list
    bool bFound = false;
    ZLOGGER.printfln(LOGMODE_INFO, "Try to find pipe %s from %s...", TString(pipename).c_str(), TString(cellname).c_str());
    
    //////////////////////////
    //OUTPUT PIPE LOCK
    m_pOutputPipeLock->enter();
    IListIterator<TOutputPipeList*>* pIterator = m_lstOutputPipes.getIterator();
    while(!bFound && pIterator->hasNextItem())
    {
      TOutputPipeList* pPipeList = pIterator->getNextItem();
      pPipeList->addRef(); //The list can be removed if theres no unlocked pipe any more
      
      TString strCellName = pPipeList->getCellName();
      TString strPipeName = pPipeList->getPipeName();
      if (cellname->equalsStr(&strCellName) && pipename->equalsStr(&strPipeName))
      {
        
        //------------------------------------------------------
        //Try to get a unlocked pipe
        while(!bFound)
        {
          ulRetval = pPipeList->getAnyPipe(pipe);
          if (ulRetval == RET_NOERROR)
          {
            if ((*pipe)->isLocked())
            {
              //CAUTION: This method enters the lock again and 
              //  is able to destroy all content of the pPipeList and 
              //  m_lstOutputPipes. We have one reference stored from
              //  pPipeList. If this list is empty, the loop will 
              //  be broken next time calling getAnyPipe()
              (*pipe)->release();
              *pipe = NULL;
              this->removeLockedOutputPipes(); 
              ulRetval = RET_REQUEST_FAILED;
            }
            else
            {
              ZLOGGER.printfln(LOGMODE_INFO, "Pipe local found %d", (long)(*pipe));
              bFound = true;
            }
          }
          else
          {
            ZLOGGER.printfln(LOGMODE_INFO, "No local pipe found ");
            bFound = true;
          }
        }
        //------------------------------------------------------
      
      }
      
      pPipeList->release();
    }
    m_lstOutputPipes.releaseIterator(pIterator);
    m_pOutputPipeLock->leave();
    //////////////////////////
        
    //--------------------------------------------------------------------------
    //No pipe found or only a locked pipe found -> try to get one of the registry
    if (ulRetval != RET_NOERROR && m_pRegistry != NULL)
    {
      ZLOGGER.printfln(LOGMODE_INFO, "Get pipe from registry...");
      TOutputPipeList* pPipeList = new TOutputPipeList(cellname, pipename, true);
      if (m_pRegistry->getPipes(cellname, pipename, (IList<ICommPipe*>*)pPipeList) == RET_NOERROR)
      {        
        //////////////////////////
        //OUTPUT PIPE LOCK
        m_pOutputPipeLock->enter();
        m_lstOutputPipes.add(pPipeList);
        ulRetval = pPipeList->getAnyPipe(pipe);
        m_pOutputPipeLock->leave();
        //////////////////////////
        
        ZLOGGER.printfln(LOGMODE_INFO, "...Pipe local created.");
      }
      else
      {
        pPipeList->release();
        ulRetval = RET_REQUEST_FAILED;
        ZLOGGER.printfln(LOGMODE_INFO, "...pipe not available.");
      }
    }
    else if (ulRetval == RET_NOERROR)
    {
      ZLOGGER.printfln(LOGMODE_INFO, "...pipe found.");
    }
  }  
  return ulRetval;
}



/***************************************************************************/
/*! Unregister a collection of remote pipes
*/
/***************************************************************************/
void TOutputPipeManager::unregisterPipes(TOutputPipeList* pipes)
{
  if (pipes != NULL && pipes->isFromRegistry() && m_pRegistry != NULL)
  {
    IListIterator<ICommPipe*>* pIterator = pipes->getIterator();
    ICommPipe* pPipe = pIterator->getNextItem();  
    while (pPipe != NULL)
    {
      //The pipe should also removed from the local pipe cach
      m_pRegistry->unregisterPipe(pPipe);
      pPipe = pIterator->getNextItem(); 
    }
    pipes->releaseIterator(pIterator);
  }
}


/***************************************************************************/
/*! this methods checks if there are other pipes available on the system.
    If there are new pipes, the pipes will be added so the system can 
    serve them.
    \retval RET_YES : Pipes added
    \retval RET_NO : No pipes have been added
*/
/***************************************************************************/
unsigned long TOutputPipeManager::refreshPipes()
{
  unsigned long ulRetval = RET_NO;
  
  if (m_pRegistry != NULL)
  {
    //////////////////////////
    //OUTPUT PIPE LOCK
    m_pOutputPipeLock->enter();
    IListIterator<TOutputPipeList*>* pIterator = m_lstOutputPipes.getIterator();
    TOutputPipeList* pPipeList = pIterator->getNextItem();
    while(pPipeList != NULL)
    {
      if (pPipeList->isFromRegistry())
      {
        TString strCellName = pPipeList->getCellName();
        TString strPipeName = pPipeList->getPipeName();
        m_pRegistry->updatePipes(&strCellName, &strPipeName, (IList<ICommPipe*>*)pPipeList);
      }
      pPipeList = pIterator->getNextItem();
    }
    m_lstOutputPipes.releaseIterator(pIterator);
    m_pOutputPipeLock->leave();
    //////////////////////////
  }
  return ulRetval;
}



