/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Module                                                        *
 * Package : MOM                                                           *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 08:57:02 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Module.cpp,v $
 *   Revision 1.2  2005/12/14 08:57:02  bhadorn
 *   Changed to new IModule Interface
 *
 *   Revision 1.1  2005/11/10 10:28:57  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <zeusbase/MOM/Module.h>
#include <zeusbase/MOM/ModuleFunctions.hpp>
#include <zeusbase/System/LibraryManager.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/MOM/ZeusAPI.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates the module
*/
/***************************************************************************/
TModule::TModule(IXMLNode* node) : TXLoaderObject(node) 
{
  m_pZeusAPI = new TZeusAPI(this);
  m_pSession = NULL;
  m_pActiveModule = NULL;
  m_bActive = false;
}


/***************************************************************************/
/*! Destroys the module
*/
/***************************************************************************/
TModule::~TModule()
{
  if (m_bActive)
  {
    this->deactivate();
  }
  if (m_pActiveModule != NULL)
  {
    m_pActiveModule->release();
    m_pActiveModule = NULL;
  }
  if (isLoaded())
  {
    unload();
  }
  
  m_pZeusAPI->release();
  m_pZeusAPI = NULL;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TModule::freeze()
{
  bool bRetval = TXLoaderObject::freeze();
  if (bRetval)
  {
    TString strName;
    this->getName(&strName);
    ZLOGGER.printfln(LOGMODE_INFO, "Freeze TModule %s.", strName.c_str());
    ZLOGGER.indent();
    ZLOGGER.unindent();
  }
  return bRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TModule::unfreeze()
{
  bool bRetval = TXLoaderObject::unfreeze();
  if (bRetval)
  {
    TString strName;
    this->getName(&strName);
    ZLOGGER.printfln(LOGMODE_INFO, "Unfreeze TModule %s.", strName.c_str());
    ZLOGGER.indent();
    ZLOGGER.unindent();
  }
  return bRetval;
}

/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TModule::getMode() const
{
  TString strAttName = L"Mode";
  unsigned long ulRetval = MODULEMODE_NORMAL;
  long lRet = 0;
  if (this->readAttribute(strAttName, lRet) == RET_NOERROR)
  {
    ulRetval = lRet;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IModule. 
    If no active module is registered and this is not a leaf module, it will
    activate the first module found in the list. This will go on until the 
    leaf is reached or a activemodule is found.
*/
/***************************************************************************/
void MQUALIFIER TModule::activate()
{
  TString strName;
  this->getName(&strName);
  ZLOGGER.printfln(LOGMODE_INFO, "Activate TModule %s.", strName.c_str());
  ZLOGGER.indent();
  
  if (!isLoaded())
  {
    load();
  }
  
  IXObject* pParent = NULL;
  if (this->getParent(&pParent) == RET_NOERROR)
  {
    IModule* pParentModule = NULL;
    if (pParent->askForInterface(INTERFACE_IModule, (IZUnknown**)&pParentModule) == RET_NOERROR)
    {
      pParentModule->registerActiveModule(this);
      pParentModule->release();
    }
    pParent->release();
  }
  m_bActive = true;
  
  //Try to activate a child module
  if (m_pActiveModule == NULL)
  {
    activateFirstChild();
  }
  ZLOGGER.unindent();
}

/***************************************************************************/
/*! This method helps to get the first child object, witch is a module
    and marks this module as activated. This module will register themself
    automatically.
*/
/***************************************************************************/
void TModule::activateFirstChild()
{
  TModuleFunctions<TModule>::activateFirstChild(this, m_bActive);
}

/***************************************************************************/
/*! The deactivation does not remove the ActiveModule Reference of this 
    module. According to the MOM2 specification, this reference is kept.
    See IModule for more information
*/
/***************************************************************************/
void MQUALIFIER TModule::deactivate()
{
  TString strName;
  this->getName(&strName);
  ZLOGGER.printfln(LOGMODE_INFO, "Deactivate TModule %s.", strName.c_str());
  ZLOGGER.indent();
  
  IXObject* pParent = NULL;
  if (this->getParent(&pParent) == RET_NOERROR)
  {
    IModule* pParentModule = NULL;
    if (pParent->askForInterface(INTERFACE_IModule, (IZUnknown**)&pParentModule) == RET_NOERROR)
    {
      pParentModule->unregisterActiveModule(this);
      pParentModule->release();
    }
    pParent->release();
  }
  m_bActive = false;
  ZLOGGER.unindent();
}


/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
bool MQUALIFIER TModule::isActive() const
{
  return m_bActive;
}

/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TModule::getZeusAPI(IZeusAPI** api)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (api == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    *api = m_pZeusAPI;
    m_pZeusAPI->addRef();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TModule::getModule(IString* name, IModule** module)
{
  return TModuleFunctions<TModule>::getModule(this, name, module);
}
    
/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TModule::getName(IString* name) const
{
  return TXObject::getName(name);
}

/***************************************************************************/
/*! See IModule.
    A module is registring themself as active. If this module (parent) is 
    not marked as active itself, iy will to it at this moment. This 
    will go on until the root module or a already active parent is reached.
*/
/***************************************************************************/
unsigned long MQUALIFIER TModule::registerActiveModule(IModule* module)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (module != NULL)
  {
    if (m_pActiveModule != NULL)
    {
      //The module will unregister 
      m_pActiveModule->deactivate();
      
      //////////////////////////////
      //Secure code only. In regular use this will never apply
      if (m_pActiveModule != NULL)
      {
        m_pActiveModule->release();
        m_pActiveModule = NULL;
      }
      //////////////////////////////
    }
    m_pActiveModule = module;
    m_pActiveModule->addRef();
    
    //If this module is not activated -> do it now
    if (!m_bActive)
    {
      this->activate();
    }
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TModule::unregisterActiveModule(IModule* module)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (module != NULL)
  {
    if (m_pActiveModule != NULL)
    {
      m_pActiveModule->release();
      m_pActiveModule = NULL;
      
      //This module stays activated even a child is unregistring themself.
      /////////////////
    }
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TModule::getActiveModule(IModule** module)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (module != NULL)
  {
    if (m_pActiveModule != NULL)
    {
      ulRetval = m_pActiveModule->getActiveModule(module);
    }
    else
    {
      *module = this;
      this->addRef();
      ulRetval = RET_NOERROR;
    }
  }
  else
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IModule
*/
/***************************************************************************/
unsigned long MQUALIFIER TModule::getModuleSession(IModuleSession** session)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (session == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    if (!isLoaded())
    {
      load();
    }
    
    *session = m_pSession;
    if (m_pSession != NULL)
    {
      m_pSession->addRef();
      ulRetval = RET_NOERROR;
    }
  }
  return ulRetval;
}


/***************************************************************************/
/*! See IXLoaderObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TModule::load()
{
  unsigned long ulRetval = TXLoaderObject::load();
  if (ulRetval == RET_NOERROR && m_pSession == NULL)
  {
    TString strLibName;
    if (this->readAttribute(L"ModuleLibrary", strLibName) == RET_NOERROR && strLibName != L"")
    {
      TLibraryManager::getInstance().addCodeModule(strLibName);
      TString strClassName = L"IModuleSession";
      if (TLibraryManager::getInstance().createObject(&strLibName, &strClassName, (IZUnknown**)&m_pSession) == RET_NOERROR)
      {
        ulRetval = RET_NOERROR;
      }
      else
      {
        ulRetval = RET_REQUEST_FAILED;
      }
    }
    
    //No library to load
    else
    {
      ulRetval = RET_NOERROR;
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IXLoaderObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TModule::unload()
{
  unsigned long ulRetval = TXLoaderObject::unload();
  if (m_pSession != NULL)
  {
    m_pSession->release();
    m_pSession = NULL;
  }
  return ulRetval;
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TModule);
  INTERFACE_CAST(IModule, INTERFACE_IModule);
MEMORY_MANAGER_IMPL_PARENT_END(TXLoaderObject);

