/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : GAChromosone                                                  *
 * Package : Genetic                                                       *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:26:39 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: GAChromosone.cpp,v $
 *   Revision 1.1  2005/11/10 10:26:39  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <zeusbase/Math/GAChromosone.h>
#include <zeusbase/System/Time.h>

USING_NAMESPACE_Zeus

/**************************************************************************/
/*! Creates a chromosone
    \param type : Type of the chromosone
    \param kind : How are the new chromosones generated
    \param funct : Customized function to cross the chromosone
*/
/**************************************************************************/
TGAChromosone::TGAChromosone(unsigned long type, TCrossOverKind kind, FCrossFunction funct /*= NULL*/) : TZObject()
{
  m_ulType = type;
  m_eCrossKind = kind;
  m_Funct = funct;
  m_lMultiPointParam = 4;
}

/**************************************************************************/
/*! Destroys the chromosone
*/
/**************************************************************************/
TGAChromosone::~TGAChromosone()
{
  long lCount = m_lstGenes.getCount();
  for (int i = 0; i < lCount; i++)
  {
    m_lstGenes[i]->release();
  }
  m_lstGenes.clear();
}    
    

/**************************************************************************/
/*! Sets the multi point parameter, witch controls how many times a cross 
    over will be performed.
    \param value : new value
*/
/**************************************************************************/
void TGAChromosone::setMultiPointParameter(long value)
{
  m_lMultiPointParam = value;
}

/**************************************************************************/
/*! Adds a ne gene to the chromosone
    \param gene : gene to add
*/
/**************************************************************************/
void TGAChromosone::addGene(IGAGene* gene)
{
  if (gene != NULL)
  {
    gene->addRef();
    m_lstGenes.add(gene);
  }
}

/**************************************************************************/
/*! See IGAChromosone
*/
/**************************************************************************/
unsigned long MQUALIFIER TGAChromosone::getType()
{
  return m_ulType;
}

/**************************************************************************/
/*! See IGAChromosone
*/
/**************************************************************************/
long MQUALIFIER TGAChromosone::getGeneCount()
{
  return m_lstGenes.getCount();
}

/**************************************************************************/
/*! See IGAChromosone
*/
/**************************************************************************/
unsigned long MQUALIFIER TGAChromosone::getGene(long index, IGAGene** gene)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (gene == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    *gene = m_lstGenes[index];
    if (*gene != NULL)
    {
      (*gene)->addRef();
      ulRetval = RET_NOERROR;
    }
  }
  return ulRetval;
}

/**************************************************************************/
/*! See IGAChromosone
*/
/**************************************************************************/
unsigned long MQUALIFIER TGAChromosone::crossOver(IGAChromosone* chrom, IGAChromosone** new_chrom1, IGAChromosone** new_chrom2)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (chrom == NULL || new_chrom1 == NULL || new_chrom2 == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  
  //Can only cross over chromosones of the same type
  else if (chrom->getType() == m_ulType && chrom->getGeneCount() == m_lstGenes.getCount())
  {
    TGAChromosone* pChromosone1 = new TGAChromosone(m_ulType, m_eCrossKind, m_Funct);
    TGAChromosone* pChromosone2 = new TGAChromosone(m_ulType, m_eCrossKind, m_Funct);
    
    switch(m_eCrossKind)
    {
      case ctUniform:
      break;
      
      case ctInterRecombination :
      break;
      
      case ctLineRecombination :
      break;
      
      case ctCustom :
        if (m_Funct != NULL)
        {
          ulRetval = m_Funct(this, chrom, pChromosone1, pChromosone2);
        }
      break;
      
      case ctMultiPoint : //fall through
      default:
      {
        doMultiCrossOver(chrom, pChromosone1, pChromosone2);
      }
      break;
    }
    *new_chrom1 = pChromosone1;
    *new_chrom2 = pChromosone2;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/**************************************************************************/
/*! This does the multi cross over
    \param parent2 : Source of parent 2
    \param chrom : Destination chromosone
*/
/**************************************************************************/
void TGAChromosone::doMultiCrossOver(IGAChromosone* parent2, TGAChromosone* chrom1, TGAChromosone* chrom2)
{
  long lCount = m_lstGenes.getCount();
  if (lCount > 1)
  {
    if (m_lMultiPointParam <= 0)
    {
      m_lMultiPointParam = 1;
    }
    else if (m_lMultiPointParam > lCount-1)
    {
      m_lMultiPointParam = lCount-1;
    }
    chrom1->setMultiPointParameter(m_lMultiPointParam);
    chrom2->setMultiPointParameter(m_lMultiPointParam);
    
    //Generate random cross over points and add them to a list
    // sorted by value
    TArrayList<unsigned long> lstParams(m_lMultiPointParam + 3);
    for(int i = 0; i < m_lMultiPointParam; i++)
    {
      unsigned long ulVal = TTime::random(0, lCount-1);
      bool bInserted = false;
      
      for(int j = 0; !bInserted && j < lstParams.getCount(); i++)
      {
        if (lstParams[j] > ulVal)
        {
          lstParams.insert(j, ulVal);
          bInserted = true;
        }
        
        // Bad luck: Never mind, we continue here
        else if (lstParams[j] == ulVal)
        {
          bInserted = true;
        }
      }
      if (!bInserted)
      {
        lstParams.add(ulVal);
      }
    }
    
    //Do the cross over
    bool bParentSwitch = false;
    for(unsigned long ul = 0; ul < (unsigned long)lCount; ul++)
    {
      //Toggle the parent source
      if (ul == lstParams[0])
      {
        bParentSwitch = !bParentSwitch;
      }
      
      IGAGene* pGene1 = NULL;
      IGAGene* pGene2 = NULL;
      if (bParentSwitch)
      {
        this->getGene(ul, &pGene1);
        parent2->getGene(ul, &pGene2);
      }
      else
      {
        this->getGene(ul, &pGene2);
        parent2->getGene(ul, &pGene1);
      }
      
      chrom1->m_lstGenes.add(pGene1);
      chrom2->m_lstGenes.add(pGene2);
      
      if (pGene1 != NULL)
      {
        pGene1->release();
      }
      if (pGene2 != NULL)
      {
        pGene2->release();
      }
    }
  }
  
  //Only one gene -> no cross over available
  else if (lCount == 1)
  {
    IGAGene* pGene = m_lstGenes[0];
    pGene->addRef();
    chrom1->m_lstGenes.add(pGene);
    chrom2->m_lstGenes.add(pGene);
  }
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TGAChromosone);
  INTERFACE_CAST(IGAChromosone, INTERFACE_IGAChromosone);
MEMORY_MANAGER_IMPL_END;
 
