/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : GAPopulation                                                  *
 * Package : Genetic                                                       *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:26:39 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: GAPopulation.cpp,v $
 *   Revision 1.1  2005/11/10 10:26:39  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

 
#include <zeusbase/Math/GAPopulation.h>
#include <zeusbase/System/Time.h>
#include <stdio.h>

USING_NAMESPACE_Zeus


/**************************************************************************/
/*! Creates a population 
*/
/**************************************************************************/
TGAPopulation::TGAPopulation(FCreateIndividual factory, 
                             TGACopyMethod method /*= cmKeepBest*/, 
                             TGACrossMethod cross /*= cmCrossBest*/,
                             TGAFitnessMethod fitness_method /*= fmBiggerBetter*/, 
                             double cross_amount /*= 10.0*/)
{
  m_Factory = factory;
  m_eCopyMethod = method;
  m_eCrossMethod = cross;
  m_eFitnessMethod = fitness_method;
  if (method == cmReplaceAll)
  {
    m_dCrossAmount = 100.0;
  }
  else
  {
    if (cross_amount > 100.0)
    {
      m_dCrossAmount = 100.0;
    }
    else if (cross_amount < 0.0)
    {
      m_dCrossAmount = 0.0;
    }
    else
    {
      m_dCrossAmount = cross_amount;
    }
  }
}

/**************************************************************************/
/*! Destroys the population
*/
/**************************************************************************/
TGAPopulation::~TGAPopulation()
{
  clearPopulation();
}
    
/**************************************************************************/
/*! Remove all Individuals
*/
/**************************************************************************/
void TGAPopulation::clearPopulation()
{
  long lCount = m_lstIndividuals.getCount();
  for (int i = 0; i < lCount; i++)
  {
    m_lstIndividuals[i]->release();
  }
  m_lstIndividuals.clear();
}

/**************************************************************************/
/*! See IGAPopulation
*/
/**************************************************************************/
long MQUALIFIER TGAPopulation::getCount()
{
  return m_lstIndividuals.getCount();
}

/**************************************************************************/
/*! Creates a new population
*/
/**************************************************************************/
void TGAPopulation::createPopulation(long count)
{
  clearPopulation();
  if (m_Factory != NULL)
  {
    for(int i = 0; i < count; i++)
    {
      IGAIndividual* pInd = NULL;
      if (m_Factory(&pInd, true) == RET_NOERROR)
      {
        m_lstIndividuals.add(pInd);
      }
    }
  }
}


/**************************************************************************/
/*! See IGAPopulation
*/
/**************************************************************************/
unsigned long MQUALIFIER TGAPopulation::getIndividual(long index, IGAIndividual** individual)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (individual == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    *individual = m_lstIndividuals[index];
    if (*individual != NULL)
    {
      (*individual)->addRef();
      ulRetval = RET_NOERROR;
    }
  }
  return ulRetval;  
}

/**************************************************************************/
/*! See IGAPopulation
*/
/**************************************************************************/
unsigned long MQUALIFIER TGAPopulation::getBestIndividual(IGAIndividual** individual)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  long lCount = m_lstIndividuals.getCount();
  if (individual == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (lCount > 0)
  {
    double dFitness = m_lstIndividuals[0]->getFitness();
    long lIndex = 0;
       
    for (int i = 0; i < lCount; i++)
    {
      switch(m_eFitnessMethod)
      {
        case fmSmallerBetter:
          if (m_lstIndividuals[i]->getFitness() < dFitness)
          {
            dFitness = m_lstIndividuals[i]->getFitness();
            lIndex = i;
          }
        break;
        
        case fmBiggerBetter: //fall through
        default:
          if (m_lstIndividuals[i]->getFitness() > dFitness)
          {
            dFitness = m_lstIndividuals[i]->getFitness();
            lIndex = i;
          }
        break;
      }
    }
    
    *individual = m_lstIndividuals[lIndex];
    if (*individual != NULL)
    {
      (*individual)->addRef();
      ulRetval = RET_NOERROR;
    }
  }

  return ulRetval;  
}

/**************************************************************************/
/*! See IGAPopulation
*/
/**************************************************************************/
unsigned long MQUALIFIER TGAPopulation::makeNextGeneration()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  long lCount = m_lstIndividuals.getCount();
  if (lCount > 1)
  {
    TArrayList<IGAIndividual*> lstNewGeneration(lCount+10);
    switch(m_eCrossMethod)
    {
      case cmCrossOnlyBest:
        ulRetval = RET_METHOD_NOT_IMPL; 
      break;
      
      case cmCrossBestPrior : 
      default:
        crossIndividualsPrior(&lstNewGeneration);
        ulRetval = RET_NOERROR;
      break;
    }
    
    //Release list of new individuals
    long lNewCount = lstNewGeneration.getCount();
    for(int i = 0; i < lNewCount; i++)
    {
      lstNewGeneration[i]->release();
    }
    lstNewGeneration.clear();
  }
  return ulRetval; 
}
 
/**************************************************************************/
/*! See IGAPopulation
*/
/**************************************************************************/
long MQUALIFIER TGAPopulation::makeGenerations(long abort)
{
  long lRetval = 0;
  long lAbortRounds = 0;
  double dBestFitness = 0.0;
  
  IGAIndividual* pInd = NULL;
  if (this->getBestIndividual(&pInd) == RET_NOERROR)
  {
    dBestFitness = pInd->getFitness();
    pInd->release();
  }
  
  //Generate populations
  while(lAbortRounds <= abort)
  {
    makeNextGeneration();
    double dFitness = 0.0;
    if (this->getBestIndividual(&pInd) == RET_NOERROR)
    {
      dFitness = pInd->getFitness();
      pInd->release();
    }
    
    if (dFitness == dBestFitness)
    {
      lAbortRounds++;
    }
    else
    {
      switch(m_eFitnessMethod)
      {
        case fmSmallerBetter:
          if (dFitness < dBestFitness)
          {
            dBestFitness = dFitness;
          }
        break;
        
        case fmBiggerBetter: //fall through
        default:
          if (dFitness > dBestFitness)
          {
            dBestFitness = dFitness;
          }
        break;
      }
      lAbortRounds = 0;
    }
    lRetval++;
  }
  
  return lRetval;
}

   
/**************************************************************************/
/*! Find an individual within this fitness range. 
    \param list : List of all individuals
    \param fitness_range : Random fitness range 
    \return Index of individual
*/
/**************************************************************************/
unsigned long TGAPopulation::findFromFitnessRange(TArrayList<RSelection>& list, double fitness_range)
{
  long lRetval = 0;
  bool bFound = false;
  long lCount = list.getCount();
  while(!bFound && lRetval < lCount-1)
  {
    RSelection& Sel = list[lRetval];
    if (Sel.dFitnessSum > fitness_range)
    {
      lRetval--;
      bFound = true;
    }
    else
    {
      lRetval++;
    }
  }
  
  if (lRetval < 0)
  {
    lRetval = 0;
  }
  
  return lRetval;
}


/**************************************************************************/
/*! This method crosses the parents. The one with higher fitness are 
    priorized to be crossed
*/
/**************************************************************************/
void TGAPopulation::crossIndividualsPrior(TArrayList<IGAIndividual*>* new_generation)
{
  //---------------------------------------------------------------
  //Add all individuals to the list
  // The fitness sum is used as a range. If the fitness is better
  // the range should be bigger, so that we have a better chance to
  // find those individuals.
  long lCount = m_lstIndividuals.getCount();
  TArrayList<RSelection> lstSelection(lCount+10);
  double dActFitnessSum = 0.0;
  for(int i = 0; i < lCount; i++)
  {      
    RSelection Selection;
    Selection.pIndividual = m_lstIndividuals[i];
    Selection.pIndividual->addRef();
    Selection.dFitness = Selection.pIndividual->getFitness();
    
    switch(m_eFitnessMethod)
    {
      case fmSmallerBetter:
        Selection.dFitnessSum = dActFitnessSum;
        if (Selection.dFitness == 0.0)
        {
          Selection.dFitness = 0.000001;
        }
        dActFitnessSum += (1.0/(double)Selection.dFitness);
      break;
      
      case fmBiggerBetter: //fall through
      default:
        Selection.dFitnessSum = dActFitnessSum;
        dActFitnessSum += Selection.dFitness; 
      break;
    }
    lstSelection.add(Selection);
  }  
    
  //---------------------------------------------------------------
  //Get the amout of new crosses
  // Divided by 2 because always two children are generated in one step
  long lCrossCount = (long)(m_dCrossAmount/100.0 * lCount/2.0);
  if (lCrossCount < 1)
  {
    lCrossCount = 1;
  }
  if (lCrossCount == lCount)
  {
    lCrossCount = lCount - 1;
  }
//  printf("%f -------------------\n", dActFitnessSum);
  for(int i = 0; i < lCrossCount; i++)
  {
    unsigned long ulSeed1 = TTime::random(0, 1000000);
    unsigned long ulSeed2 = TTime::random(0, 1000000);
    while(ulSeed1 == ulSeed2)
    {
      ulSeed2 = TTime::random(0, 1000000);
    }
    
    double dFitnessToFind1 = ulSeed1/1000000.0 * dActFitnessSum;
    double dFitnessToFind2 = ulSeed2/1000000.0 * dActFitnessSum;
    unsigned long ulIndex1 = findFromFitnessRange(lstSelection, dFitnessToFind1);
    unsigned long ulIndex2 = findFromFitnessRange(lstSelection, dFitnessToFind2);
      
/*    printf("Item[%d].Fitness=%f, Range=%f, WishRange=%f ::with Item[%d].Fitness=%f Range=%f, WishRange=%f\n", 
                                    (int)ulIndex1,
                                    lstSelection[ulIndex1].dFitness,
                                    lstSelection[ulIndex1].dFitnessSum,
                                    dFitnessToFind1, 
                                    (int)ulIndex2, 
                                    lstSelection[ulIndex2].dFitness,
                                    lstSelection[ulIndex2].dFitnessSum,
                                    dFitnessToFind2 
                                    );
*/    
    this->cross(lstSelection[ulIndex1].pIndividual, lstSelection[ulIndex2].pIndividual,new_generation);
    lstSelection[ulIndex1].bCrossed = true;  
    lstSelection[ulIndex2].bCrossed = true;  
  }
//  printf("-------------------\n");
  
  //---------------------------------------------------------------
  //Mix the new generation with the existing generation
  copyNewGeneration(lstSelection, new_generation);
 
  //---------------------------------------------------------------
  //Release the selection list
  long lSelCount = lstSelection.getCount();
  for(int i = 0; i < lSelCount; i++)
  {
    lstSelection[i].pIndividual->release();
  }
  lstSelection.clear();
  //---------------------------------------------------------------
}


/**************************************************************************/
/*! This method crosses two individuals and adds the children to the 
    list of new generated individuals
    \param parent1 : 1st parent
    \param parent2 : 2nd parent
    \param new_generation : list of new generated individuals
*/
/**************************************************************************/
void TGAPopulation::cross(IGAIndividual* parent1, IGAIndividual* parent2, TArrayList<IGAIndividual*>* new_generation)
{
  long lCount = parent1->getChromosoneCount();
  if (lCount == parent2->getChromosoneCount() && m_Factory != NULL)
  {
    //Create new instances of concrete children
    IGAIndividual* pChild1 = NULL;  
    IGAIndividual* pChild2 = NULL;  
    m_Factory(&pChild1, false);
    m_Factory(&pChild2, false);
    
    if (pChild1 != NULL && pChild2 != NULL)
    {
      //Cross over of all chromosones. Add the new chromosones to the children
      for(int i = 0; i < lCount; i++)
      {
        IGAChromosone* pChromosone1 = NULL;
        IGAChromosone* pChromosone2 = NULL;
        IGAChromosone* pNewChromosone1 = NULL;
        IGAChromosone* pNewChromosone2 = NULL;
        
        parent1->getChromosone(i, &pChromosone1);
        parent2->getChromosone(i, &pChromosone2);
        
        //Cross over algorithme
        if (pChromosone1 != NULL && pChromosone1->crossOver(pChromosone2, &pNewChromosone1, &pNewChromosone2) == RET_NOERROR)
        {
          pChild1->addChromosone(pNewChromosone1);
          pChild2->addChromosone(pNewChromosone2);

          pNewChromosone1->release();
          pNewChromosone2->release();
        }
        if (pChromosone1 != NULL)
        {
          pChromosone1->release();
        }
        if (pChromosone2 != NULL)
        {
          pChromosone2->release();
        }
      }
      new_generation->add(pChild1);
      new_generation->add(pChild2);
    }
  }
}
   
/**************************************************************************/
/*! Sorts the list using the bubble sort. The best are at front.
*/
/**************************************************************************/
void TGAPopulation::sortListFromFitness(TArrayList<RSelection>& list)
{
  bool bChanged = true;
  long lCount = list.getCount();
  while(bChanged)
  {
    bChanged = false;
    for(int i = 0; i < lCount-1; i++)
    {
      switch(m_eFitnessMethod)
      {
        case fmSmallerBetter:
          if (list[i].dFitness > list[i+1].dFitness)
          {
            RSelection Temp = list[i];
            list[i] = list[i+1];
            list[i+1] = Temp;
            bChanged = true;
//            printf("Swapped: %d, %d\n", i, i+1);
          }
        break;
        
        case fmBiggerBetter: //fall through
        default:
          if (list[i].dFitness < list[i+1].dFitness)
          {
            RSelection Temp = list[i];
            list[i] = list[i+1];
            list[i+1] = Temp;
            bChanged = true;
//            printf("Swapped: %d, %d\n", i, i+1);
          }
        break;
      }
    }
  }
/*  printf("-------------------\n");
  for(int i = 0; i < lCount; i++)
  {
    printf("Item[%d].Fitness=%f\n", i, list[i].dFitness);
  }
  printf("-------------------\n");
*/  
}
    
/**************************************************************************/
/*! This method copys the new generation to the old generation
*/
/**************************************************************************/
void TGAPopulation::copyNewGeneration(TArrayList<RSelection>& old_generation, TArrayList<IGAIndividual*>* new_generation)
{
  long lCount = m_lstIndividuals.getCount();
  long lChildCount = new_generation->getCount();
  //Clear existing population
  clearPopulation();
      
  //Add new children to the new generation
  for(int i = 0; i < lChildCount; i++)
  {
    IGAIndividual* pItem = new_generation->getItem(i);
    m_lstIndividuals.add(pItem);
    pItem->addRef();
  }
  
  //mix old generation with the new generation
  switch(m_eCopyMethod)
  {
    case cmReplaceCrossed : //Caution: the set can increese
    {
      //Removed crossed items
      long lOldCount = old_generation.getCount();
      for(int i = 0; i < lOldCount; i++)
      { 
        if (old_generation.getItem(i).bCrossed)
        {
          old_generation.deleteItem(i);
          i--;
        }
      }
      
      //Add all parents whitch are not crossed to the set.
      lOldCount = old_generation.getCount();
      for(int i = 0; i < lOldCount; i++)
      { 
        IGAIndividual* pItem = old_generation.getItem(i).pIndividual;
        m_lstIndividuals.add(pItem);
        pItem->addRef();
      }      
    }
    break;
    
    case cmReplaceAll :
      //To nothing 
    break;
    
    case cmKeepBestParents: //fall through
    default:
      //Sort the parent list. The best parents will be at front
      sortListFromFitness(old_generation);
      
      //Add best parents to the generation
      for(int i = 0; i < lCount - lChildCount; i++)
      {
        IGAIndividual* pItem = old_generation.getItem(i).pIndividual;
        if (pItem != NULL)
        {
          m_lstIndividuals.add(pItem);
          pItem->addRef();
        }
      }
    break;
  };
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TGAPopulation);
  INTERFACE_CAST(IGAPopulation, INTERFACE_IGAPopulation);
MEMORY_MANAGER_IMPL_END;
 
 
