/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : GAPopulation                                                  *
 * Package : Genetic                                                       *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:26:39 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: GAPopulation.h,v $
 *   Revision 1.1  2005/11/10 10:26:39  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GAPopulationH
#define GAPopulationH

#include <zeusbase/Math/Interfaces/IGAPopulation.hpp>
#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/ArrayList.hpp>

BEGIN_NAMESPACE_Zeus
 

/***************************************************************************/
/*! This represents a population of individuals
*/
/***************************************************************************/
class TGAPopulation : public TZObject, public IGAPopulation
{
  public:
    /***********************************************************************/
    /*! This enumration specifies the method how to copy or create a new
        population
    */
    /***********************************************************************/
    enum TGACopyMethod 
    {
      cmReplaceCrossed,  /*!< Replace all crossed individuals */
      cmReplaceAll,      /*!< Replace all individuals */
      cmKeepBestParents  /*!< Keep the best individuals of the existing population */
    };
    
    /***********************************************************************/
    /*! Type to specify how to cross the individuals
    */
    /***********************************************************************/
    enum TGACrossMethod
    {
      cmCrossBestPrior,  /*!< Cross the individuals giving the best a better chance to get crossed */
      cmCrossOnlyBest    /*!< Crosses only the best individuals */
    };
    
    /***********************************************************************/
    /*! Type to specify how to use the fitness value
    */
    /***********************************************************************/
    enum TGAFitnessMethod
    {
      fmBiggerBetter,  /*!< Bigger fitness is better*/
      fmSmallerBetter  /*1< Lower fitness is better*/
    };
    
    /***********************************************************************/
    /*! This type specifies a function, to create individuals
        \param ind : Return value of a created individual
        \param create_chromosones : If flag is set -> create chromosones
        \retval RET_NOERROR : Individual created
    */
    /***********************************************************************/
    typedef unsigned long (*FCreateIndividual)(IGAIndividual** ind, bool create_chromosones);
    
    TGAPopulation(FCreateIndividual factory, 
                  TGACopyMethod method = cmKeepBestParents, 
                  TGACrossMethod cross = cmCrossBestPrior, 
                  TGAFitnessMethod fitness_method= fmBiggerBetter,
                  double cross_amount = 10.0);
    
    void createPopulation(long count);
        
    //Methods of IGAPopulation
    virtual long MQUALIFIER getCount();
    virtual unsigned long MQUALIFIER getIndividual(long index, IGAIndividual** individual);
    virtual unsigned long MQUALIFIER getBestIndividual(IGAIndividual** individual);
    virtual unsigned long MQUALIFIER makeNextGeneration();
    virtual long MQUALIFIER makeGenerations(long abort);
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    virtual ~TGAPopulation();
    
  private:
    ///List of Individuals
    TArrayList<IGAIndividual*> m_lstIndividuals;
    ///How many individuals are crossed [in %]
    double m_dCrossAmount;
    ///Method how to compose a new generation
    TGACopyMethod m_eCopyMethod;
    ///Method witch individual can be crossed
    TGACrossMethod m_eCrossMethod;
    ///Method of fitness
    TGAFitnessMethod m_eFitnessMethod;
    ///Factory Method
    FCreateIndividual m_Factory;
      
    /*******************************************************************/
    /* Struct is used to generate a selection list out of the
       individuals
    */
    /*******************************************************************/
    struct RSelection
    {
      double dFitness;
      double dFitnessSum;
      IGAIndividual* pIndividual;
      bool bCrossed;
            
      RSelection()
      {
        dFitness = 0.0;
        dFitnessSum = 0.0;
        bCrossed = false;
        pIndividual = NULL;
      }
      
      bool operator==(const RSelection& inpar) const
      {
        return (dFitnessSum == inpar.dFitnessSum &&
                dFitness == inpar.dFitness &&
                pIndividual == inpar.pIndividual &&
                bCrossed == inpar.bCrossed);
      }
    };
    
    
    void clearPopulation();
    void crossIndividualsPrior(TArrayList<IGAIndividual*>* new_generation);
    void cross(IGAIndividual* parent1, IGAIndividual* parent2, TArrayList<IGAIndividual*>* new_generation);
    
    unsigned long findFromFitnessRange(TArrayList<RSelection>& list, double fitness_range);
    void sortListFromFitness(TArrayList<RSelection>& list);
    void copyNewGeneration(TArrayList<RSelection>& old_generation, TArrayList<IGAIndividual*>* new_generation);
};

END_NAMESPACE_Zeus

#endif
