/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XMLMessage                                                    *
 * Package : Messaging                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 08:59:56 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XMLMessage.cpp,v $
 *   Revision 1.3  2005/12/14 08:59:56  bhadorn
 *   createObject was not declared with MQUALIFIER
 *
 *   Revision 1.2  2005/11/25 09:30:12  bhadorn
 *   Eliminate compiler warnings
 *
 *   Revision 1.1  2005/11/10 10:57:11  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/Messaging/XMLMessage.h>
#include <zeusbase/System/ByteArrayOutputStream.h>
#include <zeusbase/System/CriticalSection.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a new simple xml message object
    \param name : Name of the message
    \param targetcell : Target cell system
    \param targetpipe : Target pipe name
    \param message : return parameter of the created message
    \retval RET_NOERROR : Message created
    \retval RET_INVALID_PARAMETER : Invalid parameter received
    \retval RET_REQUEST_FAILED : Could not create message
*/
/***************************************************************************/
unsigned long TXMLMessage::createMessage(TString name, TString targetcell, TString targetpipe, IXMLMessage** message)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (message == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strXMLStream = TString(L"<?xml version=\"1.0\"?>\n<") +name+ TString(L" ClassName=\"TXMLMessage\"/>");
    IXObject* pObj = NULL;
    if (TXObjectFactory::getInstance().createObjectFromStream(&strXMLStream, (IXObject**)&pObj, false) == RET_NOERROR)
    {
      ulRetval = pObj->askForInterface(INTERFACE_IXMLMessage, (IZUnknown**)message);
      
      if (ulRetval == RET_NOERROR)
      {
        (*message)->setTargetCellName(&targetcell);
        (*message)->setTargetPipeName(&targetpipe);
      }
      pObj->release();
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! Factory function to create an object out of an stream. This stream 
    is a string representing an XML data tree.
    \param objectID : ID of the Object
    \param stream : Input Data strem
    \param obj : Return parameter of the created object
    \retval RET_NOERROR : Object created
    \retval RET_INVALID_PARAMETER : Invalid parameter received
    \retval RET_REQUEST_FAILED : Could not create object (invalid stream)
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLMessage::createObject(unsigned long /*objectID*/, IInputStream* stream, IZUnknown** obj)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (obj == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  
  //Create an empty object
  else if (stream == NULL)
  {
    IXMLMessage* pXMLMessage = NULL;
    if (TXMLMessage::createMessage(L"", L"", L"", &pXMLMessage) == RET_NOERROR)
    {
      ulRetval = pXMLMessage->askForInterface(INTERFACE_ISerializable, (IZUnknown**)obj);
      pXMLMessage->release();
    }
  }
  
  //Create an object out of a stream
  else
  {
    TString strXMLData;
    stream->readString(&strXMLData);
    ulRetval = TXObjectFactory::getInstance().createObjectFromStream(&strXMLData, (IXObject**)obj, false);
    
/*    if (ulRetval == RET_NOERROR)
    {
      ulRetval = pObject->askForInterface(INTERFACE_ISerializable, (IZUnknown**)obj);
      pObject->release();
    }*/
  }
  return ulRetval;
}


/***************************************************************************/
/*! Creates a new xml message object
*/
/***************************************************************************/
TXMLMessage::TXMLMessage(IXMLNode* main_node, NAMESPACE_Zeus::IXMLDocument* docu) : TXRootObject(main_node,docu)
{
  m_pLock = new TCriticalSection();
}


/***************************************************************************/
/*! Destroys the message object
*/
/***************************************************************************/
TXMLMessage::~TXMLMessage()
{
  m_pLock->release();
}

/***************************************************************************/
/*! See IXMLMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLMessage::getDataNode(IXMLNode** node)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (node == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_pNode != NULL)
  {
    TString strNodeName = L"DataNode";
    m_pNode->resetNodeIterator(&strNodeName);
    if (m_pNode->getNextNode(node) != RET_NOERROR)
    {
      TString strURI;
      ulRetval = m_pNode->createNewChildNode(TYPE_NODE_ELEMENT, &strNodeName, &strURI, node);
    }
    else
    {
      ulRetval = RET_NOERROR;
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLMessage::getInterfaceID() const
{
  return INTERFACE_IXMLMessage;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLMessage::getMessageName(IString* name) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assign(this->getObjName().c_bstr());
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
long MQUALIFIER TXMLMessage::getNumber() const
{ 
  long lRetval = 0;
  TString strAttName = L"Number";
  TString strRet;
  if (readAttribute(strAttName, strRet) == RET_NOERROR)
  {
    lRetval = strRet.toLong();
  }
  return lRetval;
}


/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLMessage::getSenderPipeName(IString* name) const
{
  unsigned long ulRetval = RET_INTERNAL_ERROR;
  TString strAttName = L"SenderPipeName";
  TString strRet;
  if (name == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (readAttribute(strAttName, strRet) == RET_NOERROR)
  {
    if (strRet.equals(L""))
    {
      name->assign(PIPENAME_STDIN);
    }
    else
    {
      name->assignStr(&strRet);
    }
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLMessage::getSenderCellName(IString* name) const
{
  unsigned long ulRetval = RET_INTERNAL_ERROR;
  TString strAttName = L"SenderCellName";
  TString strRet;
  if (name == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (readAttribute(strAttName, strRet) == RET_NOERROR)
  {
    name->assignStr(&strRet);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLMessage::getTargetPipeName(IString* name) const
{
  unsigned long ulRetval = RET_INTERNAL_ERROR;
  TString strAttName = L"TargetPipeName";
  TString strRet;
  if (name == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (readAttribute(strAttName, strRet) == RET_NOERROR)
  {
    name->assignStr(&strRet);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLMessage::getTargetCellName(IString* name) const
{
  unsigned long ulRetval = RET_INTERNAL_ERROR;
  TString strAttName = L"TargetCellName";
  TString strRet;
  if (name == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (readAttribute(strAttName, strRet) == RET_NOERROR)
  {
    name->assignStr(&strRet);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
unsigned long MQUALIFIER TXMLMessage::setSenderCellName(IString* name)
{
  unsigned long ulRetval = RET_INTERNAL_ERROR;
  if (name == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strAttName = L"SenderCellName";
    TString strAttValue = name;
    ulRetval = storeAttribute(strAttName, strAttValue);
  }

  return ulRetval;
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TXMLMessage::setSenderPipeName(const IString* name)
{
  TString strAttName = L"SenderPipeName";
  storeAttribute(strAttName, TString(name));
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TXMLMessage::setTargetPipeName(const IString* name)
{
  TString strAttName = L"TargetPipeName";
  storeAttribute(strAttName, TString(name));
}

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TXMLMessage::setTargetCellName(const IString* name)
{
  TString strAttName = L"TargetCellName";
  storeAttribute(strAttName, TString(name));
}

/***************************************************************************/
/*! Do not use. The name must be given by the static factory method.
*/
/***************************************************************************/
void MQUALIFIER TXMLMessage::setMessageName(const IString* /*name*/)
{ /**/ }

/***************************************************************************/
/*! see IMessage
*/
/***************************************************************************/
void MQUALIFIER TXMLMessage::setNumber(long value)
{
  TString strAttName = L"Number";
  storeAttribute(strAttName, value);
}

/**************************************************************************/
/*! See ISerializable
*/
/**************************************************************************/
unsigned long MQUALIFIER TXMLMessage::serialize(IByteArray* stream) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (stream != NULL)
  {
    TByteArrayOutputStream* pStreamer = new TByteArrayOutputStream();
    TString strData = this->getXMLStream();
    TString strClassName = L"TXMLMessage";
    pStreamer->writeLong(SERIALOBJECT_HEADER);
    pStreamer->writeLong(m_ulID);
    pStreamer->writeString(&strClassName);
    pStreamer->writeString(&strData);
    pStreamer->popData(stream);
    pStreamer->release();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/**************************************************************************/
/*! See ISerializable
*/
/**************************************************************************/
unsigned long MQUALIFIER TXMLMessage::getObjectID() const
{
  return (unsigned long)this;
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL_LOCK(TXMLMessage, m_pLock);
  INTERFACE_CAST(IMessage, INTERFACE_IMessage);
  INTERFACE_CAST(IXMLMessage, INTERFACE_IXMLMessage);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
MEMORY_MANAGER_IMPL_PARENT_END(TXRootObject);

