/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : AbstractSkel                                                  *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:27:33 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: AbstractSkel.cpp,v $
 *   Revision 1.3  2005/11/25 09:27:33  bhadorn
 *   Renamed start method to startSkeleton. Using readArray() and writeArray() to read and write to socket.
 *
 *   Revision 1.2  2005/11/16 11:51:27  bhadorn
 *   Dynamically removing wasted client handler threads. Dynamic binding of skeleton server
 *
 *   Revision 1.1  2005/11/10 10:39:16  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/Remote/AbstractSkel.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/System/Protocols/CCTPRequest.h>
#include <zeusbase/System/Protocols/CCTPResponse.h>
#include <zeusbase/System/ZObjectFactory.h>
#include <zeusbase/System/NetworkInterface.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a abstract skeleton object
*/
/***************************************************************************/
TAbstractSkel::TAbstractSkel() : TThread()
{
  m_pSocket = NULL;
  m_bCleanFlag = false;
}

/***************************************************************************/
/*! Destroys the abstract skeleton
*/
/***************************************************************************/
TAbstractSkel::~TAbstractSkel()
{
  if (m_pSocket != NULL)
  {
    m_pSocket->release();
    m_pSocket = NULL;
  }
}

 
/***************************************************************************/
/*! Kills the server thread
*/
/***************************************************************************/
void TAbstractSkel::kill()
{
  ZLOGGER.printfln(LOGMODE_INFO, "Try to stop skeleton server...");
  ZLOGGER.indent();
  TThread::kill();
  ZLOGGER.unindent();
  ZLOGGER.printfln(LOGMODE_INFO, "...done");
}

/***************************************************************************/
/*! Starts the skeleton binding the server socket to a ip address. 
    This method starts the thread after successfully binding the socket.
    \param address : in out parameter for IP address. if the dynamic flag is
                     set this parameter is an out parameter otherwise the
                     address given is used for connecting
    \param port : in out parameter for tcp port. If the dynamic falg is set
                  this parameter is the out parameter otherwise the port is
                  used to connect
    \param dynamic : If this flag is set the socket is bound dynamically to
                  a network interface and an unused tcp port.
    \retval RET_NOERROR : Server socket bound and thread started
    \retval RET_REQUEST_FAILED : Could not bind socket. The server thread 
                  hasn't be started.
*/
/***************************************************************************/
unsigned long TAbstractSkel::startSkeleton(TString& address, unsigned long& port, bool dynamic)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (dynamic)
  {
    TNetworkInterface* pInterface = NULL;
    TString strName = NET_Iface_1;
    if (TNetworkInterface::getInterfaceByName(&strName, &pInterface) == RET_NOERROR)
    {
      address = pInterface->getAddress();
      pInterface->release();
    }
    else
    {
      address = L"127.0.0.1";
    }
    port = 0;
  }
  
  m_pSocket = new TServerSocket(address, port);
  m_pSocket->setReUsable();
  m_pSocket->setBlockable(false);
  if (m_pSocket->bind(dynamic) == RET_NOERROR)
  {
    port = m_pSocket->getPort();
    ZLOGGER.printfln(LOGMODE_INFO, "Skeleton server at [%s:%d]", address.c_str(), port);
    TThread::start();
    ulRetval = RET_NOERROR;
  }
  else
  {
    port = m_pSocket->getPort();
    ZLOGGER.printfln(LOGMODE_ERROR, "Could not connect skeleton server [%s:%d]", address.c_str(), port);
  }
  return ulRetval;
}


/***************************************************************************/
/*! Executes the server thread. Listening for incomming reguests
*/
/***************************************************************************/
void TAbstractSkel::execute()
{
  if(m_pSocket != NULL)
  {
    while(!this->isInterrupted())
    {
      TSocket* pClient = NULL;
      if (m_pSocket->accept(&pClient) == RET_NOERROR)
      {
        ZLOGGER.println(LOGMODE_INFO, "Skel: Client connected to skeleton.");
        startNewHandler(pClient);
        pClient->release();
      }
      
      //Clean wasted client handlers
      if (m_bCleanFlag)
      {
        m_bCleanFlag = false;
        IListIterator<TThread*>* pIt = m_lstClients.getIterator();
        while(pIt->hasNextItem())
        {
          TThread* pThread = pIt->getNextItem();
          if (pThread->isDead())
          {
            pThread->release();
            m_lstClients.remove(pThread);
            ZLOGGER.printfln(LOGMODE_INFO, "Skel: Client handler removed.");
          }
        }
        m_lstClients.releaseIterator(pIt);
      }
      TTime::usleep(1000);
    }
    ZLOGGER.printfln(LOGMODE_INFO, "Skel: Skeleton server closed.");
    
    //Kill all client threads
    while(m_lstClients.getCount() > 0)
    {
      m_lstClients[0]->kill();
      m_lstClients[0]->release();
      m_lstClients.deleteItem(0);
      ZLOGGER.printfln(LOGMODE_INFO, "Skel: Client handler removed.");
    }
  }
}
    
/***************************************************************************/
/*! Each terminating client should call this method. If the clean flag is set
    the skeleton will remove all wasted client handler from its list.
*/
/***************************************************************************/
void TAbstractSkel::setCleanFlag()
{
  m_bCleanFlag = true;
}

/***************************************************************************/
/*! Internal method for dispatching using the remote method call 
    technology.
*/
/***************************************************************************/
void TAbstractSkel::dispatch1(const TCCTPRequest& request, TCCTPResponse& response)
{
  TString strResponseData;
  unsigned long ulResponseStatus;
  ZLOGGER.println(LOGMODE_DEBUG, "Skel: Request of client received");
  TByteArray aMethodStream = request.getData();
  IRemoteMethodCall* pMethodCall = NULL;
  unsigned long ulRetval = TZObjectFactory::getInstance().createObjectFromStream(&aMethodStream, (ISerializable**)&pMethodCall);
  
  switch(ulRetval)
  {
    //Method call objetc created -> dispatch command
    case RET_NOERROR :
    {
      TString strMethodName;
      pMethodCall->getMethodName(&strMethodName);
      ZLOGGER.println(LOGMODE_DEBUG, "Skel: Method call created [%s]. Dispatching...", strMethodName.c_str());
      IRemoteMethodCall* pReturnMethodCall = NULL;
      
      //If dispatch is ok ->send remote return method
      if (dispatchCommand(pMethodCall, &pReturnMethodCall) == RET_NOERROR)
      {
        ZLOGGER.println(LOGMODE_DEBUG, "Skel: Dispatching ok.");
        TByteArray aResponseData;
        pReturnMethodCall->serialize(&aResponseData);
        response.setData(aResponseData);
        ulResponseStatus = CCTP_STATUS_OK;
        pReturnMethodCall->release();
      }
      //Failed to dispatch command
      else
      {
        ulResponseStatus = CCTP_STATUS_METHODFAILED;
        ZLOGGER.println(LOGMODE_DEBUG, "Skel: Dispatching failed. [Unknown method call]");
        strResponseData = L"Invalid method call.";
      }
      pMethodCall->release();
      pMethodCall = NULL;
    }
    break;
    
    //Class not registered
    case RET_UNREGISTERED_CLASS : 
    {
      ulResponseStatus = CCTP_STATUS_BADREQUEST;
      strResponseData = L"Unknown class";
    }
    break;
    
    //Invalid stream received
    case RET_INVALID_DATA : 
    {
      ulResponseStatus = CCTP_STATUS_BADREQUEST;
      strResponseData = L"Could not parse Stream";
    }
    break;
    
    case RET_REQUEST_FAILED :
    {
      ulResponseStatus = CCTP_STATUS_BADREQUEST;
      strResponseData = L"Error occured deserializing the object";
    }
    break;   
    
    //Other errors during deserializing object occured
    default:
    {
      ulResponseStatus = CCTP_STATUS_INTERNALSERVERERROR;
      strResponseData = L"Error creating object";
    }
    break;   
  }
  
  response.setState(ulResponseStatus);
  if (ulResponseStatus != CCTP_STATUS_OK)
  {
    TByteArray aResponseData = TString::transcode(strResponseData);
    response.setData(aResponseData);
  }
  ZLOGGER.println(LOGMODE_INFO, "Skel: Sending response to client: [%d] %s", ulResponseStatus, strResponseData.c_str());
}

/***************************************************************************/
/*! STarts a new client handle. If you want to use an other client handler 
    thread you have to overwrite this method only. Create your own client
    handler and add it to the m_lstClients object.
    \param client: Socket of the client
*/
/***************************************************************************/
void TAbstractSkel::startNewHandler(TSocket* client)
{
  TClientHandlerThread* pClientThread = new TClientHandlerThread(this, client);
  pClientThread->start();
  m_lstClients.add(pClientThread);
}

/***************************************************************************/
/* Creates a client handler thread
*/
/***************************************************************************/
TAbstractSkel::TClientHandlerThread::TClientHandlerThread(TAbstractSkel* parent, TSocket* socket) : TThread()
{
  m_pParent = parent;
  m_pClientSocket = socket;
  if (m_pClientSocket != NULL)
  {
    m_pClientSocket->addRef();
  }
}

/***************************************************************************/
/* Destroys the client handler thread
*/
/***************************************************************************/
TAbstractSkel::TClientHandlerThread::~TClientHandlerThread()
{
  if (m_pClientSocket != NULL)
  {
    m_pClientSocket->disconnect();
    m_pClientSocket->release();
    m_pClientSocket = NULL;
  }
}
    
/***************************************************************************/
/* Kills the thread
*/
/***************************************************************************/
void TAbstractSkel::TClientHandlerThread::kill()
{
  if (!this->isDead())
  {
    ZLOGGER.printfln(LOGMODE_INFO, "Skel: Try to stop client handler thread...");
    ZLOGGER.indent();
    m_bExit = true;
    TThread::kill();
    ZLOGGER.unindent();
    ZLOGGER.printfln(LOGMODE_INFO, "...done");
  }
}

/***************************************************************************/
/* Execute code of the thread
*/
/***************************************************************************/
void TAbstractSkel::TClientHandlerThread::execute()
{
  if (m_pClientSocket != NULL && m_pParent != NULL)
  {
    m_bExit = false;
    m_pClientSocket->setBlockable(false);
    long lQuitLen = TString(REMOTE_QUIT).getSize() + 1;
    while(!m_bExit)
    {
      TCCTPRequest Request;
      TCCTPResponse Response;
      
      TByteArray aData;
      m_pClientSocket->readArray(&aData);
      
      if (aData.getCount() == lQuitLen)
      {
        TString strQuit = TString::transcode(aData);
        if (strQuit == REMOTE_QUIT)
        {
          ZLOGGER.printfln(LOGMODE_INFO, "Skel: Received remote quit signal.");
          m_bExit = true;
        }
      }
      
      if (!m_bExit && aData.getCount() > 0)
      {  
        ZLOGGER.println(LOGMODE_DEBUG, "Skel: Data received [Skel: %d Bytes]", aData.getCount());
        aData.add('\0');
        if (Request.parse(aData) == RET_NOERROR)
        {
          m_pParent->dispatch1(Request, Response);
        }
        else
        {
          Response.setState(CCTP_STATUS_BADREQUEST);
          TByteArray aResponseData = TString::transcode(L"Could not parse request");
          Response.setData(aResponseData);
          ZLOGGER.println(LOGMODE_INFO, "Skel: Could not parse request");
        }
          
        //Write response
        TByteArray aOut;
        Response.toStream(&aOut);
        m_pClientSocket->writeArray(&aOut); 
        
      }
      TTime::usleep(1000);
    }
    m_pClientSocket->disconnect();
    m_pParent->setCleanFlag();
  }  
}
