/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : AbstractSkel                                                  *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:27:33 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: AbstractSkel.h,v $
 *   Revision 1.3  2005/11/25 09:27:33  bhadorn
 *   Renamed start method to startSkeleton. Using readArray() and writeArray() to read and write to socket.
 *
 *   Revision 1.2  2005/11/16 11:51:27  bhadorn
 *   Dynamically removing wasted client handler threads. Dynamic binding of skeleton server
 *
 *   Revision 1.1  2005/11/10 10:39:16  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef AbstractSkelH
#define AbstractSkelH

#include <zeusbase/System/Thread.h>
#include <zeusbase/System/ServerSocket.h>
#include <zeusbase/System/ZVariant.h>
#include <zeusbase/System/ByteArray.hpp>
#include <zeusbase/Remote/RemoteDefines.hpp>
#include <zeusbase/Remote/RemoteMethodCall.h>
#include <zeusbase/Remote/RemoteReference.h>
#include <zeusbase/Remote/Interfaces/IRemoteObject.hpp>

/***************************************************************************/
/*! Declaration of the skeleton class. The skeleton class inherits the
    abstract skeleton
    \param classid : ID of the class (Normally [Class]_Skel )
    \param objectid : ID of the wrapped class (Servant)
*/
/***************************************************************************/
#define SKEL_DECL(classid, objectid) \
class classid : public TAbstractSkel \
{ \
  public : \
    classid(objectid* instance) : TAbstractSkel() \
    { \
      m_pInstance = instance; /*No addRef needed*/\
    } \
    \
  protected: \
    virtual ~classid() \
    { \
      m_pInstance = NULL; \
    } \
    virtual unsigned long dispatchCommand(IRemoteMethodCall* method, IRemoteMethodCall** methodreturn); \
  \
  private: \
    objectid* m_pInstance; \
}; \


/***************************************************************************/
/*! Generates the name of the remote reference parameter
    \param __param : Parameter name of the method
*/
/***************************************************************************/
#define SKEL_REMOTEREF(__param) pRemote##__param


/***************************************************************************/
/*! Starts the dispatching method of the skeleton class
    \param classid : ID of the class
*/
/***************************************************************************/
#define SKEL_DISPATCH_START(classid) \
unsigned long classid::dispatchCommand(IRemoteMethodCall* method, IRemoteMethodCall** methodreturn) \
{ \
  unsigned long ulRetval = RET_REQUEST_FAILED; \
  if (method == NULL) \
  { \
    ulRetval = RET_INVALID_PARAMETER; \
  } \
  else \
  { \
    TSingleLinkedList<IZUnknown*> lstParams; \
    TString strMethodName; \
    TRemoteMethodCall* pReturnCall = new TRemoteMethodCall(strMethodName); \
    TZVariant* pVarRetval = new TZVariant(); \
    method->getMethodName(&strMethodName); \
    ZLOGGER.printfln(LOGMODE_INFO, "Method received [%s]", strMethodName.c_str()); \

/***************************************************************************/
/*! This selects the correct code for extracting the remote method call 
    according to the name of the method. The name must be given as a string
    value.
    \param __name : Name of the method (string)
*/
/***************************************************************************/
#define SKEL_DISP_SWITCH(__name) \
    if (strMethodName == __name) \
    {
    
/***************************************************************************/
/*! Ends a code of a specific method
*/
/***************************************************************************/
#define SKEL_DISP_SWITCH_END }


/***************************************************************************/
/*! Reads a parameter out of the remote method call. This parameter is
    a variant type. The value inside must be extracted manually and 
    given to the real method.<BR>
    \param __name : Name of the parameter as string
    \param __param : Parameter signature
    <PRE><CODE>
    IZVariant* pVarString = NULL;
    SKEL_READ_PARAM("text", pVarString);
    if (pVarString != NULL)
    {
      pVarString->getString(&strText);
      ulRetval = m_pInstance->getText(&strText);
    }
    </CODE></PRE>
*/
/***************************************************************************/
#define SKEL_READ_PARAM(__name, __param) \
      TString strParam##__param = __name; \
      if (method->getParameterByName(&strParam##__param, &__param) ==RET_NOERROR) \
      { \
        lstParams.add(__param); \
        ZLOGGER.printfln(LOGMODE_INFO, "Read param %s...ok",strParam##__param.c_str()); \
      } \
      else \
      { \
        ZLOGGER.printfln(LOGMODE_ERROR, "Read param %s...failed", strParam##__param.c_str()); \
      } \

/***************************************************************************/
/*! This macro reads a remote refefence out of the remote method call.
    The use of this macro is the same as shown in SKEL_READ_PARAM
    \param __name : Name of the parameter as string
    \param __param : Parameter signature
*/
/***************************************************************************/
#define SKEL_READ_PARAM_REMOTEREF(__name, __param) \
      SKEL_READ_PARAM(__name, __param) \
      ISerializable* pSerial##__param = NULL; \
      IRemoteObject* SKEL_REMOTEREF(__param) = NULL; \
      if (__param != NULL && __param->getObject(&pSerial##__param) == RET_NOERROR) \
      { \
        if (pSerial##__param->askForInterface(INTERFACE_IRemoteObject, (IZUnknown**)&SKEL_REMOTEREF(__param)) == RET_NOERROR) \
        { \
          lstParams.add(SKEL_REMOTEREF(__param)); \
        } \
        else \
        { \
          ZLOGGER.printfln(LOGMODE_ERROR, "Failed. Object is not of type IRemoteObject"); \
        } \
        pSerial##__param->release(); \
      } \
      else \
      { \
        ZLOGGER.printfln(LOGMODE_ERROR, "Failed. Object is NULL"); \
      } \

/***************************************************************************/
/*! Helper macro. Dont use directly
*/
/***************************************************************************/
#define SKEL_REMOTEREF_CHECK(__param) (SKEL_REMOTEREF(__param) != NULL)
      
/***************************************************************************/
/*! Ends the dispatching method of the skeleton. 
*/
/***************************************************************************/
#define SKEL_DISPATCH_END \
    while(lstParams.getCount() > 0) \
    { \
      lstParams[0]->release(); \
      lstParams.deleteItem(0); \
    } \
    if (methodreturn != NULL) \
    { \
      *methodreturn = pReturnCall; \
      (*methodreturn)->setReturnValue(pVarRetval); \
    } \
    else \
    { \
      pReturnCall->release(); \
    } \
    pVarRetval->release(); \
    ulRetval = RET_NOERROR; \
  } \
  return ulRetval; \
}

BEGIN_NAMESPACE_Zeus
 
class TCCTPRequest;
class TCCTPResponse;

/***************************************************************************/
/*! This class implements n abstract skeleton according to the server client
    model. The AbstractSkel is the server thread. Each incomming request
    is handled over a client handler thread, witch is a inner class of this
    server. The method calls are dispatched by the main class using a 
    customized method (overwritten by concrete skeleton class).
*/
/***************************************************************************/
class TAbstractSkel : public TThread
{
  public:
    TAbstractSkel();
    
    virtual unsigned long startSkeleton(TString& address, unsigned long& port, bool dynamic);
    virtual void kill();
    virtual void execute();
    void dispatch1(const TCCTPRequest& request, TCCTPResponse& response);
    void setCleanFlag();
    
  protected:
    ///List of all available clients
    TSingleLinkedList<TThread*> m_lstClients;
    ///Client Soket to handle
    TServerSocket* m_pSocket;
    ///Cleaning flag to remove wasted Clienthandler
    bool m_bCleanFlag;
    
    virtual ~TAbstractSkel();
    virtual unsigned long dispatchCommand(IRemoteMethodCall* method, IRemoteMethodCall** methodreturn)=0;
    virtual void startNewHandler(TSocket* client);
    
  private:

    /**********************************************************************/
    /* This inner class manages the incomming requests of clients and
       calls the parenr dispatching method
    */
    /**********************************************************************/
    class TClientHandlerThread : public TThread
    {
      public :
        TClientHandlerThread(TAbstractSkel* parent, TSocket* socket);
    
        virtual void kill();
        virtual void execute();
      
      protected:  
        virtual ~TClientHandlerThread();
        
      private:
        TAbstractSkel* m_pParent;
        TSocket* m_pClientSocket;
        bool m_bExit;
    };
    
};

END_NAMESPACE_Zeus

#endif
 
