/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : AbstractStub                                                  *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:27:48 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: AbstractStub.cpp,v $
 *   Revision 1.3  2005/11/25 09:27:48  bhadorn
 *   Using readArray() and writeArray() to read and write to socket.
 *
 *   Revision 1.2  2005/11/16 11:49:22  bhadorn
 *   Sending quit signal to skeleton when closing the stub. Changed method to [const]
 *
 *   Revision 1.1  2005/11/10 10:39:16  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <zeusbase/Remote/AbstractStub.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/Protocols/CCTPRequest.h>
#include <zeusbase/System/Protocols/CCTPResponse.h>
#include <zeusbase/System/Socket.h>

USING_NAMESPACE_Zeus 

/***************************************************************************/
/*! Creates an abstract stub object
*/
/***************************************************************************/
TAbstractStub::TAbstractStub() : TZObject()
{
  m_ulPort = 0;
  m_pSocket = NULL;
}

/***************************************************************************/
/*! Destroys the stub
*/
/***************************************************************************/
TAbstractStub::~TAbstractStub()
{
  this->disconnect();
}

/***************************************************************************/
/*! This method sends a remote method call to a remote object and waits 
    until the response will arrive. The response will be returned by 
    the 2nd argument and must be released if the method returns RET_NOERROR
    \param method : Method object to send
    \param methodreturn : Method object containing the return values. If 
            no return value must be received, this parameter must be NULL.
    \retval RET_NOERROR : Method has been returned
    \retval RET_NETWORK_ERROR : Error using the network layer
    \retval RET_INTERNAL_ERROR : Could not serialize the object
    \retval RET_REMOTECALL_ERROR : CCTP Protocol returned error or unable
             to create the returned object.
    \pre The method parameter must be valid.
*/
/***************************************************************************/
unsigned long TAbstractStub::sendMethod(TRemoteMethodCall* method, IRemoteMethodCall** methodreturn) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  TString strMethodName;
  method->getMethodName(&strMethodName);
  ZLOGGER.printfln(LOGMODE_DEBUG, "Stub: Send remote method call [%s]...", strMethodName.c_str());
  if (m_pSocket != NULL)
  {
    TByteArray aData;
    if (method->serialize(&aData) == RET_NOERROR)
    {
      TCCTPRequest Request;
      Request.setData(aData);
      Request.setMethod(CCTP_METHOD_POST);
      Request.setObjectType(INTERFACE_IRemoteMethodCall);
      
      TByteArray aOutData;
      Request.toStream(&aOutData);
      if (m_pSocket->writeArray(&aOutData) == RET_NOERROR)
      {
        ZLOGGER.printfln(LOGMODE_DEBUG, "done. [Stub: %d Bytes sent]", aOutData.getCount());
        ulRetval = getMethodReturn(methodreturn);
      }
      else
      {
        ulRetval = RET_NETWORK_ERROR;
        ZLOGGER.printfln(LOGMODE_DEBUG, "failed. [Stub: Socket returned error]");
      }
    }
    else
    {
      ulRetval = RET_INTERNAL_ERROR;
      ZLOGGER.printfln(LOGMODE_ERROR, "Stub: Error serializing the remote method call [%s]", strMethodName.c_str());
    }
  }
  
  //No socket created
  else
  {
    ZLOGGER.printfln(LOGMODE_DEBUG, "failed. [Stub: Socket has no valid socket]");
    ulRetval = RET_NETWORK_ERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! this method trys to receive the return values of the method
    \param methodreturn : Return values of the remote method call. If this
       parameter is NULL, this method checks only the CCTP return value
    \return Error codes (see sendMethod())
    \pre The parameter methodreturn and the member m_pSocket must be valid
*/
/***************************************************************************/
unsigned long TAbstractStub::getMethodReturn(IRemoteMethodCall** methodreturn) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  ZLOGGER.indent();
  ZLOGGER.printfln(LOGMODE_INFO, "...wait for response [Stub]...");
  TCCTPResponse Response;
  TByteArray aData;
  m_pSocket->readArray(&aData);
  
  aData.add('\0');
  Response.parse(aData);
        
  if (Response.getState() == CCTP_STATUS_OK)
  {    
    if (methodreturn == NULL)
    {
      ulRetval = RET_NOERROR;
      ZLOGGER.printfln(LOGMODE_INFO, "...done. [Stub: No return call]");
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_INFO, "...OK [Stub]. Return method call received...");
      const TByteArray& aObjectStream = Response.getData();
      if (TZObjectFactory::getInstance().createObjectFromStream(&aObjectStream, (ISerializable**)methodreturn) == RET_NOERROR)
      {
        ulRetval = RET_NOERROR;
        TString strMethodName;
        (*methodreturn)->getMethodName(&strMethodName);
        ZLOGGER.printfln(LOGMODE_INFO, "...done. [Stub: Method %s received]", strMethodName.c_str());
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_INFO, "Stub: could not create the return method call.");
        ulRetval = RET_REMOTECALL_ERROR;
      }
    }
  }
  else
  {
    ulRetval = RET_REMOTECALL_ERROR;
    ZLOGGER.printfln(LOGMODE_INFO, "failed. [Stub: Protocol returned error]. Error code=%d", (int)Response.getState());
  }
  ZLOGGER.unindent();
  return ulRetval;
}

/**************************************************************************/
/*! Disconnects from the remote object
*/
/**************************************************************************/
void TAbstractStub::disconnect()
{
  if (m_pSocket != NULL)
  {
    //Send quit signal to the stub
    TByteArray aData = TString::transcode(REMOTE_QUIT);
    m_pSocket->writeArray(&aData);
    m_pSocket->disconnect();
    m_pSocket->release();
    m_pSocket = NULL;
  }
}


/**************************************************************************/
/*! See IRemoteObjectStub
*/
/**************************************************************************/
unsigned long MQUALIFIER TAbstractStub::setClassName(const IString* name)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    m_strClassName = name;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/**************************************************************************/
/*! See IRemoteObjectStub
*/
/**************************************************************************/
unsigned long MQUALIFIER TAbstractStub::setCodeModuleName(const IString* name)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    m_strCodeModule = name;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/**************************************************************************/
/*! See IRemoteObjectStub
*/
/**************************************************************************/
unsigned long MQUALIFIER TAbstractStub::connect(const IString* address, unsigned long port)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  this->disconnect();
  if (address != NULL && port > 0)
  {
    m_pSocket = new TSocket(address, port);
    ulRetval = m_pSocket->connect();
  }
  return ulRetval;
}

/**************************************************************************/
/*! See IRemoteObject
*/
/**************************************************************************/
unsigned long MQUALIFIER TAbstractStub::getClassName(IString* name)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assignStr(&m_strClassName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/**************************************************************************/
/*! See IRemoteObject
*/
/**************************************************************************/
unsigned long MQUALIFIER TAbstractStub::getCodeModuleName(IString* name)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assignStr(&m_strCodeModule);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/**************************************************************************/
/*! See IRemoteObject
*/
/**************************************************************************/
unsigned long MQUALIFIER TAbstractStub::getConnectionParam(IString* address, unsigned long* port)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (address != NULL && port != NULL)
  {
    address->assignStr(&m_strAddress);
    *port = m_ulPort;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TAbstractStub);
  INTERFACE_CAST(IRemoteObject, INTERFACE_IRemoteObject);
  INTERFACE_CAST(IRemoteObjectStub, INTERFACE_IRemoteObjectStub);
MEMORY_MANAGER_IMPL_END;


