/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : AbstractStub                                                  *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:01:42 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: AbstractStub.h,v $
 *   Revision 1.4  2005/12/14 09:01:42  bhadorn
 *   createObject was not declared with MQUALIFIER
 *
 *   Revision 1.3  2005/11/25 09:27:48  bhadorn
 *   Using readArray() and writeArray() to read and write to socket.
 *
 *   Revision 1.2  2005/11/16 11:49:22  bhadorn
 *   Sending quit signal to skeleton when closing the stub. Changed method to [const]
 *
 *   Revision 1.1  2005/11/10 10:39:16  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef AbstractStubH
#define AbstractStubH

#include <zeusbase/System/ZObject.h>
#include <zeusbase/System/ZVariant.h>
#include <zeusbase/System/ByteArray.hpp>
#include <zeusbase/System/Stack.hpp>
#include <zeusbase/Remote/RemoteDefines.hpp>
#include <zeusbase/Remote/RemoteMethodCall.h>
#include <zeusbase/Remote/RemoteReference.h>
#include <zeusbase/Remote/Interfaces/IRemoteObjectStub.hpp>

BEGIN_NAMESPACE_Zeus


/***************************************************************************/
/*! Declaration of the stub class. This contains the autoregistration and
    factory method of the class (for ZObjectFactory)
    \param classid : ID of the class
    \param classname : Name of the class (String)
*/
/***************************************************************************/
#define STUB_CLASS_DECL(classid, classname) \
  public: \
    static unsigned long MQUALIFIER createObject(unsigned long /*objectID*/, IInputStream* stream, IZUnknown** obj) \
    { \
      unsigned long ulRetval = RET_INVALID_PARAMETER;\
      if (stream != NULL) \
      { \
        ulRetval = RET_REQUEST_FAILED; \
      } \
      if (obj != NULL)\
      { \
        *obj = (IRemoteObject*)new classid();\
        ulRetval = RET_NOERROR;\
      } \
      return ulRetval;\
    } \
    \
    static void autoreg(bool reg)\
    { \
      static unsigned long classid##Handle = 0;\
      TString strClassName(classname); \
      if (reg) \
      { \
        classid##Handle = TZObjectFactory::getInstance().registerClass(&strClassName, createObject); \
      } \
      else \
      { \
        TZObjectFactory::getInstance().unregisterClass(&strClassName, classid##Handle); \
      } \
    } \
    
    
/***************************************************************************/
/*! Starts the delegation of a method call to the remote method call.
    \name : Name of the method
    \method : Method signature (something like: void classid::methodname )
*/
/***************************************************************************/
#define STUB_DELEGATION(name, method) \
method \
{ \
  unsigned long ulRetval = RET_REQUEST_FAILED; \
  TRemoteMethodCall* pMethod = new TRemoteMethodCall(name); \
  TStack<IZUnknown*> lstOutReferences;
    
/***************************************************************************/
/*! Transformation of a method parameter to the parameter of the remote 
    method.
    \param __name : Name of the parameter (string)
    \param __param : Parameter of the method
*/
/***************************************************************************/
#define STUB_PARAM(__name, __param) \
  { \
    TZVariant* pVar = new TZVariant(__name, __param); \
    pMethod->addParameter(pVar); \
    pVar->release(); \
  }

/***************************************************************************/
/*! Transforms a remote object reference to a reference for remote method 
    call. The __param could be a real object with interface IRemoteObject.
    This object must be transformed to a remote reference, witch contains
    only data like address and port of the real remote object. Receiving 
    such a reference by the skeleton, a stub will be created connecting to 
    this real remote object.
    \param __name : Name of the parameter (string)
    \param __param : Parameter of the method
*/
/***************************************************************************/
#define STUB_PARAM_REMOTEREF(__name, __param) \
  { \
    TRemoteReference* pRef = new TRemoteReference(__param); \
    STUB_PARAM(__name, pRef); \
    pRef->release(); \
  } 

    
/***************************************************************************/
/*! Starts with all return (out) parameter and the return value. First it 
    sends the remote method and after successfully receiving the response of
    the remote object we can start reading all return parameters.
    \note This macro should be placed after the STUB_PARAM macros only. 
    \note To use [inout] parameters you have to place the parameter 
          before (in) and once after this macro (out).
*/
/***************************************************************************/
#define STUB_RETURN_START \
  IRemoteMethodCall* pMethodReturn = NULL; \
  ulRetval = this->sendMethod(pMethod, &pMethodReturn);\
  \
  if (ulRetval == RET_NOERROR) \
  { \
        
/***************************************************************************/
/*! Reads a return parameter as a string (IString*)
    \param __name : Name of the parameter (string)
    \param __param : Parameter of the method
*/
/***************************************************************************/
#define STUB_RETPARAM_STRING(__name, __param) \
    if (ulRetval == RET_NOERROR) \
    { \
      TString strName(__name); \
      IZVariant* pVariant; \
      if (pMethodReturn->getParameterByName(&strName, &pVariant) == RET_NOERROR) \
      { \
        TString strData; \
        if (pVariant->getString(&strData) == RET_NOERROR) \
        { \
          __param->assignStr(&strData); \
          ulRetval = RET_NOERROR; \
        } \
        else \
        { \
          ulRetval = RET_REQUEST_FAILED; \
        } \
        pVariant->release(); \
      } \
      else \
      { \
        ulRetval = RET_REMOTEPROTOCOL_ERROR; \
      } \
    } 
    
/***************************************************************************/
/*! Reads a return parameter as a string list IList<IString*>*
    \param __name : Name of the parameter (list)
    \param __param : Parameter of the method
*/
/***************************************************************************/
#define STUB_RETPARAM_STRINGLIST(__name, __param) \
    if (ulRetval == RET_NOERROR) \
    { \
      TString strCountName = RPARAM_LIST_COUNT(__name); \
      IZVariant* pCountVariant; \
      if (pMethodReturn->getParameterByName(&strCountName, &pCountVariant) == RET_NOERROR) \
      { \
        long lCount = 0; \
        pCountVariant->getLong(&lCount); \
        for(long i = 0; (i < lCount && ulRetval == RET_NOERROR); i++) \
        { \
          TString strParamName = RPARAM_LIST_ITEM(__name, i); \
          TString strParamValue; \
          IString* pParamValuePtr = &strParamValue; \
          STUB_RETPARAM_STRING(strParamName, pParamValuePtr); \
          if (ulRetval == RET_NOERROR) \
          { \
            __param->add(&strParamValue);\
          } \
        } \
        if (ulRetval != RET_NOERROR) \
        { \
          __param->clear(); \
        } \
        pCountVariant->release(); \
      } \
      else \
      { \
        ulRetval = RET_REMOTEPROTOCOL_ERROR; \
      } \
    } 

// Helpermacro
#define STUB_CREATE_STUB(__object, __stub) \
          TRemoteReference* pRef = (TRemoteReference*)__object; \
          if (pRef->createStub(&__stub) == RET_NOERROR) \
          { \
            lstOutReferences.push(__stub); \
          } \
          else \
          { \
            ulRetval = RET_UNREGISTERED_CLASS; \
          } \
    
    
/***************************************************************************/
/*! Reads a return parameter as a remote object reference. Note that after a
    successfully return of the remote method call we receive a correct remote 
    reference and a stub will be created automatically connecting to this
    remote reference.
    \param __name : Name of the parameter (string)
    \param __param : Parameter of the method
*/
/***************************************************************************/
#define STUB_RETPARAM_REMOTEREF(__name, __param) \
    if (ulRetval == RET_NOERROR) \
    { \
      TString strName(__name); \
      IZVariant* pVariant; \
      if (pMethodReturn->getParameterByName(&strName, &pVariant) == RET_NOERROR) \
      { \
        ISerializable* pObject = NULL; \
        if (pVariant->getObject(&pObject) == RET_NOERROR) \
        { \
          /*TRemoteReference* pRef = (TRemoteReference*)pObject; \
          if (pRef->createStub(__param) == RET_NOERROR) \
          { \
            lstOutReferences.push(*__param); \
          } \
          else \
          { \
            ulRetval = RET_UNREGISTERED_CLASS; \
          } */\
          STUB_CREATE_STUB(pObject, *__param); \
          pObject->release(); \
        } \
        else \
        { \
          ulRetval = RET_REQUEST_FAILED; \
        } \
        pVariant->release(); \
      } \
      else \
      { \
        ulRetval = RET_REMOTEPROTOCOL_ERROR; \
      } \
    } 
  
/***************************************************************************/
/*! Reads a return parameter as a remote object reference list. Note that after a
    successfully return of the remote method call we receive a correct remote 
    reference and all stubs will be created automatically connecting to those
    remote references.
    \param __name : Name of the parameter (string)
    \param __param : Parameter of the method
*/
/***************************************************************************/
#define STUB_RETPARAM_REMOTEREFLIST(__name, __param) \
    if (ulRetval == RET_NOERROR) \
    { \
      TString strName(__name); \
      IZVariant* pVariant; \
      if (pMethodReturn->getParameterByName(&strName, &pVariant) == RET_NOERROR) \
      { \
        TSingleLinkedList<ISerializable*> lstObjects; \
        if (pVariant->getObjectList(&lstObjects) == RET_NOERROR) \
        { \
          IListIterator<ISerializable*>* pIt = lstObjects.getIterator(); \
          while (pIt->hasNextItem()) \
          { \
            IRemoteObject* pStub = NULL;\
            ISerializable* pObject = pIt->getNextItem(); \
            STUB_CREATE_STUB(pObject, pStub); \
            if (pStub != NULL) \
            { \
              __param->add(pStub);\
            } \
          } \
          lstObjects.releaseIterator(pIt); \
          \
          while(lstObjects.getCount() > 0) \
          { \
            lstObjects[0]->release(); \
            lstObjects.deleteItem(0); \
          } \
          if (__param->getCount() > 0) \
          { \
            ulRetval = RET_NOERROR; \
          } \
        } \
        else \
        { \
          ulRetval = RET_REQUEST_FAILED; \
        } \
        pVariant->release(); \
      } \
      else \
      { \
        ulRetval = RET_REMOTEPROTOCOL_ERROR; \
      } \
    } 
        
/***************************************************************************/
/*! Reads a return value as a long value
    \param __retvar : Variable witch contains the return value of the method
*/
/***************************************************************************/
#define STUB_RETVALUE_LONG(__retvar) \
    if (ulRetval == RET_NOERROR) \
    { \
      IZVariant* pRetVar = NULL; \
      if (pMethodReturn->getReturnValue(&pRetVar) == RET_NOERROR)  \
      { \
        if (pRetVar->getType() == varLong) \
        { \
          long lVal = 0;\
          pRetVar->getLong(&lVal); \
          __retvar = lVal; \
        } \
        else \
        { \
          ulRetval = RET_REMOTEPROTOCOL_ERROR; \
        } \
        pRetVar->release(); \
      } \
    }
      
/***************************************************************************/
/*! Reads a return value as double value
    \param __retvar : Variable witch contains the return value of the method
*/
/***************************************************************************/
#define STUB_RETVALUE_DOUBLE(__retvar) \
    if (ulRetval == RET_NOERROR) \
    { \
      IZVariant* pRetVar = NULL; \
      if (pMethodReturn->getReturnValue(&pRetVar) == RET_NOERROR)  \
      { \
        if (pRetVar->getType() == varDouble) \
        { \
          pRetVar->getDouble(&__retvar); \
        } \
        else \
        { \
          ulRetval = RET_REMOTEPROTOCOL_ERROR; \
        } \
        pRetVar->release(); \
      } \
    }


/***************************************************************************/
/*! Ends the return section
    \param __retvar : Variable to return of this method.
*/
/***************************************************************************/
#define STUB_RETURN_END(__retvar) \
    pMethodReturn->release(); \
  } \
  pMethod->release(); \
  if (ulRetval != RET_NOERROR)\
  { \
    while(!lstOutReferences.isEmpty()) \
    { \
      lstOutReferences.pop()->release(); \
    } \
  } \
  return __retvar; \
}
    
/***************************************************************************/
/*! This macro returns just the error code [ulRetval] of the stub method.
*/
/***************************************************************************/
#define STUB_RETURN_ERRORCODE \
  STUB_RETURN_START \
  STUB_RETVALUE_LONG(ulRetval) \
  STUB_RETURN_END(ulRetval)
    
/***************************************************************************/
/*! Returns nothing. Use this for all methods having a [void] return.
*/
/***************************************************************************/
#define STUB_RETURN_VOID \
  ulRetval = this->sendMethod(pMethod, NULL);\
  pMethod->release(); \
}

class TSocket;

/***************************************************************************/
/*! This class implements an abstract sub for connecting, sending method 
    calls and receiving return values from the methods
*/
/***************************************************************************/
class TAbstractStub : public TZObject, public IRemoteObjectStub
{
  public :
    TAbstractStub();
    
    //Methods of IRemoteObjectStub
    virtual unsigned long MQUALIFIER setClassName(const IString* name);
    virtual unsigned long MQUALIFIER setCodeModuleName(const IString* name);
    virtual unsigned long MQUALIFIER connect(const IString* address, unsigned long port);
    
    //Methods of IRemoteObject
    virtual unsigned long MQUALIFIER getClassName(IString* name);
    virtual unsigned long MQUALIFIER getCodeModuleName(IString* name);
    virtual unsigned long MQUALIFIER getConnectionParam(IString* address, unsigned long* port);
    
    //Methods of IZUnknown
    MEMORY_MANAGER_DECL
  
  protected:
    ///Socket class
    TSocket* m_pSocket;
    ///Name of the codeModule
    TString m_strCodeModule;
    ///name of the class
    TString m_strClassName;
    ///IP address 
    TString m_strAddress;
    ///TCP port
    unsigned long m_ulPort;
  
    virtual ~TAbstractStub();
    unsigned long sendMethod(TRemoteMethodCall* method, IRemoteMethodCall** methodreturn) const;
    unsigned long getMethodReturn(IRemoteMethodCall** methodreturn) const;
    void disconnect();
  
  private:
  
};

END_NAMESPACE_Zeus
 
 
#endif
