/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : INaming                                                       *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:45:43 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: INaming.hpp,v $
 *   Revision 1.2  2005/11/16 11:45:43  bhadorn
 *   Input parameter are const. New method added for getting only the object names instead of the stubs.
 *
 *   Revision 1.1  2005/11/10 10:39:54  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef INamingHPP
#define INamingHPP

#include <zeusbase/System/Interfaces/IXObject.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/System/Interfaces/IList.hpp>
#include <zeusbase/Remote/Interfaces/IRemoteObject.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! Interface of a naming service to register and looking up remote 
    objects
*/
/***************************************************************************/
class INaming : public IZUnknown
{
  public:
    
    /***********************************************************************/
    /*! Binds a name to a object, witch can accessed from remote. If the name 
        is already registered, this method fails. To rebind an existing object
        use the method rebind()
        \param name : Name of the object
        \param object : Object reference of the skeleton object
        \retval RET_NOERROR : Object bound
        \retval RET_REQUEST_FAILED : Could not bind object
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER bind(const IString* name, IRemoteObject* object)=0;

    /***********************************************************************/
    /*! Same as bind(), but registeres the object in a specified category.
        To access such an object you must use lookfor2() and unbind2()
        \param name : Name of the object
        \param cat : Name of the category
        \param object : Object reference of the skeleton object
        \return Error codes see bind()
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER bind2(const IString* name, const IString* cat, IRemoteObject* object)=0;
    
    /***********************************************************************/
    /*! Rebinds a name to a object, witch can be accessed from remote. If
        the object hasnt been bound before, this method acts like bind()
        \param name : Name of the object
        \param object : Object reference of the skeleton object
        \retval RET_NOERROR : Object bound
        \retval RET_REQUEST_FAILED : Could not rebind object
        \retval RET_INVALID_PARAMETER : Invalid parameter received
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER rebind(const IString* name, IRemoteObject* object)=0;
    
    /***********************************************************************/
    /*! Same as rebind(), but reregisters the object using a specific 
        category.
        \param name : Name of the object
        \param cat : Categorty
        \param object : Object reference of the skeleton object
        \return Error codes see rebind()
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER rebind2(const IString* name, const IString* cat, IRemoteObject* object)=0;
    
    /***********************************************************************/
    /*! Removes an object from the naming service
        \param name : Name of the object
        \retval RET_NOERROR : Object has been unregistered
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not unregister because the entry
                                     does not exist
        \retval RET_INTERNAL_ERROR : Registry has an error (not loaded)
        \retval RET_REMOTECALL_ERROR : Communication error with remote object
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER unbind(const IString* name)=0;

    /***********************************************************************/
    /*! Removes an object from the naming service
        \param name : Name of the object
        \param cat : Categorty
        \return Error codes see unbind()
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER unbind2(const IString* name, const IString* cat)=0;
    
    /***********************************************************************/
    /*! Returns an remote object reference for a given name
        \param name : Name of object
        \param object : return parameter of first object found
        \retval RET_NOERROR : Object has been returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : No entry found
        \retval RET_INTERNAL_ERROR : Registry has an error (not loaded)
        \retval RET_REMOTECALL_ERROR : Communication error with remote object
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER lookup(const IString* name, IRemoteObject** object)=0;
    
    /***********************************************************************/
    /*! Returns an remote object reference for a given name
        \param name : Name of object
        \param cat : Categorty
        \param object : return parameter of first object found
        \return Error codes see lookup()
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER lookup2(const IString* name, const IString* cat, IRemoteObject** object)=0;
    
    /***********************************************************************/
    /*! Returns an remote object reference for a given category name. The 
        remote object is randomly choosen by the naming service.
        \param cat : Categorty
        \param object : return parameter of first object found
        \return Error codes see lookup()
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER lookupAny(const IString* cat, IRemoteObject** object)=0;
    
    /***********************************************************************/
    /*! Gets a set of objects registered. This method will return all
        objects of the naming service having the same name, regardless of
        the category
        \param name : Name of the objects
        \param objects : return parameter of all found objects
        \retval RET_NOERROR : Objects has been returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : No entry found
        \retval RET_INTERNAL_ERROR : Registry has an error (not loaded)
        \retval RET_REMOTECALL_ERROR : Communication error with remote object
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER lookupMultibleObjects(const IString* name, IList<IRemoteObject*>* objects)=0;
    
    /***********************************************************************/
    /*! Gets a set of objects registered for the same category
        \param cat : Name of the category
        \param objects : return parameter of all found objects
        \retval RET_NOERROR : Objects has been returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : No entry found
        \retval RET_INTERNAL_ERROR : Registry has an error (not loaded)
        \retval RET_REMOTECALL_ERROR : Communication error with remote object
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER lookupCategory(const IString* cat, IList<IRemoteObject*>* objects)=0;
    
    /***********************************************************************/
    /*! Gets a set of objects names registered for the same category
        \param cat : Name of the category
        \param names : return parameter of all found objects names for this
                       category
        \retval RET_NOERROR : Names has been returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : No entry found
        \retval RET_INTERNAL_ERROR : Registry has an error (not loaded)
        \retval RET_REMOTECALL_ERROR : Communication error with remote object
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getObjectNames(const IString* cat, IList<IString*>* names)=0;
    
    /***********************************************************************/
    /*! Checks if a naming service is created localy or remote.
        \retval true : Created remote
        \retval false : Created localy
    */
    /***********************************************************************/
    virtual bool MQUALIFIER isRemote() const=0;
};

END_NAMESPACE_Zeus

#endif
