/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Naming                                                        *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:29:25 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Naming.cpp,v $
 *   Revision 1.3  2005/11/25 09:29:25  bhadorn
 *   Logging method calls
 *
 *   Revision 1.2  2005/11/16 11:45:24  bhadorn
 *   Input parameter are const. New method added for getting only the object names instead of the stubs.
 *
 *   Revision 1.1  2005/11/10 10:39:16  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/Remote/Naming.h>
#include <zeusbase/System/Socket.h>
#include <zeusbase/System/Time.h>
#include <zeusbase/System/CriticalSection.h>
#include <zeusbase/System/XObjectFactory.h>
#include <zeusbase/System/Protocols/HTTPRequest.h>
#include <zeusbase/System/Protocols/HTTPResponse.h>

#define STD_CATEGORY      L"iko"

USING_NAMESPACE_Zeus


///Static reference initialisation
TNaming TNaming::m_Naming;

/***************************************************************************/
/*! Returns the singleton of the naming service
    \return singleton
*/
/***************************************************************************/
TNaming& TNaming::getInstance()
{
  return m_Naming; 
}


/***************************************************************************/
/*! Creates the naming service
*/
/***************************************************************************/
TNaming::TNaming()
{
  m_pLock = new TCriticalSection();
  m_pDelegationInterface = NULL;
  m_bRemote = false;
}

/***************************************************************************/
/*! Destroys the naming service
*/
/***************************************************************************/
TNaming::~TNaming()
{
  reset();
  m_pLock->release();
  m_pLock = NULL;
}

/***************************************************************************/
/*! Removes all registered objects
*/
/***************************************************************************/
void TNaming::removeAllObjects()
{
  m_pLock->enter();
  
  //Iterator over the categories
  IListIterator<TRemoteObjectMap*>* pIt = m_mapCategory.getIterator();
  while(pIt->hasNextItem())
  {
    TRemoteObjectMap* pMap = pIt->getNextItem();

    ////////
    //Iterator over the objects    
    IListIterator<IRemoteObject*>* pObjIt = pMap->getIterator();
    while(pObjIt->hasNextItem())
    {
      pObjIt->getNextItem()->release();
    }
    pMap->releaseIterator(pObjIt);
    ////////
    
    pMap->clear();
    delete pMap;
  }
  m_mapCategory.releaseIterator(pIt);
  m_mapCategory.clear();
  m_pLock->leave();
}

/***************************************************************************/
/*! Resets the naming service. All registered object will be released
*/
/***************************************************************************/
void TNaming::reset()
{
  removeAllObjects();
  if (m_pDelegationInterface != NULL)
  {
    m_pDelegationInterface->release();
    m_pDelegationInterface = NULL;
  }
}

/***************************************************************************/
/*! Sets the delegation interface 
*/
/***************************************************************************/
void TNaming::setDelegationInterface(INaming* iface)
{
  reset();
  m_pDelegationInterface = iface;
  if (m_pDelegationInterface != NULL)
  {
    m_pDelegationInterface->addRef();
  }
}

/***************************************************************************/
/*! Sets the naming service for remote. This method mus be called before
    connect() otherwise the connection will fail.
    \param mode : true means using for Remote (default). False is to reset 
                  this. 
*/
/***************************************************************************/
void TNaming::useRemote(bool mode /*=true*/)
{
  if (m_bRemote && !mode)
  {
    reset();
  }
  m_bRemote = mode;
}


/***************************************************************************/
/*! This creates a stub of the naming server to access a remote naming 
    service. This method fails if the useRemote() method wasnt called before.
    \param address : Address of the real server
    \param port : Port of the real server
*/
/***************************************************************************/
unsigned long TNaming::connect(const TString& address, unsigned long port)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_bRemote)
  {
    reset();
    
    TString strClassName(L"TNaming_Stub");
    IZUnknown* pObject = NULL;
    if (TZObjectFactory::getInstance().createObjectByClassName(&strClassName, NULL, &pObject) == RET_NOERROR)
    {
      ulRetval = pObject->askForInterface(INTERFACE_INaming, (IZUnknown**)&m_pDelegationInterface);
      
      IRemoteObjectStub* pStub = NULL;
      if (ulRetval == RET_NOERROR && pObject->askForInterface(INTERFACE_IRemoteObjectStub, (IZUnknown**)&pStub) == RET_NOERROR)
      {
        //Error connecting to the remote object
        if (pStub->connect(&address, port) != RET_NOERROR)
        {
          m_pDelegationInterface->release();
          m_pDelegationInterface = NULL;
          ulRetval = RET_NETWORK_ERROR;
        }
        else
        {
          ulRetval = RET_NOERROR;
        }
        pStub->release(); 
      }
      else
      {
        ulRetval = RET_UNKNOWN_INTERFACE;
      }
      pObject->release();
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! See INaming
*/
/***************************************************************************/
unsigned long MQUALIFIER TNaming::bind(const IString* name, IRemoteObject* object)
{
  TString strCat(STD_CATEGORY);
  return bind2(name, &strCat, object);
}

/***************************************************************************/
/*! See INaming
*/
/***************************************************************************/
unsigned long MQUALIFIER TNaming::bind2(const IString* name, const IString* cat, IRemoteObject* object)
{ 
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (object == NULL || name == NULL || cat == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_bRemote)
  {
    if (m_pDelegationInterface != NULL)
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate bind2.");
      ulRetval = m_pDelegationInterface->bind2(name, cat, object);
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate of bind2 failed. No remote connection.");
    }
  }
  else
  {
    //Register local
    m_pLock->enter();
    TString strName(name);
    TString strCat(cat);
    
    TRemoteObjectMap* pCategory = NULL;
    
    //Search for category
    if (!m_mapCategory.hasItem(strCat))
    {
      pCategory = new TRemoteObjectMap();
      m_mapCategory.setItem(strCat, pCategory);
    }
    else
    {
      pCategory = m_mapCategory.getItem(strCat);
    }
    
    //Add Object to the category
    if (pCategory != NULL)
    {
      if (!pCategory->hasItem(strName))
      {
        pCategory->setItem(strName, object);
        object->addRef();
        ulRetval = RET_NOERROR;
        ZLOGGER.printfln(LOGMODE_INFO, "Naming: Object added [%s at %s].", strCat.c_str(), strName.c_str());
      }
    }
    m_pLock->leave();
  }  
  return ulRetval;
}

/***************************************************************************/
/*! See INaming
*/
/***************************************************************************/
unsigned long MQUALIFIER TNaming::rebind(const IString* name, IRemoteObject* object)
{
  TString strCat(STD_CATEGORY);
  return this->rebind2(name, &strCat, object);
}

/***************************************************************************/
/*! See INaming
*/
/***************************************************************************/
unsigned long MQUALIFIER TNaming::rebind2(const IString* name, const IString* cat, IRemoteObject* object)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (object == NULL || name == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_bRemote)
  {
    if (m_pDelegationInterface != NULL)
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate rebind2.");
      ulRetval = m_pDelegationInterface->rebind2(name, cat, object);
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate of rebind2 failed. No remote connection.");
    }
  }
  else
  {
    //Register local
    m_pLock->enter();
    TString strName(name);
    TString strCat(cat);
    if (m_mapCategory.hasItem(strCat))
    {
      TRemoteObjectMap* pCategory = m_mapCategory.getItem(strCat);
      if (pCategory->hasItem(strName))
      {
        pCategory->getItem(strName)->release();
        pCategory->deleteItem(strName);
      }
    }
    m_pLock->leave();
    ulRetval = this->bind2(name, cat, object);
  }  
  return ulRetval;

}


/***************************************************************************/
/*! See INaming
*/
/***************************************************************************/
unsigned long MQUALIFIER TNaming::unbind(const IString* name)
{
  TString strCat(STD_CATEGORY);
  return this->unbind2(name, &strCat);
}

/***************************************************************************/
/*! See INaming
*/
/***************************************************************************/
unsigned long MQUALIFIER TNaming::unbind2(const IString* name, const IString* cat)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_bRemote)
  {
    if (m_pDelegationInterface != NULL)
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate unbind2.");
      ulRetval = m_pDelegationInterface->unbind2(name, cat);
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate of unbind2 failed. No remote connection.");
    }
  }
  else
  {
    //Unregister local
    m_pLock->enter();
    TString strName(name);
    TString strCat(cat);
    if (m_mapCategory.hasItem(strCat))
    {
      TRemoteObjectMap* pCategory = m_mapCategory.getItem(strCat);
      if (pCategory->hasItem(strName))
      {
        pCategory->getItem(strName)->release();
        pCategory->deleteItem(strName);
        ulRetval = RET_NOERROR;
        ZLOGGER.printfln(LOGMODE_INFO, "Naming: Object removed [%s at %s].", strCat.c_str(), strName.c_str());
      }
    }
    m_pLock->leave();
   }
  
  return ulRetval;
}


/***************************************************************************/
/*! See INaming
*/
/***************************************************************************/
unsigned long MQUALIFIER TNaming::lookup(const IString* name, IRemoteObject** object)
{
  TString strCat(STD_CATEGORY);
  return this->lookup2(name, &strCat, object);
}


/***************************************************************************/
/*! See INaming
*/
/***************************************************************************/
unsigned long MQUALIFIER TNaming::lookup2(const IString* name, const IString* cat, IRemoteObject** object)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL || object == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_bRemote)
  {
    if (m_pDelegationInterface != NULL)
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate lookup2.");
      ulRetval = m_pDelegationInterface->lookup2(name, cat, object);
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate of lookup2 failed. No remote connection.");
    }
  }
  else
  {
    //Try to get the local object first (increases the performance)
    TString strName(name);
    TString strCat(cat);
    m_pLock->enter();
    if (m_mapCategory.hasItem(strCat))
    {
      TRemoteObjectMap* pCategory = m_mapCategory.getItem(strCat);
      if (pCategory->hasItem(strName))
      {
        *object = pCategory->getItem(strName);
        (*object)->addRef();
        ZLOGGER.printfln(LOGMODE_INFO, "Naming: lookup sucessfull [%s at %s].", strCat.c_str(), strName.c_str());
        ulRetval = RET_NOERROR;
      }
    }
    m_pLock->leave();
  }
  return ulRetval;

}

/***************************************************************************/
/*! See INaming
*/
/***************************************************************************/
unsigned long MQUALIFIER TNaming::lookupAny(const IString* cat, IRemoteObject** object)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (cat == NULL || object == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_bRemote)
  {
    if (m_pDelegationInterface != NULL)
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate lookupAny.");
      ulRetval = m_pDelegationInterface->lookupAny(cat, object);
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate of lookupAny failed. No remote connection.");
    }
  }
  else
  {
    //Try to get the local object first (increases the performance)
    TString strCat(cat);
    m_pLock->enter();
    if (m_mapCategory.hasItem(strCat))
    {
      TRemoteObjectMap* pCategory = m_mapCategory.getItem(strCat);
      long lCount = pCategory->getCount();
      long lIndex = TTime::random(0, lCount-1);
      
      IListIterator<IRemoteObject*>* pIt = pCategory->getIterator();
      while(pIt->hasNextItem() && lIndex >= 0)
      {
        *object = pIt->getNextItem();
        lIndex--;
      }
      pCategory->releaseIterator(pIt);
      
      if (*object != NULL)
      {
        (*object)->addRef();
        ZLOGGER.printfln(LOGMODE_INFO, "Naming: lookupAny sucessfull [%s].", strCat.c_str());
        ulRetval = RET_NOERROR;
      }
    }
    m_pLock->leave();    
  }
  return ulRetval;
}


/***************************************************************************/
/*! See INaming
*/
/***************************************************************************/
unsigned long MQUALIFIER TNaming::lookupMultibleObjects(const IString* name, IList<IRemoteObject*>* objects)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL || objects == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_bRemote)
  {
    if (m_pDelegationInterface != NULL)
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate lookupMultibleObjects.");
      ulRetval = m_pDelegationInterface->lookupMultibleObjects(name, objects);
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate of lookupMultibleObjects failed. No remote connection.");
    }
  }
  else
  {
    TString strName(name);
    m_pLock->enter();
    IListIterator<TRemoteObjectMap*>* pIt = m_mapCategory.getIterator();
    while(pIt->hasNextItem())
    {
      TRemoteObjectMap* pCategory =  pIt->getNextItem();
      if (pCategory->hasItem(strName))
      {
        IRemoteObject* pObject = pCategory->getItem(strName);
        pObject->addRef();
        objects->add(pObject);
      }
    }
    m_mapCategory.releaseIterator(pIt);  
    if (objects->getCount() > 0)
    {
      ZLOGGER.printfln(LOGMODE_INFO, "Naming: lookupMultibleObjects sucessfull [%s]. %d found.", 
                       strName.c_str(), 
                       objects->getCount());
      ulRetval = RET_NOERROR;
    }
    m_pLock->leave();
  }
  return ulRetval;
}

/***************************************************************************/
/*! See INaming
*/
/***************************************************************************/
unsigned long MQUALIFIER TNaming::lookupCategory(const IString* cat, IList<IRemoteObject*>* objects)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (cat == NULL || objects == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_bRemote)
  {
    if (m_pDelegationInterface != NULL)
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate lookupCategory.");
      ulRetval = m_pDelegationInterface->lookupCategory(cat, objects);
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate of lookupCategory failed. No remote connection.");
    }
  }
  else
  {
    TString strCat(cat);
    m_pLock->enter();
    if (m_mapCategory.hasItem(strCat))
    {
      TRemoteObjectMap* pCategory = m_mapCategory.getItem(strCat);
      IListIterator<IRemoteObject*>* pIt = pCategory->getIterator();
      while(pIt->hasNextItem())
      {
        IRemoteObject* pObj = pIt->getNextItem();
        pObj->addRef();
        objects->add(pObj);
      }
      pCategory->releaseIterator(pIt);
      if (objects->getCount() > 0)
      {
        ZLOGGER.printfln(LOGMODE_INFO, "Naming: lookupCategory sucessfull [%s]. %d found.", strCat.c_str(), objects->getCount());
        ulRetval = RET_NOERROR;
      }
    }
    m_pLock->leave();
  }
  return ulRetval;
}

/***************************************************************************/
/*! See INaming
*/
/***************************************************************************/
unsigned long MQUALIFIER TNaming::getObjectNames(const IString* cat, IList<IString*>* names)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (cat == NULL || names == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_bRemote)
  {
    if (m_pDelegationInterface != NULL)
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate getObjectNames.");
      ulRetval = m_pDelegationInterface->getObjectNames(cat, names);
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Naming: Delegate of getObjectNames failed. No remote connection.");
    }
  }
  else
  {
    TString strCat(cat);
    m_pLock->enter();
    if (m_mapCategory.hasItem(strCat))
    {
      TRemoteObjectMap* pCategory = m_mapCategory.getItem(strCat);
      IStringMapIterator<IRemoteObject*>* pIt = pCategory->getIterator();
      while(pIt->hasNextItem())
      {
        TString strKey;
        pIt->getNextKey(&strKey);
        names->add(&strKey);
      }
      pCategory->releaseIterator(pIt);
      if (names->getCount() > 0)
      {
        ZLOGGER.printfln(LOGMODE_INFO, "Naming: getObjectNames sucessfull [%s]. %d found.", strCat.c_str(), names->getCount());
        ulRetval = RET_NOERROR;
      }
    }
    m_pLock->leave();
  }
  return ulRetval;

}


/***************************************************************************/
/*! See INaming
*/
/***************************************************************************/
bool MQUALIFIER TNaming::isRemote() const 
{
  return (m_pDelegationInterface != NULL);
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL_LOCK(TNaming, m_pLock);
  INTERFACE_CAST(INaming, INTERFACE_INaming);
MEMORY_MANAGER_IMPL_END;

//#########################################################################

/**************************************************************************/
/*! Creates a stub of the naming service
*/
/**************************************************************************/
TNaming_Stub::TNaming_Stub() : TAbstractStub()
{
}

/**************************************************************************/
/*! Destroys the naming stub
*/
/**************************************************************************/
TNaming_Stub::~TNaming_Stub()
{}

//Methods of INaming
STUB_DELEGATION("bind", unsigned long MQUALIFIER TNaming_Stub::bind(const IString* name, IRemoteObject* object))
  STUB_PARAM("name", name);
  STUB_PARAM_REMOTEREF("object", object);
  STUB_RETURN_ERRORCODE;

STUB_DELEGATION("bind2", unsigned long MQUALIFIER TNaming_Stub::bind2(const IString* name, const IString* cat, IRemoteObject* object))
  STUB_PARAM("name", name);
  STUB_PARAM("cat", cat);
  STUB_PARAM_REMOTEREF("object", object);
  STUB_RETURN_ERRORCODE;

STUB_DELEGATION("rebind", unsigned long MQUALIFIER TNaming_Stub::rebind(const IString* name, IRemoteObject* object))
  STUB_PARAM("name", name);
  STUB_PARAM_REMOTEREF("object", object);
  STUB_RETURN_ERRORCODE;
  
STUB_DELEGATION("rebind2", unsigned long MQUALIFIER TNaming_Stub::rebind2(const IString* name, const IString* cat, IRemoteObject* object))
  STUB_PARAM("name", name);
  STUB_PARAM("cat", cat);
  STUB_PARAM_REMOTEREF("object", object);
  STUB_RETURN_ERRORCODE;

STUB_DELEGATION("unbind", unsigned long MQUALIFIER TNaming_Stub::unbind(const IString* name))
  STUB_PARAM("name", name);
  STUB_RETURN_ERRORCODE;
  
STUB_DELEGATION("unbind2", unsigned long MQUALIFIER TNaming_Stub::unbind2(const IString* name, const IString* cat))
  STUB_PARAM("name", name);
  STUB_PARAM("cat", cat);
  STUB_RETURN_ERRORCODE;
  
STUB_DELEGATION("lookup", unsigned long MQUALIFIER TNaming_Stub::lookup(const IString* name, IRemoteObject** object))
  STUB_PARAM("name", name);
  STUB_RETURN_START;
    STUB_RETPARAM_REMOTEREF("object", object);
    STUB_RETVALUE_LONG(ulRetval);
  STUB_RETURN_END(ulRetval);
  
STUB_DELEGATION("lookup2", unsigned long MQUALIFIER TNaming_Stub::lookup2(const IString* name, const IString* cat, IRemoteObject** object))
  STUB_PARAM("name", name);
  STUB_PARAM("cat", cat);
  STUB_RETURN_START;
    STUB_RETPARAM_REMOTEREF("object", object);
    STUB_RETVALUE_LONG(ulRetval);
  STUB_RETURN_END(ulRetval);
  
STUB_DELEGATION("lookupAny", unsigned long MQUALIFIER TNaming_Stub::lookupAny(const IString* cat, IRemoteObject** object))
  STUB_PARAM("cat", cat);
  STUB_RETURN_START;
    STUB_RETPARAM_REMOTEREF("object", object);
    STUB_RETVALUE_LONG(ulRetval);
  STUB_RETURN_END(ulRetval);
    
STUB_DELEGATION("lookupMultibleObjects", unsigned long MQUALIFIER TNaming_Stub::lookupMultibleObjects(const IString* name, IList<IRemoteObject*>* objects))
  STUB_PARAM("name", name);
  STUB_RETURN_START;
    STUB_RETPARAM_REMOTEREFLIST("objects", objects);
    STUB_RETVALUE_LONG(ulRetval);
  STUB_RETURN_END(ulRetval);

STUB_DELEGATION("lookupCategory", unsigned long MQUALIFIER TNaming_Stub::lookupCategory(const IString* cat, IList<IRemoteObject*>* objects))
  STUB_PARAM("cat", cat);
  STUB_RETURN_START;
    STUB_RETPARAM_REMOTEREFLIST("objects", objects);
    STUB_RETVALUE_LONG(ulRetval);
  STUB_RETURN_END(ulRetval);

STUB_DELEGATION("getObjectNames", unsigned long MQUALIFIER TNaming_Stub::getObjectNames(const IString* cat, IList<IString*>* names))
  STUB_PARAM("cat", cat);
  STUB_RETURN_START;
    STUB_RETPARAM_STRINGLIST("names", names);
    STUB_RETVALUE_LONG(ulRetval);
  STUB_RETURN_END(ulRetval);
  
    
/***************************************************************************/
/* Allways remote
*/
/***************************************************************************/
bool MQUALIFIER TNaming_Stub::isRemote() const 
{
  return true;
}

MEMORY_MANAGER_IMPL(TNaming_Stub);
  INTERFACE_CAST(INaming, INTERFACE_INaming);
MEMORY_MANAGER_IMPL_PARENT_END(TAbstractStub);

