/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : RemoteReference                                               *
 * Package : Remote                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:39:16 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: RemoteReference.cpp,v $
 *   Revision 1.1  2005/11/10 10:39:16  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <zeusbase/Remote/Interfaces/IRemoteObjectStub.hpp>
#include <zeusbase/Remote/RemoteReference.h>
#include <zeusbase/System/ZObjectFactory.h>
#include <zeusbase/System/LoggerManager.h>

USING_NAMESPACE_Zeus
 
SERIAL_CONSTRUCTOR(TRemoteReference);


/***************************************************************************/   
/*! Creates an empty remote reference
*/
/***************************************************************************/   
TRemoteReference::TRemoteReference(): TZObject()
{}

/***************************************************************************/   
/*! Creates a remote reference out of a remote object
    \param object : Remote object
*/
/***************************************************************************/   
TRemoteReference::TRemoteReference(IRemoteObject* object) : TZObject()
{
  m_ulPort = 0;
  
  if (object != NULL)
  {
    object->getClassName(&m_strClassName);
    object->getCodeModuleName(&m_strCodeModuleName);
    object->getConnectionParam(&m_strAddress, &m_ulPort);
  }
}

/***************************************************************************/   
/*! Creates a remote reference 
    \param address : Address of the remote server
    \param port : TCP Port of the server
    \param codemodule : Name of the code module to create the stub object
    \param classname : Classname to create the stub object
*/
/***************************************************************************/   
TRemoteReference::TRemoteReference(TString address, long port, TString codemodule, TString classname) : TZObject()
{
  m_strAddress = address;
  m_ulPort = port;
  m_strCodeModuleName = codemodule;
  m_strClassName = classname;
}
 
/***************************************************************************/   
/*! Destroys the remote reference
*/
/***************************************************************************/   
TRemoteReference::~TRemoteReference()
{}


/***************************************************************************/   
/*! Creates a stub object out of the information of the remote object 
    refefence
    \param obj : Return parameter of the created object
    \retval RET_NOERROR : Stub created
    \retval RET_REQUEST_FAILED : Could not create a stub out of this 
                                 information
*/
/***************************************************************************/   
unsigned long TRemoteReference::createStub(IRemoteObject** obj)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (obj == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    IZUnknown* pObject = NULL;
    TString strClassName = m_strClassName + L"_Stub";
    ZLOGGER.printfln(LOGMODE_INFO, "Create stub class for %s@%s...", m_strClassName.c_str(), m_strCodeModuleName.c_str());
    if (TZObjectFactory::getInstance().createObjectByClassName(&strClassName, &m_strCodeModuleName, &pObject) == RET_NOERROR)
    {
      IRemoteObjectStub* pRemoteObject = NULL;
      if (pObject->askForInterface(INTERFACE_IRemoteObjectStub, (IZUnknown**)&pRemoteObject) == RET_NOERROR)
      {
        pRemoteObject->setClassName(&m_strClassName);
        pRemoteObject->setCodeModuleName(&m_strCodeModuleName);
        pRemoteObject->connect(&m_strAddress, m_ulPort);
        *obj = pRemoteObject;
        ulRetval = RET_NOERROR;
        ZLOGGER.printfln(LOGMODE_INFO, "...done.");
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_INFO, "...failed. Class does not implement IRemoteObjectStub interface");
      }
      pObject->release();
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_INFO, "...failed. Class not registered");
    }
  }
  return ulRetval;
}


/***************************************************************************/   
/*! Returns the port number
*/
/***************************************************************************/   
unsigned long TRemoteReference::getPort()
{
  return m_ulPort;
}

/***************************************************************************/   
/*! Returns the ip address
*/
/***************************************************************************/   
TString TRemoteReference::getAddress()
{
  return m_strAddress;
}

/***************************************************************************/   
/*! Returns the name of the code module
*/
/***************************************************************************/   
TString TRemoteReference::getCodeModule()
{
  return m_strCodeModuleName;
}

/***************************************************************************/   
/*! Returns the name of the class
*/
/***************************************************************************/   
TString TRemoteReference::getClassName()
{
  return m_strClassName;
}

/***************************************************************************/   
/*! See IRemoteObject
*/
/***************************************************************************/   
unsigned long MQUALIFIER TRemoteReference::getClassName(IString* name)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assignStr(&m_strClassName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/   
/*! See IRemoteObject
*/
/***************************************************************************/   
unsigned long MQUALIFIER TRemoteReference::getCodeModuleName(IString* name)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    name->assignStr(&m_strCodeModuleName);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/   
/*! See IRemoteObject
*/
/***************************************************************************/   
unsigned long MQUALIFIER TRemoteReference::getConnectionParam(IString* address, unsigned long* port)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (address != NULL && port != NULL)
  {
    address->assignStr(&m_strAddress);
    *port = m_ulPort;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TRemoteReference);
  INTERFACE_CAST(IRemoteObject, INTERFACE_IRemoteObject);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
MEMORY_MANAGER_IMPL_END;

