/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FingerPrint                                                   *
 * Package : Security                                                      *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:43:51 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: FingerPrint.cpp,v $
 *   Revision 1.2  2005/11/16 11:43:51  bhadorn
 *   Method added to limit the file size for generate the hash.
 *
 *   Revision 1.1  2005/11/10 10:41:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
 
//---------------------------------------------------------------------------

#include <zeusbase/Security/FingerPrint.h>
#include <zeusbase/Security/Algorithmes/Whirlpool.hpp>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/FileInputStream.h>

#include <stdio.h>

USING_NAMESPACE_Zeus

SERIAL_CONSTRUCTOR(TFingerPrint)

/***************************************************************************/
/*! Creates a finger print object
*/
/***************************************************************************/
TFingerPrint::TFingerPrint() :TZObject(), m_lstData(DIGESTBYTES)
{
  m_bGenerated = false;
}

/***************************************************************************/
/*! Destroys the fingerprint
*/
/***************************************************************************/
TFingerPrint::~TFingerPrint()
{
  m_lstData.clear();
}

/***************************************************************************/
/*! Generates a fingerprint using the input buffer
    \pre buffer must be valid
*/
/***************************************************************************/
unsigned long TFingerPrint::generate(const TString& keyname, const unsigned char* buffer, long size)
{
  ZLOGGER.printfln(LOGMODE_DEBUG, "Generate finger print: Input buffer size = %d", size);
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (size > 0)
  {
    unsigned char* pDigest = new unsigned char[DIGESTBYTES];  
    struct NESSIEstruct w;
    
    ::NESSIEinit(&w);
    ::NESSIEadd(buffer, 8*size, &w);
    ::NESSIEfinalize(&w, pDigest);
    
    ulRetval = m_lstData.setArray((char*)pDigest, DIGESTBYTES);
    delete [] pDigest;
    
    if (ulRetval == RET_NOERROR)
    {
      m_bGenerated = true;
      m_strKeyName = keyname;
    }
  }
  return ulRetval;
}


/***************************************************************************/
/*! Generate a finger print out of an array of data
   \param keyname : Name of the key
   \param arr : Array of data
   \retval RET_NOERROR : Finger print generared
   \retval RET_INVALID_PARAMETER : arr is NULL
   \retval RET_REQUEST_FAILED : Finger print already generated
*/
/***************************************************************************/
unsigned long TFingerPrint::generatePrint(const TString& keyname, IByteArray* arr)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (m_bGenerated)
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  else if (arr != NULL)
  {
    ulRetval = generate(keyname, (unsigned char*)arr->getArray(), arr->getCount());
  }
  //display(m_pPrint, DIGESTBYTES); printf("\n\n");
  return ulRetval;
}


/***************************************************************************/
/*! Generate a finger print out of a text
   \param keyname : Name of the key
   \param data : String data
   \retval RET_NOERROR : Finger print generared
   \retval RET_INVALID_PARAMETER : arr is NULL
   \retval RET_REQUEST_FAILED : Finger print already generated
*/
/***************************************************************************/
unsigned long TFingerPrint::generatePrint(const TString& keyname, const TString& data)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (m_bGenerated)
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  else
  {
    ulRetval = generate(keyname, (unsigned char*)data.c_str(), data.getSize()+1);
  }
  //display(m_pPrint, DIGESTBYTES); printf("\n\n");
  return ulRetval;
}


/***************************************************************************/
/*! Generate a finger print out of file
   \param keyname : Name of the key
   \param filename : Name of the file
   \param max_size : Maximum size of the file to check. All bytes above this
                     maximum will be ignored.
   \retval RET_NOERROR : Finger print generared
   \retval RET_REQUEST_FAILED : Finger print already generated
   \retval RET_INVALID_DATA : Could not access the file
*/
/***************************************************************************/
unsigned long TFingerPrint::generatePrintFromFile(const TString& keyname, const TString& filename, unsigned long max_size /*=2000000*/)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  if (m_bGenerated)
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  else
  {
    long lSize = TFile(filename).getSize();
    TFileInputStream* pStream = new TFileInputStream(filename);
    ZLOGGER.printfln(LOGMODE_DEBUG, "Generate finger from file [Name=%s, Size=%d, Avail=%d]", filename.c_str(), lSize, pStream->available());
    if (pStream->available() && lSize > 0)
    {      
      //Limit the size to 2MB
      if (lSize > (long)max_size)
      {
        lSize = max_size;
      }
      
      char* pCBuffer = new char[lSize];
      long lRetSize = 0;
      if (pStream->read(pCBuffer, lSize, &lRetSize) == RET_NOERROR)
      {
        ulRetval = generate(keyname, (unsigned char*)pCBuffer, lRetSize);
      }
      else
      {
        ulRetval = RET_INVALID_DATA;
      }
      delete [] pCBuffer;
      
      pStream->close();
    }
    
    pStream->release();
  }
  return ulRetval;
}

/***************************************************************************/
/*! Returns the fingerprint as a byte array
    \param value : Return parameter 
    \retval RET_NOERROR : Successfully returned
    \retval RET_REQUEST_FAILED : No Finger print has been generated
    \retval RET_INVALID_PARAMETER : Invalid parameter received
*/
/***************************************************************************/
unsigned long TFingerPrint::getFingerPrint(IByteArray* value) const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (value != NULL)
  {
    if (!m_bGenerated)
    {
      ulRetval = RET_REQUEST_FAILED;
    }
    else
    {
      m_lstData.copyToArray(value, 0, m_lstData.getCount()-1);
      ulRetval = RET_NOERROR;
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! Returns the name of the finger print
    \return key name
*/
/***************************************************************************/
TString TFingerPrint::getKeyName() const
{
  return m_strKeyName;
}


/***************************************************************************/
/*! Compares two finger prints to be equal
    \param inpar : Object to check
    \retval true : Equal
    \retval false : different
*/
/***************************************************************************/
bool TFingerPrint::operator==(const TFingerPrint& inpar) const
{
  TByteArray aData;
  inpar.getFingerPrint(&aData);
//  printf("\nDebug DigestBuffer=%s; Size=%d\n", aData.getArray(), aData.getCount());
//  printf("\nDebug DigestBuffer=%s; Size=%d\n", m_lstData.getArray(), m_lstData.getCount());
  
  return (m_lstData == inpar.m_lstData && m_strKeyName == inpar.m_strKeyName);
}

/***************************************************************************/
/*! Compares two finger prints to be equal
    \param inpar : Object to check
    \retval true : Equal
    \retval false : different
*/
/***************************************************************************/
bool TFingerPrint::operator!=(const TFingerPrint& inpar) const
{
  TByteArray aData;
  inpar.getFingerPrint(&aData);
  return !(aData == m_lstData);
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TFingerPrint);
  INTERFACE_CAST(ISerializable, INTERFACE_ISerializable);
MEMORY_MANAGER_IMPL_END;

