/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : SecurityManager                                               *
 * Package : Security                                                      *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:43:51 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: SecurityManager.cpp,v $
 *   Revision 1.2  2005/11/16 11:43:51  bhadorn
 *   Method added to limit the file size for generate the hash.
 *
 *   Revision 1.1  2005/11/10 10:41:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  
 
#include <zeusbase/Security/SecurityManager.h> 
#include <zeusbase/System/File.h> 
#include <zeusbase/System/FileInputStream.h> 
#include <zeusbase/System/FileOutputStream.h>
#include <zeusbase/System/ZObjectFactory.h> 
#include <zeusbase/System/LoggerManager.h> 

USING_NAMESPACE_Zeus

//Initialises the static instance
TSecurityManager TSecurityManager::m_Instance;

/***************************************************************************/
/*! Returns the signelton object
*/
/***************************************************************************/
TSecurityManager& TSecurityManager::getInstance()
{
  return m_Instance;
}

/***************************************************************************/
/*! Creates the security manager
*/
/***************************************************************************/
TSecurityManager::TSecurityManager()
{
  m_bLoaded = false;
  m_bChanged = false;
  m_ulMaxFileSize = 2000000;
}

/***************************************************************************/
/*! Destroys the security manager
*/
/***************************************************************************/
TSecurityManager::~TSecurityManager()
{
  closeRegistry();
}


/***************************************************************************/
/*! Max size of file to evaluate
*/
/***************************************************************************/
void TSecurityManager::setMaxFileSize(unsigned long max_size)
{
  m_ulMaxFileSize = max_size;
}

/***************************************************************************/
/*! Registers a code module at the security registration
    \param name : Name of the code module
    \param path : Path of the code module
    \retval RET_NOERROR : Module registered
    \retval RET_INVALID_DATA : Module not found for registration
    \retval RET_REQUEST_FAILED : Module already registrated
    \retval RET_NOT_READY : Security data base not loaded
*/
/***************************************************************************/
unsigned long TSecurityManager::registerModule(const TString& name, const TString& path)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (!m_bLoaded)
  {
    ulRetval = RET_NOT_READY;
  }
  else if (!m_mapReg.hasItem(name))
  {
    TFingerPrint* pFingerPrint = new TFingerPrint();
    
    if (pFingerPrint->generatePrintFromFile(name, path, m_ulMaxFileSize) == RET_NOERROR)
    {
      m_mapReg.setItem(name, pFingerPrint);
      ulRetval = RET_NOERROR;
      m_bChanged = true;
    }
    else
    {
      pFingerPrint->release();
      ulRetval = RET_INVALID_DATA;
    }
  }
  return ulRetval;
}


/***************************************************************************/
/*! Unregisteres a code module from registry. This is nessesary if you have
    to register the same module with a new finger print.
    \param name : Name of the code module
    \retval RET_NOERROR: Code module removed
    \retval RET_REQUEST_FAILED : No such code module registered
    \retval RET_NOT_READY : Security data base not loaded
*/
/***************************************************************************/
unsigned long TSecurityManager::unregisterModule(const TString& name)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (!m_bLoaded)
  {
    ulRetval = RET_NOT_READY;
  }
  else if (m_mapReg.hasItem(name))
  {
    TFingerPrint* pFingerPrint = m_mapReg.getItem(name);
    pFingerPrint->release();
    m_mapReg.deleteItem(name);
    m_bChanged = true;
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! Closes the registry data base and if the flag m_bChanged is set 
    it will store the data back to file first
*/
/***************************************************************************/
void TSecurityManager::closeRegistry()
{
  if (m_bLoaded)
  {
    //Store data
    if (m_bChanged)
    {
      TFileOutputStream* pStream = new TFileOutputStream(m_strDataBaseName, false);
      
      IListIterator<TFingerPrint*>* pIt =  m_mapReg.getIterator();
      while(pIt->hasNextItem())
      {
        TByteArray aData;
        pIt->getNextItem()->serialize(&aData);
        pStream->writeArray(&aData);
      }
      m_mapReg.releaseIterator(pIt);
      
      pStream->close();
      pStream->release();
    }
    
    //Remove all instances
    IListIterator<TFingerPrint*>* pIt =  m_mapReg.getIterator();
    while(pIt->hasNextItem())
    {
      pIt->getNextItem()->release();
    }
    m_mapReg.releaseIterator(pIt);
    m_mapReg.clear();
  
  }
  m_bChanged = false;
  m_bLoaded = false;
}


/***************************************************************************/
/*! Returns all registered module names of the security manager
    \param list : Return value
    \retval RET_NOERROR : List returned
    \retval RET_INVALID_PARAMETER : Invalid parameter received
*/
/***************************************************************************/
unsigned long TSecurityManager::getModules(TStringList* list)
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (list != NULL)
  {
    IListIterator<TFingerPrint*>* pIt =  m_mapReg.getIterator();
    while(pIt->hasNextItem())
    {
      TString strName = pIt->getNextItem()->getKeyName();
      list->add(&strName);
    }
    m_mapReg.releaseIterator(pIt);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! Checks if the data base already exists, if not it will create a new data
    base.
    \param filename : Name of the data base.
*/
/***************************************************************************/
void TSecurityManager::checkAndCreateNewDB(const TString& filename)
{
  if (!TFile(filename).exists())
  {
    TFileOutputStream* pFile = new TFileOutputStream(filename);
    pFile->close();
    pFile->release();
  }
}

/***************************************************************************/
/*! Loads the registry from a file
    \param filename : Name of the file
    \retval RET_REQUEST_FAILED : Could not load file
    \retval RET_NOERROR : File loaded;
*/
/***************************************************************************/
unsigned long TSecurityManager::loadRegistry(const TString& filename)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  closeRegistry();
  checkAndCreateNewDB(filename);
  
  TFileInputStream* pStream = new TFileInputStream(filename);
  if (pStream->available())
  {
    while(!pStream->isEmpty())
    {
      TByteArray aData;
      if(pStream->readArray(&aData) == RET_NOERROR)
      {
        ISerializable* pPrint = NULL;
        if (aData.getCount() > 0 && TZObjectFactory::getInstance().createObjectFromStream(&aData, &pPrint) == RET_NOERROR)
        {
          TFingerPrint* pFingerPrint = (TFingerPrint*)pPrint;
          m_mapReg.setItem(pFingerPrint->getKeyName(), pFingerPrint);
          ZLOGGER.printfln(LOGMODE_DEBUG, "Finger print extracted. %s", pFingerPrint->getKeyName().c_str());
/*          TByteArray aArray;
          pFingerPrint->getFingerPrint(&aArray);
          long lCount = aArray.getCount();
          for (int i = 0; i < lCount; i++) 
          {
            if (i%32 == 0) 
            {
              printf("\n");
            }
            if (i%8 == 0) 
            {
              printf(" ");
            }
            printf("%02X", (unsigned char)aArray[i]);
          }
          printf("\n\n");
*/
        }
        else
        {
          ZLOGGER.printfln(LOGMODE_ERROR, "Could not extract the finger print out of the data base.");
        }
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_ERROR, "Error reading the security data base.");
        break;
      }
    }
    m_strDataBaseName = filename;
    m_bLoaded = true;
    ulRetval = RET_NOERROR;
  }
  pStream->release();
  
  return ulRetval;
}

/***************************************************************************/
/*! this method checks if a finger print is valid or not. The given finger 
    print will be evaluated with the entries inside the registry
    \param print : Finger print to check
    \retval true : Finger print is valid
    \retval false: Finger print is not valid
*/
/***************************************************************************/
bool TSecurityManager::eval(const TFingerPrint* print) const
{
  bool bRetval = false;
  if (print != NULL && m_bLoaded)
  {
    TString strKeyName = print->getKeyName();
    if (m_mapReg.hasItem(strKeyName))
    {
      const TFingerPrint* pCheck = m_mapReg.getItemCopy(strKeyName);
      if (*pCheck == *print)
      {
        bRetval = true;
      }
    }
  }
  return bRetval;
}
    
/***************************************************************************/
/*! this method checks if the code module given by name and path is valid.
    This will generate the finger print itself.
    \param name : Name of the code module
    \param path : path of the code module
    \retval true : Finger print is valid
    \retval false: Finger print is not valid
*/
/***************************************************************************/
bool TSecurityManager::evalCodeModule(const TString& name, const TString& path) const
{
  bool bRetval = false;
  TFingerPrint* pFingerPrint = new TFingerPrint();
  if (pFingerPrint->generatePrintFromFile(name, path, m_ulMaxFileSize) == RET_NOERROR)
  {
    bRetval = this->eval(pFingerPrint);
  }
  pFingerPrint->release();
  return bRetval;
}

