/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CriticalSection                                               *
 * Package : System                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:53:26 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CriticalSection.cpp,v $
 *   Revision 1.2  2005/11/25 09:53:26  bhadorn
 *   Eliminate compiler warnings
 *
 *   Revision 1.1  2005/11/10 10:18:25  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

 
#include <zeusbase/System/CriticalSection.h>
#include <zeusbase/System/Logger.h>

#ifdef LINUX
  #include <errno.h>
#else
  #include <windows.h>
#endif


USING_NAMESPACE_Zeus

/**************************************************************************/
/*! Creates a critical section object
*/
/**************************************************************************/
TCriticalSection::TCriticalSection(TLockType type /*= ltFast*/) : TZObject()
{
  #ifdef LINUX
    ::pthread_mutexattr_init(&m_Attribute);
    
    switch(type)
    {
      case ltError : 
        ::pthread_mutexattr_settype(&m_Attribute, PTHREAD_MUTEX_ERRORCHECK_NP);
      break;
      
      case ltRecursive:
        ::pthread_mutexattr_settype(&m_Attribute, PTHREAD_MUTEX_RECURSIVE_NP);
      break;
      
      default:
        ::pthread_mutexattr_settype(&m_Attribute, PTHREAD_MUTEX_FAST_NP);      
    }
    ::pthread_mutex_init(&m_Mutex, &m_Attribute);
  #else
    switch(type)
    {
      default:
        ::InitializeCriticalSection(&m_CriticalSection);
      break;
    }
  #endif

  m_lCount = 0;
}


/**************************************************************************/
/*! Destroys a critical section object
*/
/**************************************************************************/
TCriticalSection::~TCriticalSection()
{
  //Only checking if the mutex is unlocked
  #ifdef LINUX
    ::pthread_mutexattr_destroy(&m_Attribute);
    ::pthread_mutex_destroy(&m_Mutex);
  #else
    ::DeleteCriticalSection(&m_CriticalSection);
  #endif
}

/**************************************************************************/
/*! Enters the lock
*/
/**************************************************************************/
void MQUALIFIER TCriticalSection::enter()
{
  #ifdef LINUX
    ::pthread_mutex_lock(&m_Mutex);
  #else
    ::EnterCriticalSection(&m_CriticalSection);
  #endif

  m_lCount++;
  //TLogger::getLogger().printfln(m_lCount);
}


/**************************************************************************/
/*! Leaves the lock
*/
/**************************************************************************/
void MQUALIFIER TCriticalSection::leave()
{
  m_lCount--;
  //TLogger::getLogger().printfln(m_lCount);
  #ifdef LINUX
    ::pthread_mutex_unlock(&m_Mutex);
  #else
    ::LeaveCriticalSection(&m_CriticalSection);
  #endif
}


/**************************************************************************/
/*! Checks if the lock is active.
    \retval true : active (an other thread is using the lock)
    \retval false : lock is free
*/
/**************************************************************************/
bool TCriticalSection::isLocked()
{
  bool bRetval = false;
  #ifdef LINUX
    bRetval = (::pthread_mutex_trylock(&m_Mutex) == EBUSY);
  #else
    bRetval = ::TryEnterCriticalSection(&m_CriticalSection);
  #endif

  return bRetval;
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TCriticalSection);
MEMORY_MANAGER_IMPL_END;

