/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : File                                                          *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:50:04 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: File.cpp,v $
 *   Revision 1.2  2005/11/25 09:50:04  bhadorn
 *   Added defines to be win32 compatible
 *
 *   Revision 1.1  2005/11/10 10:18:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <zeusbase/System/File.h> 

#include <string.h>
#include <stdlib.h>
#include <errno.h>

#ifdef LINUX
  #include <wchar.h>
  #include <unistd.h>
  #include <sys/types.h>
  #include <sys/stat.h>
#else
  #include <sys/types.h>
  #include <sys/stat.h>
  #include <io.h>
  #include <fcntl.h>
  #include <stdio.h>
  
  #define W_OK      02
  #define R_OK      04
#endif


USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates an new Directory wrapper object
*/
/***************************************************************************/
TFile::TFile(TString abs_name) : TZObject()
{
  m_strAbsName = abs_name;
}


/***************************************************************************/
/*! Destroys the Directory wrapper object
*/
/***************************************************************************/
TFile::~TFile()
{}

/***************************************************************************/
/*! Returns the absolute name of the directory
*/
/***************************************************************************/
TString TFile::getAbsoluteName()
{
  return m_strAbsName;
}

/***************************************************************************/
/*! Checks if the file exists
    \retval true : File exists
    \retval false : File doesnt exist
*/
/***************************************************************************/
bool TFile::exists() const
{
  bool bRetval  = (::access(m_strAbsName.c_str(), 0x00) == 0);
  return bRetval;
}

/***************************************************************************/
/*! This method is used to extract the filename out of the whole path
*/
/***************************************************************************/
TString TFile::extractFileName()
{
  TString strRetval = m_strAbsName;
  long lPos = findLastPos(m_strAbsName, L'/');
  if(lPos == -1)
  {
    lPos = findLastPos(m_strAbsName, L'\\');
  }
  if (lPos > -1)
  {
    strRetval = m_strAbsName.getSubString(lPos+1,m_strAbsName.getSize()-1);
  }
  return strRetval;
}

/***************************************************************************/
/*! This method is used to extract the file extentsion out of the whole path
*/
/***************************************************************************/
TString TFile::extractFileExt()
{
  TString strRetval;
  TString strFileName = this->extractFileName();
  long lPos = this->findLastPos(strFileName, L'.');
  if (lPos != -1)
  {
    strRetval = strFileName.getSubString(lPos+1,strFileName.getSize()-1);
  }
  return strRetval;
}


/***************************************************************************/
/*! This method is used to extract the file base name out of the whole path
*/
/***************************************************************************/
TString TFile::extractFileNameBase()
{
  TString strRetval = this->extractFileName();
  long lPos = this->findLastPos(strRetval, L'.');
  if (lPos != -1)
  {
    strRetval = strRetval.getSubString(0,lPos-1);
  }
  return strRetval;
}


/***************************************************************************/
/*! Search the last position of a specific character in the filename
    \param str : base string
    \param chr : Character to find
    \retval > -1 : Position of the character
    \retval -1 : Character not found
*/
/***************************************************************************/
long TFile::findLastPos(const TString& str, wchar_t chr)
{
  long lRetval = -1;
  wchar_t* pBase = str.c_bstr();
  wchar_t* pPos = wcsrchr(pBase, chr);
  if (pPos != NULL)
  {
    lRetval = (pPos - pBase);
  }
  return lRetval;
}

/***************************************************************************/
/*! Deletes the file
    \retval true : File deleted
    \retval false: could not delete file
*/
/***************************************************************************/
bool TFile::deleteFile()
{
  bool bRetval = false;
  if (::unlink(m_strAbsName.c_str()) == 0)
  {
    bRetval = true;
  }
  return bRetval;
}

/***************************************************************************/
/*! Copies the file and renames it
    \param targetname : Name of the copied file
    \retval true : File deleted
    \retval false: could not delete file
*/
/***************************************************************************/
bool TFile::copyTo(TString targetname)
{
  #ifdef LINUX
    targetname.c_bstr();
    return false;
  #else
    return ::CopyFile(m_strAbsName.c_str(), targetname.c_str(), false);
  #endif
}


/***************************************************************************/
/*! Checks if the file can be written
    \retval true : Able to write file
    \retval false: Unable to write file
*/
/***************************************************************************/
bool TFile::canWrite()
{
  bool bRetval  = (::access(m_strAbsName.c_str(), W_OK) == 0);
  return bRetval;
}

/***************************************************************************/
/*! Checks if the file can be read
    \retval true : Able to read file
    \retval false: Unable to read file
*/
/***************************************************************************/
bool TFile::canRead()
{
  bool bRetval  = (::access(m_strAbsName.c_str(), R_OK) == 0);
  return bRetval;
}

/***************************************************************************/
/*! Reads the size of the file
    \return File size in bytes
*/
/***************************************************************************/
long TFile::getSize()
{
  long lRetval = 0;
  if (exists())
  {
  #ifdef LINUX
    struct stat Buffer;
    if (::stat(m_strAbsName.c_str(), &Buffer) == 0)
    {
      lRetval = Buffer.st_size;
    }
  #else
    struct stat Buffer;
    if (::stat(m_strAbsName.c_str(), &Buffer) == 0)
    {
      lRetval = Buffer.st_size;
    }
  #endif
  }
  return lRetval;
}
