/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IXObjectFactory                                               *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IXObjectFactory.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IXObjectFactoryHPP
#define IXObjectFactoryHPP

#include <zeusbase/System/Interfaces/IXObject.hpp>
#include <zeusbase/System/Interfaces/IXMLNode.hpp>
#include <zeusbase/System/Interfaces/IXMLDocument.hpp>


BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! This interface defines the x object factory. It can create a x object
    tree out of a xml stream or xml file. 
*/
/***************************************************************************/
class IXObjectFactory : public IZUnknown
{
  public:  

    /***********************************************************************/
    /*! This creates a xrootobject out of a xml stream.
        \param stream : Stream data (xml format)
        \param obj : Return parameter of created xrootobject
        \param onlyroot : Creates only the root object
        \param dounfreeze : If this flag is set, the object tree will be 
            unfrozen after creation
        \retval RET_UNREGISTERED_CLASS : Class is not registered
        \retval RET_REQUEST_FAILED : No valid XML
        \retval RET_INTERNAL_ERROR : Internal Error occured
        \retval RET_INVALID_DATA : Not a valid XML Format
        \retval RET_NOERROR : Object created
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER createObjectFromStream(const IString* stream, 
                                                            IXObject** obj, 
                                                            bool onlyroot = true, 
                                                            bool dounfreeze = true) = 0;
    
    /***********************************************************************/
    /*! This creates only a xrootobject out of a xml stream.
        \param filename : File name 
        \param obj : Return parameter of created xrootobject
        \param onlyroot : Creates only the root object
        \param dounfreeze : If this flag is set, the object tree will be 
            unfrozen after creation
        \retval RET_UNREGISTERED_CLASS : Class is not registered
        \retval RET_REQUEST_FAILED : No valid XML
        \retval RET_INTERNAL_ERROR : Internal Error occured
        \retval RET_INVALID_DATA : Not a valid XML Format
        \retval RET_NOERROR : Object created
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER createObjectFromFile(const IString* filename, 
                                                          IXObject** obj, 
                                                          bool onlyroot = true, 
                                                          bool dounfreeze = true)=0;
    
    /***********************************************************************/
    /*! This creates all child objects of a existing parent object. This 
        method can been called to create all children to a later point of 
        time.
        \param parentnode : Parent XML Node
        \param parent : Parent Object
        \param dounfreeze : If this flag is set, the child object will be 
            unfrozen after creation
        \retval RET_INTERNAL_ERROR : Internal Error occured
        \retval RET_INVALID_DATA : Not a valid XML Format
        \retval RET_NOERROR : Object created
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER createChildObjects(IXMLNode* parentnode, 
                                                        IXObject* parent, 
                                                        bool dounfreeze = true)=0;
        
    /***********************************************************************/
    /*! This creates all child objects of a new XML node and add them to the
        parent
        \param new_node : New XML Node
        \param parent : Parent Object
        \param child : return of the child object
        \param dounfreeze : If this flag is set, the child object will be 
            unfrozen after creation
        \retval RET_INTERNAL_ERROR : Internal Error occured
        \retval RET_INVALID_DATA : Not a valid XML Format
        \retval RET_NOERROR : Object created
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER addNewChildObjects(IXMLNode* new_node, 
                                                        IXObject* parent, 
                                                        IXObject** child,
                                                        bool dounfreeze = true)=0;
    
    /***********************************************************************/
    /*! This method registeres a factory method to create a specific xobject
        named by classname
        \param classname : Name of class
        \param funct : Function to call
        \return Handle. If the handle is < 0 the registration failed
    */
    /***********************************************************************/
    virtual long MQUALIFIER registerXClass(const IString* classname, IXObject::FCreateXObject funct)=0;
    
    /***********************************************************************/
    /*! This method unregisteres a factory method 
        \param handle : Handle of the registration
        \param classname : Name of class
        \retval true : Class unregistered
        \retval false : Could not unregister class
    */
    /***********************************************************************/
    virtual bool MQUALIFIER unregisterXClass(long handle, const IString* classname)=0;
    
};

END_NAMESPACE_Zeus

#endif
