/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : IZVariant                                                     *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:53 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: IZVariant.hpp,v $
 *   Revision 1.1  2005/11/10 10:22:53  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef IZVariantHPP
#define IZVariantHPP
 

#include <zeusbase/System/Interfaces/ISerializable.hpp>
#include <zeusbase/System/Interfaces/IString.hpp>
#include <zeusbase/System/Interfaces/IList.hpp>

BEGIN_NAMESPACE_Zeus

/***************************************************************************/
/*! Datatypes of a variant 
*/
/***************************************************************************/
enum TZVariantType 
{
  varEmpty,    /*!< Unknown or Empty data */
  varByte,     /*!< a simple byte (8bit) */
  varLong,     /*!< a long value */
  varDouble,   /*!< a double value */ 
  varString,   /*!< a wide char based string */ 
  varByteArray,/*!< a byte array (stream) */
  varObject,   /*!< a object */ 
  varObjectList/*!< list of objects */
};

/***************************************************************************/
/*! This interface defines a variant datatype. This datatype is designed
    to wrap regular primitive datatypes and complex serializabl;e objects.
    Its used for serializing and deserializing data over the network
*/
/***************************************************************************/
class IZVariant : public ISerializable
{
  public:
    /***********************************************************************/
    /*! Returns the datatype of the variant
        \return datatype (see TZVariantType)
    */
    /***********************************************************************/
    virtual TZVariantType MQUALIFIER getType() const=0;
    
    /***********************************************************************/
    /*! Returns the name of the variant type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getVariantName(IString* name) const=0;
    
    /***********************************************************************/
    /*! Returns the variant data as a byte value
        \param ret : Return value
        \retval RET_NOERROR : Value returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not return the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getByte(char* ret) const=0;
    
    /***********************************************************************/
    /*! Returns the variant data as a long value
        \param ret : Return value
        \retval RET_NOERROR : Value returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not return the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getLong(long* ret) const=0;
    
    /***********************************************************************/
    /*! Returns the variant data as a double value
        \param ret : Return value
        \retval RET_NOERROR : Value returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not return the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getDouble(double* ret) const=0;
    
    /***********************************************************************/
    /*! Returns the variant data as a string value
        \param ret : Return value
        \retval RET_NOERROR : Value returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not return the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getString(IString* ret) const=0;
    
    /***********************************************************************/
    /*! Returns the variant data as a byte array
        \param ret : Return value
        \retval RET_NOERROR : Value returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not return the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getByteArray(IByteArray* ret) const=0;
    
    /***********************************************************************/
    /*! Returns the variant data as an object
        \param ret : Return value
        \retval RET_NOERROR : Value returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not return the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getObject(ISerializable** ret) const=0;
    
    /***********************************************************************/
    /*! Returns the variant data as an object list
        \param ret : Return value
        \retval RET_NOERROR : Value returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not return the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER getObjectList(IList<ISerializable*>* ret) const=0;
  
    /***********************************************************************/
    /*! Sets the name of the variant type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setVariantName(IString* name)=0;
    
    /***********************************************************************/
    /*! Sets a byte value to the variant
        \param val : value to be set
        \retval RET_NOERROR : Value set
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not set the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setByte(char val)=0;
    
    /***********************************************************************/
    /*! Sets a long value to the variant
        \param val : value to be set
        \retval RET_NOERROR : Value set
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not set the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setLong(long val)=0;
    
    /***********************************************************************/
    /*! Sets a double value to the variant
        \param val : value to be set
        \retval RET_NOERROR : Value set
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not set the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setDouble(double val)=0;
    
    /***********************************************************************/
    /*! Sets a string value to the variant
        \param val : value to be set
        \retval RET_NOERROR : Value set
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not set the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setString(const IString* val)=0;
    
    /***********************************************************************/
    /*! Sets an array to the variant
        \param val : value to be set
        \retval RET_NOERROR : Value set
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not set the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setByteArray(const IByteArray* val)=0;
    
    /***********************************************************************/
    /*! Sets an object to the variant
        \param val : value to be set
        \retval RET_NOERROR : Value set
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not set the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setObject(ISerializable* val)=0;
    
    /***********************************************************************/
    /*! Sets an aobject list to the variant
        \param val : value to be set
        \retval RET_NOERROR : Value set
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not set the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER setObjectList(IList<ISerializable*>* val)=0;

    /***********************************************************************/
    /*! assigns a variant to this variant
        \param inpar : Variant object to set
        \retval RET_NOERROR : Values set
        \retval RET_INVALID_PARAMETER : Invalid parameter received
        \retval RET_REQUEST_FAILED : Could not set the requested type
    */
    /***********************************************************************/
    virtual unsigned long MQUALIFIER assign(const IZVariant* inpar)=0;

    /***********************************************************************/
    /*! Checks if two variants are equal
        \param inpar : Variant to check
        \retval true : Variants are equal
        \retval false : Variants are not equal
    */
    /***********************************************************************/
    virtual bool MQUALIFIER equals(const IZVariant* inpar) const=0;
};

END_NAMESPACE_Zeus

#endif
