/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Code Module                                                   *
 * Package : Code Package                                                  *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:20:17 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: LibLoader.cpp,v $
 *   Revision 1.1  2005/11/10 10:20:17  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <zeusbase/System/LibLoader.h>

#include <zeusbase/System/SingletonManager.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/XObjectFactory.h>

#ifdef LINUX
  #include <dlfcn.h>
  #include <stdio.h>
#else
  #include <Windows.h>

#endif

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Erstellen des Library Loaders
*/
/***************************************************************************/
TLibLoader::TLibLoader(TString name, const TString& filename) : TZObject(name), m_lstRegFunction(RRegFunction(), 10)
{
  m_strFileName = filename;
  m_pHandle = NULL;
  m_bRegistered = false;
}


/***************************************************************************/
/*! Zerstoeren des Library Loaders
*/
/***************************************************************************/
TLibLoader::~TLibLoader()
{
  if (m_pHandle != NULL)
  {
    this->unload();
  }
}
    
    
/***************************************************************************/
/*! Returns the file name of the library
*/
/***************************************************************************/
TString TLibLoader::getFileName()
{
  return m_strFileName;
}
    
/***************************************************************************/
/*! Register a code module library
*/
/***************************************************************************/
unsigned long TLibLoader::registerLibrary()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pHandle == NULL)
  {
    load();
  }
  
  if (m_pHandle != NULL && !m_bRegistered)
  {
    typedef unsigned long MODULIMPORT_POST(*FRegisterLibrary)(ISingletonManager* manager);
    TString strFunct = LIB_FUNCTION_REG;

    FRegisterLibrary fRegisterLibrary = (FRegisterLibrary)this->getFunction(strFunct);
    if (fRegisterLibrary != NULL)
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Try to register library...");
      if (fRegisterLibrary(&TSingletonManager::getInstance()) == RET_NOERROR)
      {
        ZLOGGER.printfln(LOGMODE_DEBUG, "...done");
        m_bRegistered = true;
        ulRetval = RET_NOERROR;
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_DEBUG, "...failed");
      }
    }
  }
  return ulRetval;
}


/***************************************************************************/
/*! Unregister a code module library
*/
/***************************************************************************/
unsigned long TLibLoader::unregisterLibrary()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;

  if (m_pHandle != NULL && m_bRegistered)
  {
    while(!m_lstRegFunction.isEmpty())
    {
      RRegFunction& RegFunction = m_lstRegFunction[0];
      TXObjectFactory::getInstance().unregisterXClass(RegFunction.lHandle, &RegFunction.strName);
      m_lstRegFunction.deleteItem(0);
    }

    typedef unsigned long MODULIMPORT_POST(*FUnregisterLibrary)(ISingletonManager* manager);
    TString strFunct = LIB_FUNCTION_UNREG;

    FUnregisterLibrary fUnregisterLibrary = (FUnregisterLibrary)this->getFunction(strFunct);
    if (fUnregisterLibrary != NULL)
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Try to unregister library...");
      if (fUnregisterLibrary(&TSingletonManager::getInstance()) == RET_NOERROR)
      {
        ZLOGGER.printfln(LOGMODE_DEBUG, "...done");
        m_bRegistered = false;
        ulRetval = RET_NOERROR;
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_DEBUG, "...failed");
      }
    }
  }
  return ulRetval;
}


/***************************************************************************/
/*! Register a exported function
*/
/***************************************************************************/
unsigned long TLibLoader::registerXClass(const IString* classname, long* index)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pHandle != NULL)
  {
    TString strFunctName = TString(L"create") + classname;
    IXObject::FCreateXObject pFunct = (IXObject::FCreateXObject)this->getFunction(strFunctName);

    if (pFunct != NULL)
    {
      long lHandle = TXObjectFactory::getInstance().registerXClass(classname, pFunct);
      if (lHandle != -1)
      {
        RRegFunction RegFunction;
        RegFunction.strName = classname;
        RegFunction.lHandle = lHandle;
        m_lstRegFunction.add(RegFunction);
        ulRetval = RET_NOERROR;
      }
      else
      {
        ulRetval = RET_REQUEST_FAILED;
      }

      if (index != NULL)
      {
        *index = lHandle;
      }
    }
    else
    {
      ulRetval = RET_INVALID_DATA;
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! This method creates an object out of the codemodule
    \param classname : Name of the class
    \param iface : Reference to return
    \retval RET_NOERROR : Object created
    \retval RET_REQUEST_FAILED : Class not exported
    \retval RET_INVALID_PARAMETER : Invalid parameter received
    \retval RET_INTERNAL_ERROR : Internal error occured
*/
/***************************************************************************/
unsigned long TLibLoader::createObject(const TString& classname, IZUnknown** iface)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pHandle == NULL)
  {
    load();
  }
  
  if (iface != NULL && m_pHandle != NULL)
  {
    typedef unsigned long MODULIMPORT_POST(*FCreateClass)(IZUnknown** iface);
    TString strFunct = TString(L"create") + classname;

    FCreateClass fCreateClass = (FCreateClass)this->getFunction(strFunct);
    if (fCreateClass != NULL)
    {
      ZLOGGER.printfln(LOGMODE_DEBUG, "Try to create object of class %s...", strFunct.c_str());
      if (fCreateClass(iface) == RET_NOERROR)
      {
        ZLOGGER.printfln(LOGMODE_DEBUG, "...done");
        ulRetval = RET_NOERROR;
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_DEBUG, "...failed");
      }
    }
  }
  else if (m_pHandle != NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  return ulRetval;
}

/***************************************************************************/
/*! Checks if the libray is loaded
    \retval true : Lib is loaded
    \retval false: Lib is not loaded
*/
/***************************************************************************/
bool TLibLoader::isLoaded()
{
  return (m_pHandle != NULL);
}


/***************************************************************************/
/*! loads the Library
    \retval RET_NOERROR : Library loaded
    \retval RET_INVALID_DATA : not able to load the library 
*/
/***************************************************************************/
unsigned long TLibLoader::load()
{
  ZLOGGER.printf(LOGMODE_DEBUG, "Try to load library: %s...", m_strFileName.c_str());
  unsigned long ulRetval = RET_INVALID_DATA;

  //-------------------------------------------------------------------------
  //The Library is already loaded
  if (m_pHandle != NULL)
  {
    unload();
  }

  //-------------------------------------------------------------------------
  //Loading the library
  #ifdef LINUX
    m_pHandle = dlopen(m_strFileName.c_str() , RTLD_LAZY);//RTLD_LAZY);//RTLD_LAZY);RTLD_NOW,RTLD_GLOBAL
    ::dlerror(); //clear errors
  #else
    m_pHandle = ::LoadLibrary(m_strFileName.c_str());
  #endif
  
  if (m_pHandle != NULL)
  {
    ulRetval = RET_NOERROR;
    ZLOGGER.println(LOGMODE_DEBUG, "...done");
    
    //Tries to load classes
    registerLibrary();
  }
  else
  {
    ulRetval = RET_INVALID_DATA;
    ZLOGGER.println(LOGMODE_DEBUG, "...failed");
    ZLOGGER.println(LOGMODE_ERROR, "Unable to load the library %s", m_strFileName.c_str());
  }

  return ulRetval;
}

/***************************************************************************/
/*! Entladen der Library

*/
/***************************************************************************/
unsigned long TLibLoader::unload()
{
  ZLOGGER.printf(LOGMODE_DEBUG, "Unload library: %s...", m_strFileName.c_str());
  unsigned long ulRetval = RET_REQUEST_FAILED;

  if (m_pHandle != NULL)
  {
    //Tries to unload classes
    unregisterLibrary();
    #ifdef LINUX
      ::dlclose(m_pHandle);
    #else
      ::FreeLibrary(m_pHandle);
    #endif

    m_bRegistered = false;
    ulRetval = RET_NOERROR;
    ZLOGGER.println(LOGMODE_DEBUG, "...done");
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
    ZLOGGER.println(LOGMODE_DEBUG, "...done. [No handle released]");
  }
  m_pHandle = NULL;
  return ulRetval;
}


/***************************************************************************/
/*! Ermitteln des LibraryFunktions Handle
    \return Handle (NULL = Funktion oder Library nicht geladen)
*/
/***************************************************************************/
void* TLibLoader::getFunction(TString& functname)
{
  void* pRetval = NULL;
  if (m_pHandle == NULL)
  {
    load();
  }
  
  //Versuchen die Funktion zu laden
  if (m_pHandle != NULL)
  {
    #ifdef LINUX
      pRetval = ::dlsym(m_pHandle, functname.c_str());
    #else
      pRetval = ::GetProcAddress((HINSTANCE)m_pHandle, functname.c_str());
    #endif
  }
  else
  {
    pRetval = NULL;
  }
  return pRetval;
}

