/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : LibraryManager                                                *
 * Package : System                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:20:17 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: LibraryManager.cpp,v $
 *   Revision 1.1  2005/11/10 10:20:17  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
#include <zeusbase/System/LibraryManager.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/StringMap.hpp>
#include <zeusbase/System/LibLoader.h>
#include <zeusbase/Security/SecurityManager.h>

USING_NAMESPACE_Zeus

TLibraryManager TLibraryManager::m_LibManager(L"LibManager");


/***************************************************************************/
/*! Returns the singleton object
*/
/***************************************************************************/
TLibraryManager& TLibraryManager::getInstance()
{
  return m_LibManager;
}


/***************************************************************************/
/*! Creates the library manager
    \param name : Name of the object
*/
/***************************************************************************/
TLibraryManager::TLibraryManager(TString name) : TZObject(name)
{
  m_pDelegation = NULL;
  m_Security = slNoSecurity;
  #ifdef LINUX
    m_strCodeModulePath = L"./modules/";
    m_strExtentions = L".so";
  #else
    m_strCodeModulePath = L".\\modules\\";
    m_strExtentions = L".dll";
  #endif
}

/***************************************************************************/
/*! Destroy the library manager
*/
/***************************************************************************/
TLibraryManager::~TLibraryManager()
{
  this->removeAll();
  releaseDelegationInterface();
}

/***************************************************************************/
/*! Sets a delegation interface. All method calls will be delegated.
    \param manager : Interface
*/
/***************************************************************************/
void TLibraryManager::setDelegationInterface(ILibraryManager* manager)
{
  releaseDelegationInterface();
  m_pDelegation = manager;
  if (m_pDelegation != NULL)
  {
    m_pDelegation->addRef();
  }
}


/***************************************************************************/
/*! Release the delegation interface
*/
/***************************************************************************/
void TLibraryManager::releaseDelegationInterface()
{
  if (m_pDelegation != NULL)
  {
    m_pDelegation->release();
    m_pDelegation = NULL;
  }
}


/***************************************************************************/
/*! See ILibraryManager
*/
/***************************************************************************/
unsigned long MQUALIFIER TLibraryManager::createObject(const IString* codemodule, const IString* classname, IZUnknown** iface)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pDelegation != NULL)
  {
    ulRetval = m_pDelegation->createObject(codemodule, classname, iface);
  }
  else if (classname == NULL || codemodule == NULL || iface == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strCodeModule(codemodule);
    TString strClassName(classname);
    if (!m_mapLibraries.hasItem(strCodeModule))
    {
      ulRetval = this->addCodeModule(strCodeModule);
    }

    TLibLoader* pLib = m_mapLibraries.getItem(strCodeModule);
    if (pLib != NULL)
    {
      ulRetval = pLib->createObject(strClassName, iface);
    }
  }
  return ulRetval;
}


/***************************************************************************/
/*! See ILibraryManager
*/
/***************************************************************************/
unsigned long MQUALIFIER TLibraryManager::registerXClass(const IString* codemodule, const IString* classname, long* index)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pDelegation != NULL)
  {
    ulRetval = m_pDelegation->registerXClass(codemodule, classname, index);
  }
  else if (codemodule == NULL || classname == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strCodeModule(codemodule);
    if (!m_mapLibraries.hasItem(strCodeModule))
    {
      ulRetval = this->addCodeModule(strCodeModule);
    }

    TLibLoader* pLib = m_mapLibraries.getItem(strCodeModule);
    if (pLib != NULL)
    {
      ulRetval = pLib->registerXClass(classname, index);
    }
  }

  return ulRetval;
}


/***************************************************************************/
/*! This adds a new code module to the library manager. The code module will
    be loaded if possible. If the loading fails, the module will be added 
    anyway, so a conflict can been seen.
    \param name : Name of the code module. 
    \return For return values see addCodeModule(name, path)
*/
/***************************************************************************/
unsigned long TLibraryManager::addCodeModule(const TString& name)
{
  TString strFilePath = m_strCodeModulePath + name + m_strExtentions;
  return addCodeModule(name, strFilePath);
}

/***************************************************************************/
/*! Adds a libaray to the manager
    \param name : Name of the code module
    \param path : code module path
    \retval RET_NOERROR : Code module loaded
    \retval RET_ACCESS_DENIED : Security policy denies to load this code 
            module 
    \retval RET_REQUEST_FAILED : Library already loaded
    \retval RET_INVALID_DATA : Could not load library. (Invalid name)
*/
/***************************************************************************/
unsigned long TLibraryManager::addCodeModule(const TString& name, const TString& path)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  ZLOGGER.indent();
  if (!m_mapLibraries.hasItem(name))
  {    
    TLibLoader* pLib = new TLibLoader(name, path);
    ///Security code must come here
    if (m_Security == slUseSecurity)
    {
      if (TSecurityManager::getInstance().evalCodeModule(name, path))
      {
        ulRetval = pLib->load();
        m_mapLibraries.setItem(name, pLib);
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_ERROR, "Security failed. Could not load code module %s", name.c_str());
        pLib->release();
      }
    }
    //No security activated
    else
    {
      ulRetval = pLib->load();
      m_mapLibraries.setItem(name, pLib);
    }
  }
  
  ZLOGGER.unindent();
  return ulRetval;
}  


/***************************************************************************/
/*! Checks if a code module has been added already.
    \param name : Name of the code module
    \retval true : Code module has been added
    \retval false : Code module has not been added
*/
/***************************************************************************/
bool TLibraryManager::hasCodeModule(const TString& name)
{
  return m_mapLibraries.hasItem(name);
}

/***************************************************************************/
/*! Sets the code module path.
    \param path : basepath of the modules
*/
/***************************************************************************/
void TLibraryManager::setCodeModulePath(const TString& path)
{
  m_strCodeModulePath = path;
}
    
/***************************************************************************/
/*! Sets the security level
    \param mode : new security mode
*/
/***************************************************************************/
void TLibraryManager::setSecurity(TSecurityLevel mode)
{
  m_Security = mode;
}

/***************************************************************************/
/*! Removes a code module from the list
    \param name : Name of the code module
    \retval RET_NOERROR : Code module removed
    \retval RET_REQUEST_FAILED: Code module was not registered
*/
/***************************************************************************/
unsigned long TLibraryManager::remove(const TString& name)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  if (m_mapLibraries.hasItem(name))
  {
    TLibLoader* pLib = m_mapLibraries.getItem(name);
    m_mapLibraries.deleteItem(name);
    
    pLib->unload();
    pLib->release();
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}


/***************************************************************************/
/*! Removes all libraries
*/
/***************************************************************************/
void TLibraryManager::removeAll()
{
  IListIterator<TLibLoader*>* pIterator = m_mapLibraries.getIterator();
  while(pIterator->hasNextItem())
  {
    TLibLoader* pLib = pIterator->getNextItem();
    pLib->unregisterLibrary();
  }
  pIterator->reset();

  while(pIterator->hasNextItem())
  {
    TLibLoader* pLib = pIterator->getNextItem();
    pLib->unload();
    pLib->release();
  }
  m_mapLibraries.releaseIterator(pIterator);
  m_mapLibraries.clear();
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TLibraryManager);
MEMORY_MANAGER_IMPL_END;

