/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ObjectPtr                                                  *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:20:44 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ObjectPtr.hpp,v $
 *   Revision 1.1  2005/11/10 10:20:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef ObjectPtrHPP
#define ObjectPtrHPP

#include <zeusbase/System/ZObject.h>

BEGIN_NAMESPACE_Zeus


/**************************************************************************/
/*! This object wrapps an interface of the zeus frameworks and manages its
    pointer. The release() and addRef() methods are called automaticaly
    destroying or adding new interfaces of the same kind.
*/
/**************************************************************************/
template <class T> class TZObjectPtr : public TZObject
{
  public:
    
    /***********************************************************************/
    /*! Creates an Interface Smart pointer 
    */
    /***********************************************************************/
    TZObjectPtr()
    {
      m_pInterface = NULL;
    }
    
    /***********************************************************************/
    /*! Creates an Interface Smart pointer 
        \param iface : Interface
    */
    /***********************************************************************/
    TZObjectPtr(T* iface)
    {
      m_pInterface = iface;
      if(m_pInterface != NULL)
      {
        m_pInterface->addRef();
      }
    }
      
    /***********************************************************************/
    /*! Creates an Interface Smart pointer 
        \param wrapper : Interfacewrapper
    */
    /***********************************************************************/
    TZObjectPtr(const TZObjectPtr& wrapper)
    {
      m_pInterface = wrapper.m_pInterface;
      if(m_pInterface != NULL)
      {
        m_pInterface->addRef();
      }
    }
   
    /***********************************************************************/
    /*! Destroys the smart pointer
    */
    /***********************************************************************/
    virtual ~TZObjectPtr()
    {
      if (this->m_pInterface != NULL)
      {
        this->m_pInterface->release();
      }
    }

    
    /***********************************************************************/
    /*! Assign operator 
        \param iface : Interface
    */
    /***********************************************************************/
    TZObjectPtr<T>& operator=(T* iface)
    {
      if (this->m_pInterface != NULL)
      {
        this->m_pInterface->release();
      }
      m_pInterface = iface;
      if(m_pInterface != NULL)
      {
        m_pInterface->addRef();
      }
      return *this;
    }

    /***********************************************************************/
    /*! Assign operator 
        \param wrapper : Interfacewrapper
    */
    /***********************************************************************/
    TZObjectPtr<T>& operator=(const TZObjectPtr<T>& wrapper)
    {
      if (this->m_pInterface != NULL)
      {
        this->m_pInterface->release();
      }
      m_pInterface = wrapper.m_pInterface;
      if(m_pInterface != NULL)
      {
        m_pInterface->addRef();
      }
      return *this;
    }
   
    /***********************************************************************/
    /*! Equals operator 
        \param wrapper : Interface wrapper
    */
    /***********************************************************************/
    bool operator==(const TZObjectPtr<T>& wrapper) const { return (this->m_pInterface == wrapper.m_pInterface); }
    
    /***********************************************************************/
    /*! Equals operator 
        \param wrapper : Interface wrapper
    */
    /***********************************************************************/
    bool operator!=(const TZObjectPtr<T>& wrapper) const { return (this->p != wrapper.m_pInterface); }
    
    /***********************************************************************/
    /*! Equals operator 
        \param iface : Interface 
    */
    /***********************************************************************/
    bool operator==(const T* iface) const { return (this->m_pInterface == iface); }
    
    /***********************************************************************/
    /*! Equals operator 
        \param iface : Interface 
    */
    /***********************************************************************/
    bool operator!=(const T* iface) const { return (this->m_pInterface != iface); }
    
    /***********************************************************************/
    /*! Operator is used to return the interface pointer and to access to
        all methods of this interface
        \return pointer to interface
    */
    /***********************************************************************/
    T* operator->() {return m_pInterface; }
    
    /***********************************************************************/
    /*! Operator is used to return the interface pointer and to access to
        all methods of this interface
        \return pointer to interface
    */
    /***********************************************************************/
    const T* operator->() const {return m_pInterface; }
    
    /***********************************************************************/
    /*! Returns the dereferenced pointer of an interface
        \return pointer to interface
    */
    /***********************************************************************/
    T& operator*() const {return *m_pInterface; }
    
    
    /***********************************************************************/
    /*! Returns the dereferenced pointer of an interface
        \return pointer to interface
    */
    /***********************************************************************/
    operator T*() const {return m_pInterface; }
      
  protected:
    //Interface pointer
    T* m_pInterface;
};

END_NAMESPACE_Zeus

#endif
