/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : PropertyFile                                                  *
 * Package : system                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:20:44 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: PropertyFile.cpp,v $
 *   Revision 1.1  2005/11/10 10:20:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
 
#include <zeusbase/System/LoggerManager.h> 
#include <zeusbase/System/FileInputStream.h> 
#include <zeusbase/System/PropertyFile.h> 
#include <zeusbase/System/ByteArray.hpp> 
 

//USES_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a new property file
*/
/***************************************************************************/
TPropertyFile::TPropertyFile(TString filename) : TFile(filename)
{
}

/***************************************************************************/
/*! Destroys a property file
*/
/***************************************************************************/
TPropertyFile::~TPropertyFile()
{
  removeAll();
}


/***************************************************************************/
/*! Remove all properties
*/
/***************************************************************************/
void TPropertyFile::removeAll()
{
  while(m_lstProperties.getCount() > 0)
  {
    delete m_lstProperties[0];
    m_lstProperties.deleteItem(0);
  }
}

/***************************************************************************/
/*! Loads the data from the file
    \retval RET_NOERROR : Properties loaded
    \retval RET_REQUEST_FAILED : Could not load properties 
*/
/***************************************************************************/
unsigned long TPropertyFile::load()
{
  enum TScannerState {ScanStateName, ScanStateValue, ScanStateComment};
  
  unsigned long ulRetval = RET_REQUEST_FAILED;
  TFileInputStream* pStream = new TFileInputStream(this);
  long lSize = this->getSize();
  
  if (pStream->available() && lSize > 0)
  {
    char* cBuffer = new char[lSize];
    long lRet = 0;
    if (pStream->read(cBuffer, lSize, &lRet) == RET_NOERROR)
    {
      TByteArray aInput(cBuffer, lRet);
      bool bError = false;
      TScannerState ScannerState = ScanStateName;
      long lStart = 0;
      long lEnd = 0;
      TString strName;
      TString strValue;
      for (int i = 0; i < lRet; i++)
      {
        switch(aInput[i])
        {
          // Separator of properties
          case '=' :
          {
            if (ScannerState == ScanStateName && lStart < lEnd-1)
            {
              TByteArray aName;
              aInput.copyToArray(&aName,lStart, lEnd-1);
              strName = TString::transcode(aName);
              ScannerState = ScanStateValue;
              lStart = i+1;
              lEnd = i+1;
            }
            else if (ScannerState != ScanStateComment)
            {
              bError = true; 
            }
          }
          break;
          
          // Following Spaces are ignored
          case ' ' :
            if (ScannerState == ScanStateValue)
            {
              lEnd++;
            }
          break;
          
          
          //End of a row
          case '\n' : //fall through
          case '\r' : 
          {
            if (ScannerState == ScanStateValue)
            {
              if (lStart < lEnd)
              {
                TByteArray aValue;
                aInput.copyToArray(&aValue,lStart, lEnd-1);
                strValue = TString::transcode(aValue);
              }
              else
              {
                strValue = L"";
              }
              ZLOGGER.printfln(LOGMODE_DEBUG, "Property %s=%s added.", strName.c_str(), strValue.c_str());
              this->addProperty(&strName, &strValue);
            }
            
            //Ignore empty lines and invalid stuff
            else
            {/**/}
            lStart = i+1;
            lEnd = i+1;
            ScannerState = ScanStateName;
          }
          break;
          
          //Comment
          case '#' :
          {
            //Comment is at the end of a valid line
            if (ScannerState == ScanStateValue)
            {
              if (lStart < lEnd)
              {
                TByteArray aValue;
                aInput.copyToArray(&aValue,lStart, lEnd);
                strValue = TString::transcode(aValue);
              }
              else
              {
                strValue = L"";
              }
            ZLOGGER.printfln(LOGMODE_DEBUG, "Property %s=%s added.", strName.c_str(), strValue.c_str());
              this->addProperty(&strName, &strValue);
            }
            
            //Ignore full commented lines
            else
            {/**/}
            
            lStart = i+1;
            lEnd = i+1;
            ScannerState = ScanStateComment;
          }
          break;
          
          default:
          {
            if (ScannerState != ScanStateComment)
            {
              lEnd++;
            }
          }
        }
        
        if (bError)
        {
          ZLOGGER.printfln(LOGMODE_DEBUG, "Error occured reading the file %s on character %d.", this->getAbsoluteName().c_str(), i);
        }
        bError = false;
      }
      ulRetval = RET_NOERROR;
    }
    delete [] cBuffer;
    
    pStream->close();
  }
  pStream->release();
  return ulRetval;
}

/***************************************************************************/
/*! Siehe IPropertyAccess
*/
/***************************************************************************/
unsigned long MQUALIFIER TPropertyFile::addProperty(IString* name, IString* value)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL || value == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (this->setPropertyValue(name, value) != RET_NOERROR)
  {
    RProperty* pProp = new RProperty();
    pProp->strName = name;
    pProp->strValue = value;
    
    m_lstProperties.add(pProp);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! Siehe IPropertyAccess
*/
/***************************************************************************/
bool MQUALIFIER TPropertyFile::existsProperty(IString* name)
{
  bool bRetval = false;
  
  IListIterator<RProperty*>* pIterator = m_lstProperties.getIterator();
  RProperty* pProp = pIterator->getNextItem();
  
  while(pProp != NULL && !bRetval)
  {
    if (pProp->strName.equalsStr(name))
    {
      bRetval = true;
    }
    pProp = pIterator->getNextItem();
  }
  m_lstProperties.releaseIterator(pIterator);
  return bRetval;
}

/***************************************************************************/
/*! Siehe IPropertyAccess
*/
/***************************************************************************/
unsigned long MQUALIFIER TPropertyFile::getPropertyNames(IStringList* list)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (list == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    list->clear();
    
    IListIterator<RProperty*>* pIterator = m_lstProperties.getIterator();
    RProperty* pProp = pIterator->getNextItem();
    
    while(pProp != NULL)
    {
      list->add(&pProp->strName);
      pProp = pIterator->getNextItem();
    }
    m_lstProperties.releaseIterator(pIterator);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! Siehe IPropertyAccess
*/
/***************************************************************************/
unsigned long MQUALIFIER TPropertyFile::getPropertyValue(IString* name, IString* value)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL || value == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    IListIterator<RProperty*>* pIterator = m_lstProperties.getIterator();
    RProperty* pProp = pIterator->getNextItem();
    
    while(pProp != NULL && ulRetval != RET_NOERROR)
    {
      if (pProp->strName.equalsStr(name))
      {
        value->assignStr(&pProp->strValue);
        ulRetval = RET_NOERROR;
      }
      pProp = pIterator->getNextItem();
    }
    m_lstProperties.releaseIterator(pIterator);
  }
  return ulRetval;
}
    
/***************************************************************************/
/*! Siehe IPropertyAccess
*/
/***************************************************************************/
unsigned long MQUALIFIER TPropertyFile::setPropertyValue(IString* name, IString* value)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (name == NULL || value == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    IListIterator<RProperty*>* pIterator = m_lstProperties.getIterator();
    RProperty* pProp = pIterator->getNextItem();
    
    while(pProp != NULL && ulRetval != RET_NOERROR)
    {
      if (pProp->strName.equalsStr(name))
      {
        pProp->strValue.assignStr(value);
        ulRetval = RET_NOERROR;
      }
      pProp = pIterator->getNextItem();
    }
    m_lstProperties.releaseIterator(pIterator);
  }
  return ulRetval;
}
 

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TPropertyFile);
  INTERFACE_CAST(IPropertyAccess, INTERFACE_IPropertyAccess);
MEMORY_MANAGER_IMPL_PARENT_END(TFile);


