/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ServerSocket                                                  *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/16 11:29:16 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ServerSocket.cpp,v $
 *   Revision 1.2  2005/11/16 11:29:16  bhadorn
 *   Added dynamic binding for servers. The port addess will be choosen dynamically
 *
 *   Revision 1.1  2005/11/10 10:20:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

 
#include "zeusbase/System/ServerSocket.h"
#include "zeusbase/System/LoggerManager.h"
#include "zeusbase/System/Time.h"

#ifdef LINUX
  #include <unistd.h>
  #include <stdio.h>
  #include <stdlib.h>
  #include <string.h>
  #include <errno.h>
  #include <sys/select.h>
#else
  #include <Windows.h>
#endif

USING_NAMESPACE_Zeus

/**************************************************************************/
/*! Creates a Server Socket
    \param port : Listening port
    \param addr : address or name of the server
*/
/**************************************************************************/
TServerSocket::TServerSocket(TString addr, unsigned long port) : TSocket(addr,port)
{
  m_lMaxClients = 100;
  m_bResusable = false;
}

/**************************************************************************/
/*! Destroys a Server Socket
*/
/**************************************************************************/
TServerSocket::~TServerSocket()
{

}

/**************************************************************************/
/*! Call this method to reuse the server connection even if the 
    previous server connection is still waiting for closing. This will avoid
    problems reconnection to the address:port.
    \note to use this method you need valid socket descripitor. Therefore 
          you must call bind() first and then setReUsable().
*/
/**************************************************************************/
void TServerSocket::setReUsable()
{
  if (m_iSocketFD >= 0)
  {
  #ifdef LINUX
    int iReuseAddr = 1;
    ::setsockopt(m_iSocketFD, SOL_SOCKET, SO_REUSEADDR, &iReuseAddr, sizeof(iReuseAddr)); 
  #else
    
  #endif
  }
  m_bResusable = true;
}

/**************************************************************************/
/*! Creats a valid server socket descripitor and binds the server to 
    the port of the network interface
    \param dynamic_binding : Binds dynamicaly to a unused port if this flag 
               is set. Default is static binding.
    \retval RET_NOERROR : successful binded
    \retval RET_INTERNAL_ERROR : Could not create server socket
    \retval RET_REQUEST_FAILED ; Could not bind server socket
*/
/**************************************************************************/
unsigned long TServerSocket::bind(bool dynamic_binding /*= false*/)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  m_iSocketFD = socket(AF_INET, SOCK_STREAM,0);
  if (m_iSocketFD >= 0)
  {    
    if (m_bResusable)
    {
      setReUsable();
    }
    
    //-------------------------------------------------------------
    //dynamic binding
    if (dynamic_binding)
    {
      //Trys of dynamic connections
      long lCount = 0;
      long lMax = 100;
      unsigned long ulPort = TTime::random(3000, 65000);
      while(lCount < lMax && ulRetval != RET_NOERROR)
      {
        m_Address.sin_port = htons((unsigned short)ulPort);
        ulRetval = bindInternal();
        if (ulRetval != RET_NOERROR)
        {
          ulPort += TTime::random(1, 5);
        }
        lCount++;
      }
    }
    
    //-------------------------------------------------------------
    //static binding
    else
    {
      ulRetval = bindInternal();
    }
  }
  else
  {
    ulRetval = RET_INTERNAL_ERROR;
  }
  return ulRetval;
}
  
/**************************************************************************/
/*! Binds  the server to an address and a port
    \retval RET_NOERROR : Socket bound (connected)
    \retval RET_REQUEST_FAILED : binding failed. Address is busy
*/
/**************************************************************************/
unsigned long TServerSocket::bindInternal()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  int iServerAddrLen = sizeof(m_Address);
  if( ::bind(m_iSocketFD, (struct sockaddr*)&m_Address, iServerAddrLen) >= 0)
  {
    ::listen(m_iSocketFD,m_lMaxClients);
    ulRetval = RET_NOERROR;
    m_bConnected = true;
  }
  return ulRetval;
}

  
/**************************************************************************/
/*! Accepts a incomming connection to a client socket. The Socket will be
    wrapped into a TSocket Object. This method blocks the calling thread
    unitl a client connects to this server socket.
    For non blocking server sockets this method will return imediately. 
    \note If the server socket is used in a child thread of the 
          applications its important that the socket is non blocking, 
          otherwise the thread will hang.
    \param client : return parameter of the client
    \retval RET_INVALID_PARAMETER : Invalid parameter received
    \retval RET_NOERROR : Client socket accepted
    \retval RET_REQUEST_FAILED : Invalid socket
*/
/**************************************************************************/
unsigned long TServerSocket::accept(TSocket** client)
{
  return acceptInternal(client, 0.001);
}

/**************************************************************************/
/*! Same as the accept() method, but with a timeout. The time is only 
    effective if the socket is non blocking. To use non blocking call 
    setBlockable(false). For blocking server sockets this method is 
    identical to accept().
    \param client : return parameter of the client
    \param timeout : The time out in seconds 
    \return Error code see accept()
*/
/**************************************************************************/
unsigned long TServerSocket::acceptTO(TSocket** client, double timeout)
{
  return acceptInternal(client, timeout);
}

/**************************************************************************/
/* Internal accepting method
*/
/**************************************************************************/
unsigned long TServerSocket::acceptInternal(TSocket** client, double timeout)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (client == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }  
  else
  {
    fd_set rfds;
    FD_ZERO(&rfds);
    FD_SET(m_iSocketFD, &rfds);
    
//     ZLOGGER.printfln(LOGMODE_DEBUG, "Select...");
    int iRet = 0;
    if (!m_bBlockable)
    {
      struct timeval tv;
      
      long lSecs = (long)timeout;
      long lMicroSecs = (long)((double)(timeout - lSecs) * 1000);
      tv.tv_sec = lSecs;
      tv.tv_usec = lMicroSecs;
//       ZLOGGER.printfln(LOGMODE_DEBUG, "Time set [%d sec, %d usec]", lSecs, lMicroSecs);
      iRet = ::select(m_iSocketFD + 1, &rfds, NULL, NULL, &tv);
    }
    else
    {
      iRet = ::select(m_iSocketFD + 1, &rfds, NULL, NULL, NULL);
    }
//     ZLOGGER.printfln(LOGMODE_DEBUG, "Select returned [%d]", iRet);
    if (iRet)
    //if (iRet && listen(m_iSocketFD,m_lMaxClients) >= 0)
    {
      struct sockaddr_in ClientAddr;
      int iClientAddrLen = sizeof(ClientAddr);

      #ifdef LINUX
        int iClientFD = ::accept(m_iSocketFD, (struct sockaddr*)&ClientAddr, (socklen_t*)&iClientAddrLen);
      #else
        int iClientFD = ::accept(m_iSocketFD,(struct sockaddr*)&ClientAddr, &iClientAddrLen);
      #endif
      if (iClientFD >= 0)
      {
        *client = new TSocket(iClientFD, ClientAddr);
        ulRetval = RET_NOERROR;
      }
      else
      {
        ulRetval = RET_REQUEST_FAILED;
      }    
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  return ulRetval;
}

