/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : Socket                                                        *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 14:58:26 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Socket.cpp,v $
 *   Revision 1.4  2005/11/25 14:58:26  bhadorn
 *   Sending more than 65000 bytes supported.
 *
 *   Revision 1.3  2005/11/25 09:54:01  bhadorn
 *   Implemented read and write methods for array and long values.
 *
 *   Revision 1.2  2005/11/16 11:32:21  bhadorn
 *   Improved error evaluation when reading fails
 *
 *   Revision 1.1  2005/11/10 10:20:44  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "zeusbase/System/Socket.h"

#ifdef LINUX
  #include <unistd.h>
  #include <fcntl.h>
  #include <stdio.h>
  #include <ctype.h>
  #include <netdb.h>
  #include <string.h>
  #include <errno.h>
#else
  #include <windows.h>
  #include <winsock.h>
#endif

USING_NAMESPACE_Zeus

unsigned long TSocket::m_ulObjectCount = 0;

/**************************************************************************/
/*! Creates a Socket
*/
/**************************************************************************/
TSocket::TSocket() : TZObject()
{
  m_iSocketFD = -1;
  m_ulObjectCount++;
  m_bConnected = false;
  m_bBlockable = true;
  initSocket();
}

/**************************************************************************/
/*! Creates a Client Socket
    \param port : Listening port
    \param addr : address or name of the server
*/
/**************************************************************************/
TSocket::TSocket(TString addr, unsigned long port)
{
  m_iSocketFD = -1;
  m_bBlockable = true;
  m_ulObjectCount++;
  initSocket();

  struct hostent* pHost;
  if (::isalpha(addr.getChar(0)))
  {
    pHost = gethostbyname(addr.c_str());
  }
  else
  {
    unsigned long ulAddr = inet_addr(addr.c_str());
    pHost = ::gethostbyaddr( (char*)&ulAddr, 4, AF_INET);
  }

  if (pHost != NULL)
  {
    memcpy( &(m_Address.sin_addr), pHost->h_addr, pHost->h_length );
    m_Address.sin_family = pHost->h_addrtype;
    m_Address.sin_port = htons((unsigned short)port);
    //m_Address.sin_addr.s_addr = inet_addr(addr.c_str());
  }
  m_bConnected = false;
}

/**************************************************************************/
/*! Creates a Socket out of a file descripitor
    \param fd : file desc.
    \param addr : address object
*/
/**************************************************************************/
TSocket::TSocket(int fd, struct sockaddr_in& addr)
{
  m_ulObjectCount++;
  m_bBlockable = true;
  initSocket();

  m_Address = addr;
  m_iSocketFD = fd;
  m_bConnected = (m_iSocketFD >= 0);
}

/**************************************************************************/
/*! Destroys the Socket
*/
/**************************************************************************/
TSocket::~TSocket()
{
  disconnect();
  m_ulObjectCount--;
//  if (m_ulObjectCount == 0)
  {
    #ifdef LINUX
    #else
      WSACleanup();
    #endif
  }
}

/**************************************************************************/
/*! Prepares the socket for communication
*/
/**************************************************************************/
void TSocket::initSocket()
{
  memset( &m_Address, 0, sizeof(m_Address));
//  if (m_ulObjectCount == 1)
  {
    #ifdef LINUX
    #else

      WSADATA WsaData;
      WSAStartup( 0x202, &WsaData );
    #endif
  }
}

/**************************************************************************/
/*! sets or removes the blockable mode of a socket
*/
/**************************************************************************/
void TSocket::setBlockable(bool mode)
{
  if (m_iSocketFD >= 0)
  {
  #ifdef LINUX
    int iOldFlags = fcntl(m_iSocketFD, F_GETFL);
    if (!mode) /*NON Blocking*/
    {
      fcntl(m_iSocketFD, F_SETFL, iOldFlags | O_NONBLOCK);
    }
    else
    {
      unsigned long ulOldFlags = iOldFlags;
      unsigned long ulFlags = O_NONBLOCK;
      
      ulOldFlags &= ~ulFlags;
      fcntl(m_iSocketFD, F_SETFL, ulOldFlags);
    }
  #else
    if (!mode)   /*NON Blocking*/
    {
      unsigned long ulArg = 1;
      ::ioctlsocket( m_iSocketFD, FIONBIO, &ulArg );
    }
    else
    {
      unsigned long ulArg = 0;
      ::ioctlsocket( m_iSocketFD, FIONBIO, &ulArg );
    }
  #endif
  }
  m_bBlockable = mode;
}

/**************************************************************************/
/*! Returns the socket status
    \retval true : Socket is connected
    \retval false : socket is disconnected
*/
/**************************************************************************/
bool TSocket::isConnected()
{
  return m_bConnected;
}

/**************************************************************************/
/*! see IOutputStream
*/
/**************************************************************************/
unsigned long MQUALIFIER TSocket::writeString(const IString* data)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (data == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strData(data);
    ulRetval = this->write(strData.c_str(), strData.getSize());
  }
  return ulRetval;
}

/**************************************************************************/
/*! Returns the address of a socket
*/
/**************************************************************************/
TString TSocket::getAddress()
{
  char* cBuffer = inet_ntoa(m_Address.sin_addr);
  TString strRetval = cBuffer;
  return strRetval;
}

/**************************************************************************/
/*! Returns the port of the socket
*/
/**************************************************************************/
unsigned long TSocket::getPort()
{
  return ntohs(m_Address.sin_port);
}

/**************************************************************************/
/*! Trys to connect to a server
*/
/**************************************************************************/
unsigned long TSocket::connect()
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (!this->isConnected())
  {
    int iAddrLen = sizeof(m_Address);
    m_iSocketFD = socket(AF_INET, SOCK_STREAM,0);
    if (m_iSocketFD >= 0)
    {
      setBlockable(m_bBlockable);
      if (::connect(m_iSocketFD, (struct sockaddr*)&m_Address, iAddrLen) >= 0)
      {
        ulRetval = RET_NOERROR;
        m_bConnected = true;
      }
      else
      {
        ulRetval = RET_REQUEST_FAILED;
      }
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }

  return ulRetval;
}

/**************************************************************************/
/*! Disconnect
*/
/**************************************************************************/
unsigned long TSocket::disconnect()
{
  if (m_iSocketFD != -1)
  {
    #ifdef LINUX
      ::close(m_iSocketFD);
    #else
      ::closesocket(m_iSocketFD);
    #endif
  }

  m_iSocketFD = -1;
  m_bConnected = false;
  return RET_NOERROR;
}



/**************************************************************************/
/*! see IInputStream
*/
/**************************************************************************/
unsigned long MQUALIFIER TSocket::read(char* buffer, long buffer_size, long* valid_size) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (buffer == NULL || valid_size == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_bConnected)
  {
    int iRet = 0;
    #ifdef LINUX
      iRet = ::read(m_iSocketFD, buffer,buffer_size);
    #else
      iRet = ::recv(m_iSocketFD, buffer,buffer_size, 0);
    #endif
    
    if (iRet >= 0)
    {
      *valid_size = iRet;
      ulRetval = RET_NOERROR;
    }
    else
    {
      *valid_size = 0;
      int iError = errno;
      switch(iError)
      {
        case EAGAIN : 
          ulRetval = RET_NOERROR;
        break;
        
        case EINTR :
        default: //fall through
          ulRetval = RET_REQUEST_FAILED;
        break;
      }
    }
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}

/**************************************************************************/
/*! see IInputStream
*/
/**************************************************************************/
long MQUALIFIER TSocket::readLong() const
{
  long lRetval = 0;
  long lSize = sizeof(long);
  char* pBuffer = new char[lSize];
  ::memset(pBuffer, 0, sizeof(char)*lSize);
  
  if (this->read(pBuffer, lSize, &lSize) == RET_NOERROR && lSize == 4)
  {
    lRetval = *((long*)pBuffer);    
  }
  delete [] pBuffer;
  return lRetval;
}

/**************************************************************************/
/*! see IInputStream
*/
/**************************************************************************/
unsigned long MQUALIFIER TSocket::readArray(IByteArray* data) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (data == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    long lSize = this->readLong();
    long lTotalReceived = 0;
    if (lSize > 0)
    {
      //printf("Buffer size is %d\n", lSize);
      char* pBuffer = new char[lSize];
      ::memset(pBuffer, 0, sizeof(char)*lSize);
      ulRetval = RET_NOERROR;
      
      while(lSize > lTotalReceived && ulRetval == RET_NOERROR)
      {
        long lReceived = 0;
        long lSizeToRead = lSize-lTotalReceived;
        
        //Head to insert data
        char* pHead = (char*)(pBuffer + lTotalReceived);
        
        ulRetval = this->read(pHead, lSizeToRead, &lReceived);
        lTotalReceived += lReceived;
        //printf("Data reading: Size=%d; Received=%d\n", lSize, lTotalReceived);
      }
      if (ulRetval != RET_NOERROR)
      {
        //printf("Error on connection [%d]\n",ulRetval);
      }

      data->setArray(pBuffer, lSize);
      delete [] pBuffer;    
    }
  }
  return ulRetval;
}


/**************************************************************************/
/*! see IOutputStream
*/
/**************************************************************************/
unsigned long MQUALIFIER TSocket::write(char* buffer, long buffer_size)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (buffer == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else if (m_bConnected)
  {
    #ifdef LINUX
      if (::write(m_iSocketFD, buffer,buffer_size) >= 0)
      {
        ulRetval = RET_NOERROR;
      }
      else
      {
        ulRetval = RET_REQUEST_FAILED;
      }
    #else
      if (::send(m_iSocketFD, buffer,buffer_size, 0) >= 0)
      {
        ulRetval = RET_NOERROR;
      }
      else
      {
        ulRetval = RET_REQUEST_FAILED;
      }
    #endif
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}

/**************************************************************************/
/*! see IOutputStream
*/
/**************************************************************************/
unsigned long MQUALIFIER TSocket::writeLong(long data)
{
  return this->write((char*)&data, sizeof(long));
}

/**************************************************************************/
/*! see IOutputStream
*/
/**************************************************************************/
unsigned long MQUALIFIER TSocket::writeArray(const IByteArray* data)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (data == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    ulRetval = this->writeLong(data->getCount()); //Size of the array
    if (ulRetval == RET_NOERROR)
    {
      ulRetval = this->write(data->getArray(), data->getCount());
    }
  }
  return ulRetval;
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TSocket);
MEMORY_MANAGER_IMPL_END;


