/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : StringMap                                                     *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:46:47 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: StringMap.hpp,v $
 *   Revision 1.3  2005/11/25 09:46:47  bhadorn
 *   Implemented const_iterator
 *
 *   Revision 1.2  2005/11/16 11:31:22  bhadorn
 *   New iterator type to receive information about the key of an item.
 *
 *   Revision 1.1  2005/11/10 10:22:07  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef StringMapH
#define StringMapH

#include <zeusbase/System/String.h>
#include <zeusbase/System/Interfaces/IList.hpp>
#ifdef LINUX
  #include <string.h>
  #include <string>
  #include <map>
#else
  #include <mem.h>
  #include <string>
  #include <map>
#endif

BEGIN_NAMESPACE_Zeus
using namespace std;

/*****************************************************************************/
/*! Special string map iterator to receive the keys instead of the values
*/
/*****************************************************************************/
template <class T> class IStringMapIterator : public IListIterator<T>
{
  public:
    /*************************************************************************/
    /*! This method returns the key of an item instead of the value
        \param key : return parameter of the key
        \param item : Return parameter of the item. If the item is NULL
                      it will be not returned. Item is only valid if the 
                      return value of this method is RET_NOERROR
        \retval RET_NOERROR : key returned
        \retval RET_INVALID_PARAMETER : Invalid parameter received.
    */
    /*************************************************************************/
    virtual unsigned long MQUALIFIER getNextKey(IString* key, T* item = NULL)=0;
    
};

/*****************************************************************************/
/*! The String map wrapps a std::map class using a string as key
*/
/*****************************************************************************/
template <class T> class TStringMap
{
  typedef std::map<std::string, T>  TStrMap;
  public:
    /************************************************************************/
    /*! Creates a empty map
    */
    /************************************************************************/
    TStringMap()
    {
      m_pIterator = NULL;
      memset(&m_tEmpty, 0, sizeof(T));
    }
    
    /************************************************************************/
    /*! Creates a empty map, defining an empty item
        \param emptyitem : Empty item
    */
    /************************************************************************/
    TStringMap(T emptyitem)
    {
      m_pIterator = NULL;
      m_tEmpty = emptyitem;
    }
    
    /************************************************************************/
    /*! Destroys the map
    */
    /************************************************************************/
    virtual ~TStringMap()
    {
      clear();
    }
    
    /************************************************************************/
    /*! Gets an item out of the map
        \param key : Key of the item
        \return item
    */
    /************************************************************************/
    T& getItem(const TString& key)
    {      
      std::string strKey = TString(key).c_str();
      typename TStrMap::iterator it;
      it = m_Map.find(strKey);
      if (it != m_Map.end())
      {
        return (*it).second;
      }
      return m_tEmpty;
    }
    
    /************************************************************************/
    /*! Gets a copy of an item out of the map
        \param key : Key of the item
        \return item copy
    */
    /************************************************************************/
    T getItemCopy(const TString& key) const
    {
      T tRetval = m_tEmpty;
      
      std::string strKey = TString(key).c_str();
      typename TStrMap::const_iterator it;
      it = m_Map.find(strKey);
      if (it != m_Map.end())
      {
        tRetval = (*it).second;
      }
      return tRetval;
    }
    
    /************************************************************************/
    /*! adds a new item to the map
        
    */
    /************************************************************************/
    void setItem(const TString& key, T data)
    {
      std::string strKey = TString(key).c_str();
      m_Map[strKey] = data;
    }
    
    /************************************************************************/
    /*! Checks if the map is empty
        \retval true : Map is empty
        \retval false : Map is not empty
    */
    /************************************************************************/
    bool isEmpty() const
    {
      return m_Map.empty();
    }
    
    /************************************************************************/
    /*! Clears the map
    */
    /************************************************************************/
    long getCount() const
    {
      return m_Map.size();
    }

    /************************************************************************/
    /*! Deletes an item out of the map
    */
    /************************************************************************/
    bool deleteItem(const TString& key)
    {
      std::string strKey = TString(key).c_str();
      return (m_Map.erase(strKey) == 1);
    }
    
    /************************************************************************/
    /*! Checks if an item is inside the map
    */
    /************************************************************************/
    bool hasItem(const TString& key) const
    {
      bool bRetval = false;
      std::string strKey = TString(key).c_str();
      typename TStrMap::const_iterator it;
      it = m_Map.find(strKey);
      if (it != m_Map.end())
      {
        bRetval = true;
      }
      return bRetval;
    }
        
    /************************************************************************/
    /*! Clears the map
    */
    /************************************************************************/
    void clear()
    {
      m_Map.clear();
    }
    
    /************************************************************************/
    /*! See IList
    */
    /************************************************************************/
    virtual IStringMapIterator<T>* MQUALIFIER getIterator()
    {
      TStringMapIterator* iterator = new TStringMapIterator(m_Map, m_tEmpty);
      iterator->setNextIterator(m_pIterator);
      m_pIterator = iterator;
      return m_pIterator;
    }
    
    /************************************************************************/
    /*! See IList
    */
    /************************************************************************/
    virtual void MQUALIFIER releaseIterator(IListIterator<T>* iterator)
    {
      TStringMapIterator* it = m_pIterator;
      TStringMapIterator* it_prev = NULL;
      while (it != NULL)
      {
        if (it == iterator)
        {
          //Erster Iterator loeschen
          if (it_prev == NULL)
          {
            m_pIterator = it->getNextIterator();
            delete it;
          }
          else
          {
            it_prev->setNextIterator(it->getNextIterator());
            delete it;
          }
          break;
        }
        it_prev = it;
        it = it->getNextIterator();
      }
    }
    
  private:
    ///Empty item
    T m_tEmpty; 
    ///Binary search 
    TStrMap m_Map;
    
    /*********************************************************************/
    /*! Iterator of the map
    */
    /*********************************************************************/
    class TStringMapIterator :public IStringMapIterator<T>
    {
      public:
        /*****************************************************************/
        /*! Creates the iterator
        */
        /*****************************************************************/
        TStringMapIterator(TStrMap& map, T& emptyelement) : m_Map(map)
        {
          m_tEmpty = emptyelement;
          m_Iterator = m_Map.begin();
          pNextIterator = NULL;
        }
        /*****************************************************************/
        /*! Destroys the iterator
        */
        /*****************************************************************/
        virtual ~TStringMapIterator()
        {}
        
        /*****************************************************************/
        /*! see IListIterator
        */
        /*****************************************************************/
        virtual unsigned long MQUALIFIER getNextKey(IString* key, T* item /*= NULL*/)
        {
          unsigned long ulRetval = RET_INVALID_PARAMETER;
          if (key != NULL)
          {
            TString strKey;
            if (this->hasNextItem())
            {
              strKey = (*m_Iterator).first.c_str();
              if (item != NULL)
              {
                *item = (*m_Iterator).second;
              }
              m_Iterator++;
            }
            key->assignStr(&strKey);
            ulRetval = RET_NOERROR;
          }
          return ulRetval;
        
        }
        
        /*****************************************************************/
        /*! see IListIterator
        */
        /*****************************************************************/
        virtual void MQUALIFIER reset() const
        {
          m_Iterator = m_Map.begin();
        }

        /*****************************************************************/
        /*! see IListIterator
        */
        /*****************************************************************/
        virtual T& MQUALIFIER getNextItem() const
        {
          T& tRetval = m_tEmpty;
          if (this->hasNextItem())
          {
            tRetval = (*m_Iterator).second;
            m_Iterator++;
          }
          return tRetval;
        }

        /*****************************************************************/
        /*! see IListIterator
        */
        /*****************************************************************/
        virtual const T& MQUALIFIER getNextItemConst() const
        {
          return (const T&)this->getNextItem();
        }

        /*****************************************************************/
        /*! see IListIterator
        */
        /*****************************************************************/
        virtual bool MQUALIFIER hasNextItem() const
        {
          return (m_Map.end() != m_Iterator);
        }
        
        TStringMapIterator* getNextIterator()
        { return pNextIterator; }
        
        void setNextIterator(TStringMapIterator* it)
        { pNextIterator = it; }
        
      private:
        ///Binary search 
        TStrMap& m_Map;
        ///Empty item
        mutable T m_tEmpty; 
        ///Iterator of the map
        mutable typename TStrMap::iterator m_Iterator;
        ///Next iterator
        mutable TStringMapIterator* pNextIterator;
    };
    
    ///Iterator Zeiger
    TStringMapIterator* m_pIterator;
};

END_NAMESPACE_Zeus

#endif
