/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CellLife                                                      *
 * Package : CellModel                                                     *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:22:07 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: Thread.cpp,v $
 *   Revision 1.1  2005/11/10 10:22:07  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <zeusbase/System/Thread.h>
#include <zeusbase/System/LoggerManager.h>

#ifdef LINUX
  #include <signal.h>
  #include <unistd.h>
  #include <sys/types.h>
  #include <sys/wait.h>
  #include <signal.h>
  #include <pthread.h>
  #include <linux/unistd.h>
#else
  #include <process.h>
#endif

USING_NAMESPACE_Zeus

//##########################################################################
// LINUX SPECIFIC FUNCTIONS

#ifdef LINUX
/***************************************************************************/
/*! Handles the signal
*/
/***************************************************************************/
static void sigThreadHandler(int sig)
{
  ZLOGGER.printfln(LOGMODE_DEBUG, "Signal received: %d", sig);
}

/***************************************************************************/
/*! Initializes the signal handler of the current thread object
*/
/***************************************************************************/
void initThreadSigHandler()
{
  sigset_t Mask;
  struct sigaction SigAction;
  
  sigemptyset(&Mask);
  SigAction.sa_handler = sigThreadHandler;
  SigAction.sa_mask = Mask;
  SigAction.sa_flags = 0;
  
  if (sigaction(SIGPIPE, &SigAction, NULL) < 0)
  {
    ZLOGGER.printfln(LOGMODE_WARN, "Unable to init sig handler for SIGPIPE");
  }
/*  if (sigaction(SIGINT, &SigAction, NULL) < 0)
  {
    ZLOGGER.printfln(LOGMODE_WARN, "Unable to init sig handler for SIGINT");
  }*/
}

/***************************************************************************/
/*! Diese Methode wird durch den Thread bearbeitet
    \param life : Das Life Objekt wird als Parameter uebergeben
    \return NULL;
*/
/***************************************************************************/
void* createThread(void* life)
{

  if (life != NULL)
  {
    ZLOGGER.printfln(LOGMODE_DEBUG, "Thread %d executed.",(int)life);
    initThreadSigHandler();
    ((TThread*)life)->execute();
    ((TThread*)life)->doKill();
    ZLOGGER.printfln(LOGMODE_DEBUG, "Thread %d terminated.",(int)life);
  }
  else
  {
    ZLOGGER.printfln(LOGMODE_FATAL, "Invalid Thread object.");
  }
  return NULL;
}
//##########################################################################


//##########################################################################
// WINDOWS SPECIFIC FUNCTIONS
#else


/***************************************************************************/
/*! Diese Methode wird durch den Thread bearbeitet
    \param life : Das Life Objekt wird als Parameter uebergeben
    \return NULL;
*/
/***************************************************************************/
unsigned long __stdcall createThread(void* life)
{
  if (life != NULL)
  {
    ZLOGGER.printfln(LOGMODE_DEBUG, "Thread %d executed.",(int)life);
    ((TThread*)life)->execute();
    ((TThread*)life)->doKill();
    ZLOGGER.printfln(LOGMODE_DEBUG, "Thread %d terminated.",(int)life);
  }
  else
  {
    ZLOGGER.printfln(LOGMODE_FATAL, "Invalid Thread object.");
  }
  return 0;
}
//##########################################################################


#if _WIN32_WINNT >= 0x0400
  static void CALLBACK UserAPCProc(DWORD /*dwParam*/)
  {
  }
#endif

#endif


/***************************************************************************/
/*! Erstellen des Threads
*/
/***************************************************************************/
TThread::TThread() : TZObject()
{
  m_bCreated = false;
  m_bInterrupted = false;
  m_pRunObj = NULL;

  #ifndef LINUX
    m_hHandle = NULL;
  #endif

}

/***************************************************************************/
/*! Erstellen des Threads
*/
/***************************************************************************/
TThread::TThread(IRunnable* obj) : TZObject()
{
  m_bCreated = false;
  m_bInterrupted = false;
  m_pRunObj = obj;
  if (m_pRunObj != NULL)
  {
    m_pRunObj->addRef();
  }
}


/***************************************************************************/
/*! Zerstoeren des Threads
*/
/***************************************************************************/
TThread::~TThread()
{
  if (m_bCreated)
  {
    this->kill();
  }
  
  if (m_pRunObj != NULL)
  {
    m_pRunObj->release();
    m_pRunObj = NULL;
  }  
}

/***************************************************************************/
/*! Pruefen ob der Thread noch existiert
    \retval true : Der Thread existiert nicht mehr
    \retval false : Der Thread existiert noch
*/
/***************************************************************************/
bool TThread::isDead()
{
  return !m_bCreated;
}

/***************************************************************************/
/*! Checks if the thread is interrupted. This flag must be asked inside the
    execute method, to abort the process.
    \retval true : Interrupt flag is set
    \retval false : Interrupt flag is not set
*/
/***************************************************************************/
bool TThread::isInterrupted()
{
  return m_bInterrupted;
}


/***************************************************************************/
/*! Der Thread wird gestartet
*/
/***************************************************************************/
void TThread::start()
{
  m_bInterrupted = false;
  m_bCreated = true;
  m_Event.reset();

  #ifdef LINUX
    int rc = ::pthread_create(&Thread, NULL,createThread, this);
    if (rc == 0)
    {
      m_bCreated = true;
    }
    else
    {
      m_bCreated = false;
    }
  #else
    DWORD dwThreadId = 0;
    m_hHandle = ::CreateThread( NULL, 0, createThread, this, 0, &dwThreadId);

    if (m_hHandle != NULL)
    {
      m_bCreated = true;
    }
    else
    {
      m_bCreated = false;
    }
  #endif
}

/***************************************************************************/
/*! Der Thread zerstoert.
*/
/***************************************************************************/
void TThread::kill()
{
  m_bInterrupted = true;

  //Warten auf Thread zum Beenden.
  #ifdef LINUX
    if (m_bCreated)
    {
      //pthread_kill(Thread, SIGALRM);
      pthread_join(Thread, NULL);
    }
  #else
    if (!m_Event.wait(30.0))
    {
      ::TerminateThread(m_hHandle, 0);
    }
    ::CloseHandle(m_hHandle);

  #endif
  doKill();
}


/***************************************************************************/
/*! Internal clearance. All flag are reset.
*/
/***************************************************************************/
void TThread::doKill()
{
  m_bCreated = false;
  m_Event.set();
}

/***************************************************************************/
/*! In dieser Methode lebt der Thread
*/
/***************************************************************************/
void TThread::execute()
{
  if (m_pRunObj != NULL && m_bCreated == true)
  {
    m_pRunObj->execute();
  }
  else
  {
    ZLOGGER.printfln(LOGMODE_WARN, "Method not implemented");
  }
}
