 
/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : XMLObject                                                     *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/12/14 09:03:25 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: XObject.cpp,v $
 *   Revision 1.2  2005/12/14 09:03:25  bhadorn
 *   new methods for call backs added.
 *
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/System/CriticalSection.h>
#include <zeusbase/System/XObject.h>
#include <zeusbase/System/XObjectPath.h>
#include <zeusbase/System/Interfaces/IXMLNode.hpp> 
#include <zeusbase/System/LoggerManager.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates the xml object
    \param node : XML Node representing this object
    \param attributeschema : An attribute schema (default is ATTSCHEMA_NORMAL).
*/
/***************************************************************************/
TXObject::TXObject(IXMLNode* node, unsigned long attributeschema /*= ATTSCHEMA_NORMAL*/) : TZObject()
{
  m_pParent = NULL;
  m_pTreeLock = NULL;
  m_pNode = node;
  m_bFrozen = true;
  m_ulAttributeSchema = attributeschema; 
  if (m_pNode != NULL)
  {
    m_pNode->addRef();
    TString strName;
    m_pNode->getNodeName(&strName);
    TZObject::setObjName(strName);
    
    //Set Schema value of this node
    TString strAttValue;
    TString strAttSchema = L"AttributeSchema";
    if (m_pNode->getAttributeValue(&strAttSchema, &strAttValue) == RET_NOERROR)
    {
      this->m_ulAttributeSchema = strAttValue.toLong(ATTSCHEMA_NORMAL);
    }
  }
  m_pLock = new TCriticalSection();
}

/***************************************************************************/
/*! Creates an empty XML object. This constructor is protected
*/
/***************************************************************************/
TXObject::TXObject() : TZObject()
{
  m_ulAttributeSchema = ATTSCHEMA_NORMAL; 
  m_bFrozen = true;
  m_pParent = NULL;
  m_pNode = NULL;
  m_pTreeLock = NULL;
  m_pLock = new TCriticalSection();
}

/***************************************************************************/
/*! Destroyes the Object
*/
/***************************************************************************/
TXObject::~TXObject()
{
  while(!m_lstChildren.isEmpty())
  {
    IXObject* pObj = m_lstChildren[0];
    pObj->setParent(NULL);
    m_lstChildren.deleteItem(0);
    this->onChildDeleted(pObj);
    pObj->release();
  }
  
  m_pParent = NULL;
  if (m_pNode != NULL)
  {
    m_pNode->release();
    m_pNode = NULL;
  }
  if(m_pTreeLock != NULL)
  {
    m_pTreeLock->release();
    m_pTreeLock = NULL;
  }
  m_pLock->release();
}

/***************************************************************************/
/*! Returns the codemodule of this object
    \return name of code module
*/
/***************************************************************************/
TString TXObject::getCodeModule() const
{
  TString strRetval;
  if (m_pNode != NULL)
  {
    TString strAttCodeModule=L"CodeModule";
    m_pNode->getAttributeValue(&strAttCodeModule, &strRetval);
  }
  return strRetval;
}


/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::getAttributeSchema() const
{
  return m_ulAttributeSchema;
}


/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::getName(IString* name)const
{
  unsigned long ulRetval = RET_INVALID_PARAMETER;
  if (name != NULL)
  {
    TString strRet = TZObject::getObjName();
    name->assignStr(&strRet);
    ulRetval = RET_NOERROR;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::getID() const
{
  unsigned long ulRetval = 0;
  if (m_pNode != NULL)
  {
    TString strAttName = L"ID";
    TString strAttValue;
    m_pNode->getAttributeValue(&strAttName, &strAttValue);
    ulRetval = strAttValue.toLong(0);
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::addChild(IXObject* child)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (child != NULL)
  {
    //Removes the object from the old parent
    ulRetval = RET_NOERROR;
    IXObject* pParent = NULL;
    if (child->getParent(&pParent) == RET_NOERROR)
    {
      ulRetval = pParent->removeChild(child);
      pParent->release();
    }
    
    //Removing was successful -> add child to this children list
    if (ulRetval == RET_NOERROR)
    {
      IXMLNode* pChildXMLNode = NULL;
      if (child->getXMLNode(&pChildXMLNode) == RET_NOERROR)
      {
        if (m_pNode->appendNode(pChildXMLNode) == RET_NOERROR)
        {
          child->setParent(this);
          child->addRef();
          child->setTreeLock(m_pTreeLock);
          m_lstChildren.add(child);
          this->onChildAdded(child);
          ulRetval = RET_NOERROR;
        }
        else
        {
          ulRetval = RET_REQUEST_FAILED;
        }
        pChildXMLNode->release();
      }
      else
      {
        ulRetval = RET_REQUEST_FAILED;
      }
    }
  }
  else
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  return ulRetval;
}

/***************************************************************************/
/*! DO NOT USE THIS METHOD. ITS USED FROM THE FACTORY.
    See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::setChild(IXObject* child)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (child != NULL)
  {
    child->setParent(this);
    child->addRef();
    child->setTreeLock(m_pTreeLock);
    m_lstChildren.add(child);
    this->onChildAdded(child);
    ulRetval = RET_NOERROR;
  }
  else
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  return ulRetval;  
}


/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::removeChild(IXObject* child)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (child != NULL)
  {
    long lIndex = m_lstChildren.indexOf(child);
    IXObject* pFoundObj = m_lstChildren[lIndex];
    if (pFoundObj != NULL)
    {
      IXMLNode* pNode = NULL;
      if (child->getXMLNode(&pNode) == RET_NOERROR)
      {
        //Remove child out of the list
        pFoundObj->release();
        m_lstChildren.deleteItem(lIndex);

        //Remove XML Node and unregister the parent
        pNode->removeMe();
        child->setParent(NULL);
        child->setTreeLock(NULL);
        this->onChildDeleted(child);
        ulRetval = RET_NOERROR;
        pNode->release();
      }
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  else
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::deleteChild(long index)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  IXObject* pFoundObj = m_lstChildren[index];
  if (pFoundObj != NULL)
  {
    IXMLNode* pNode = NULL;
    if (pFoundObj->getXMLNode(&pNode) == RET_NOERROR)
    {
      //Remove XML Node and unregister the parent
      pNode->removeMe();
      pFoundObj->setParent(NULL);
      pFoundObj->setTreeLock(NULL);
      pNode->release();
      
      //Remove child out of the list
      m_lstChildren.deleteItem(index);
      this->onChildDeleted(pFoundObj);
      pFoundObj->release();
      ulRetval = RET_NOERROR;
    }
  }
  else
  {
    ulRetval = RET_REQUEST_FAILED;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
long MQUALIFIER TXObject::getChildCount() const
{
  return m_lstChildren.getCount();
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::getChild(long index, IXObject** child)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (child != NULL)
  {
    *child = m_lstChildren[index];
    if (*child != NULL)
    {
      (*child)->addRef();
      ulRetval = RET_NOERROR;
    }
  }
  else
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::getChildByName(const IString* name, IXObject** child)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (child != NULL && name != NULL)
  {
    TString strNameIn(name);
    long lElementCount = 0;
    TString strOpenBracket = L"[";
    long lOpenBracketPos = name->getPos(&strOpenBracket);
    if (lOpenBracketPos != -1)
    {
      TString strCloseBracket = L"]";
      long lCloseBracketPos = name->getPos(&strCloseBracket);
      lElementCount = strNameIn.getSubString(lOpenBracketPos+1, lCloseBracketPos-1).toLong();
      //ZLOGGER.printfln(LOGMODE_DEBUG, "Brackets found found in %s: Index=%d", 
      //  strNameIn.c_str(), strNameIn.getSubString(lOpenBracketPos+1, lCloseBracketPos-1).toLong());
      strNameIn = strNameIn.getSubString(0,lOpenBracketPos-1);
    }
  
    bool bFound = false;
    IListIterator<IXObject*>* pIt = m_lstChildren.getIterator();
    while(pIt->hasNextItem() && !bFound)
    {
      TString strName;
      IXObject* pObject = pIt->getNextItem();
      pObject->getName(&strName);
      if (strNameIn.equalsStr(&strName))
      {
        if (lElementCount <= 0)
        {
          *child = pObject;
          pObject->addRef();
          bFound = true;
          ulRetval = RET_NOERROR;
        }
        else
        {
          lElementCount--;
        }
      }
    }
    m_lstChildren.releaseIterator(pIt);
  }
  else
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::getParent(IXObject** parent)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (parent != NULL)
  {
    *parent = m_pParent;
    if (m_pParent != NULL)
    {
      m_pParent->addRef();
      ulRetval = RET_NOERROR;
    }
    else
    {
      ulRetval = RET_REQUEST_FAILED;
    }
  }
  else
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::setParent(IXObject* parent)
{
  unsigned long ulRetval = RET_NOERROR;
  m_pParent = parent; //To not allocate a reference, because of circularity
  if (m_pParent != NULL)
  {    
    //Try to get Schema value of this node and check if it has to be
    //set from the parent (inheritance of schemas)
    TString strAttValue;
    TString strAttSchema = L"AttributeSchema";
    this->m_ulAttributeSchema = m_pParent->getAttributeSchema();
    if (m_pNode != NULL && m_pNode->getAttributeValue(&strAttSchema, &strAttValue) == RET_NOERROR)
    {
      this->m_ulAttributeSchema = strAttValue.toLong(ATTSCHEMA_NORMAL);
    }
  }

  return ulRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::getXMLNode(IXMLNode** node) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (node != NULL)
  {
    *node = m_pNode;
    if (m_pNode != NULL)
    {
      m_pNode->addRef();
      ulRetval = RET_NOERROR;
    }
  }
  else
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  return ulRetval;
}
    
/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::getRootObject(IXObject** obj)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (obj != NULL)
  {
    if (m_pParent != NULL)
    {
      ulRetval = m_pParent->getRootObject(obj);
    }
    else
    {
      *obj = this;
      this->addRef();
      ulRetval = RET_NOERROR;
    }
  }
  else
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::getObject(IString* path, IXObject** obj)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  
  if (path != NULL && obj != NULL)
  {
    TXObjectPath ObjPath(this);
    ulRetval = ObjPath.getObject(path, obj);
  }
  else
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TXObject::canCreateChildren() const
{
  return true;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TXObject::freeze()
{
  bool bRetval = false;
  if (!m_bFrozen)
  {
    m_bFrozen = true;
    IListIterator<IXObject*>* pIt = m_lstChildren.getIterator();
    while(pIt->hasNextItem())
    {
      IXObject* pObject = pIt->getNextItem();
      pObject->freeze();
    }
    m_lstChildren.releaseIterator(pIt);
    bRetval = true;
  }
  return bRetval;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TXObject::unfreeze()
{
  bool bRetval = false;
  if (m_bFrozen)
  {
    m_bFrozen = false;
    unfreezeChildren();
    bRetval = true;
  }
  return bRetval;
}

/***************************************************************************/
/*! Unfreezes all children objects
*/
/***************************************************************************/
void TXObject::unfreezeChildren()
{
  IListIterator<IXObject*>* pIt = m_lstChildren.getIterator();
  while(pIt->hasNextItem())
  {
    IXObject* pObject = pIt->getNextItem();
    pObject->unfreeze();
  }
  m_lstChildren.releaseIterator(pIt);
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
bool MQUALIFIER TXObject::isFrozen() const
{
  return m_bFrozen;
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
void MQUALIFIER TXObject::setTreeLock(ICriticalSection* section)
{
  if (m_pTreeLock != NULL)
  {
    m_pTreeLock->release();
  }
  m_pTreeLock = section;
  if (m_pTreeLock != NULL)
  {
    m_pTreeLock->addRef();
  }
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
void MQUALIFIER TXObject::lockTree()
{
  if (m_pTreeLock != NULL)
  {
    m_pTreeLock->enter();
  }
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
void MQUALIFIER TXObject::unlockTree()
{
  if (m_pTreeLock != NULL)
  {
    m_pTreeLock->leave();
  }
}

/***************************************************************************/
/*! See IXObject
*/
/***************************************************************************/
unsigned long MQUALIFIER TXObject::onExecute(const ISimpleCommand* /*command*/, unsigned long /*ulMode*/)
{
  return RET_REQUEST_FAILED;
}


/***************************************************************************/
/*! Get infomation abaout the xmlobject
*/
/***************************************************************************/
TString TXObject::getXMLStream() const
{
  TString strRetval;
  if (m_pNode != NULL)
  {
    m_pNode->serializeNode(&strRetval);
  }
  return strRetval;
}


/***************************************************************************/
/*! See TZObject
*/
/***************************************************************************/
void TXObject::setObjName(const TString& /*name*/)
{
  //Can not change the node name
}


/***************************************************************************/
/*! This method reades an extended attribute from the node
    \pre the instance m_pNode must be valid
*/
/***************************************************************************/
unsigned long TXObject::readAttAsNode(TString name, TString& value) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  TString strAttribute =L"Attribute";
  TString strAttName = L"Name";
  TString strAttValue = L"Value";
  TString strChkName;
  
  IXMLNode* pAttNode = NULL;
  bool bFound = false;
  m_pNode->resetNodeIterator(&strAttribute);
  while (!bFound && m_pNode->getNextNode(&pAttNode) == RET_NOERROR)
  {
    if (pAttNode->getAttributeValue(&strAttName, &strChkName) == RET_NOERROR && strChkName == name)
    {
      bFound = true;
      if (m_ulAttributeSchema == ATTSCHEMA_EXTENDED)
      {
        ulRetval = pAttNode->getAttributeValue(&strAttValue, &value);
      }
      else
      {
        ulRetval = pAttNode->getNodeValue(&value);
      }
    }
    pAttNode->release();
  }
  
  return ulRetval;
}

/***************************************************************************/
/*! read an attribute value
    \param name : name of the attribute
    \param value : [OUT] value of the attribute 
    \retval RET_NOERROR : Value readed
    \retval RET_REQUEST_FAILED : No persistent object.
*/
/***************************************************************************/
unsigned long TXObject::readAttribute(TString name, TString& value) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pNode != NULL)
  {
    if (m_ulAttributeSchema == ATTSCHEMA_EXTENDED || m_ulAttributeSchema == ATTSCHEMA_COMPLEX)
    {
      ulRetval = readAttAsNode(name, value);
    }
    else
    {
      ulRetval = m_pNode->getAttributeValue(&name,&value);
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! read an attribute value
    \param name : name of the attribute
    \param value : [OUT] value of the attribute 
    \retval RET_NOERROR : Value readed
    \retval RET_REQUEST_FAILED : No persistent object.
*/
/***************************************************************************/
unsigned long TXObject::readAttribute(TString name, long& value) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pNode != NULL)
  {
    TString strValue;
    if (m_ulAttributeSchema == ATTSCHEMA_EXTENDED || m_ulAttributeSchema == ATTSCHEMA_COMPLEX)
    {
      ulRetval = readAttAsNode(name, strValue);
    }
    else
    {
      ulRetval = m_pNode->getAttributeValue(&name,&strValue);
    }
    value = strValue.toLong();
  }
  return ulRetval;
}

/***************************************************************************/
/*! read an attribute value
    \param name : name of the attribute
    \param value : [OUT] value of the attribute 
    \retval RET_NOERROR : Value readed
    \retval RET_REQUEST_FAILED : No persistent object.
*/
/***************************************************************************/
unsigned long TXObject::readAttribute(TString name, unsigned long& value) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pNode != NULL)
  {
    TString strValue;
    if (m_ulAttributeSchema == ATTSCHEMA_EXTENDED || m_ulAttributeSchema == ATTSCHEMA_COMPLEX)
    {
      ulRetval = readAttAsNode(name, strValue);
    }
    else
    {
      ulRetval = m_pNode->getAttributeValue(&name,&strValue);
    }
    value = strValue.toLong();
  }
  return ulRetval;
}


/***************************************************************************/
/*! read an attribute value
    \param name : name of the attribute
    \param value : [OUT] value of the attribute 
    \retval RET_NOERROR : Value readed
    \retval RET_REQUEST_FAILED : No persistent object.
*/
/***************************************************************************/
unsigned long TXObject::readAttribute(TString name, double& value) const
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pNode != NULL)
  {
    TString strValue;
    if (m_ulAttributeSchema == ATTSCHEMA_EXTENDED || m_ulAttributeSchema == ATTSCHEMA_COMPLEX)
    {
      ulRetval = readAttAsNode(name, strValue);
    }
    else
    {
      ulRetval = m_pNode->getAttributeValue(&name,&strValue);
    }
    value = strValue.toDouble();
  }
  return ulRetval;
}


/***************************************************************************/
/*! This method writes an extended attribute back to the attributenode
    \pre the instance m_pNode must be valid
*/
/***************************************************************************/
unsigned long TXObject::writeAttAsNode(TString name, TString value)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  TString strAttribute =L"Attribute";
  TString strAttName = L"Name";
  TString strAttValue = L"Value";
  TString strChkName;
  
  IXMLNode* pAttNode = NULL;
  bool bFound = false;
  m_pNode->resetNodeIterator(&strAttribute);
  while (!bFound && m_pNode->getNextNode(&pAttNode) == RET_NOERROR)
  {
    if (pAttNode->getAttributeValue(&strAttName, &strChkName) == RET_NOERROR && strChkName == name)
    {
      bFound = true;
      if (m_ulAttributeSchema == ATTSCHEMA_EXTENDED)
      {
        ulRetval = pAttNode->setAttributeValue(&strAttValue, &value);
      }
      else
      {
        ulRetval = pAttNode->setNodeValue(&value);
      }
    }
    pAttNode->release();
  }
  
  //Node not available -> create a new node
  if(!bFound)
  {
    TString strAttName = L"Attribute";
    TString strURI;
    IXMLNode* pChildNode= NULL;
    if (m_pNode->createNewChildNode(TYPE_NODE_ELEMENT, &strAttName, &strURI, &pChildNode) == RET_NOERROR)
    {
      pChildNode->setAttributeValue(&strAttName, &name);
      if (m_ulAttributeSchema == ATTSCHEMA_EXTENDED)
      {
        ulRetval = pChildNode->setAttributeValue(&strAttValue, &value);
      }
      else
      {
        ulRetval = pChildNode->setNodeValue(&value);
      }
      pChildNode->release();
    }
  }
  
  return ulRetval;
}


/***************************************************************************/
/*! sets an attribute value
    \param name : name of the attribute
    \param value : [OUT] value of the attribute 
    \retval RET_NOERROR : Value readed
    \retval RET_REQUEST_FAILED : No persistent object.
*/
/***************************************************************************/
unsigned long TXObject::storeAttribute(TString name, TString value)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pNode != NULL)
  {
    if (m_ulAttributeSchema == ATTSCHEMA_EXTENDED || m_ulAttributeSchema == ATTSCHEMA_COMPLEX)
    {
      ulRetval = writeAttAsNode(name, value);
    }
    else
    {
      ulRetval = m_pNode->setAttributeValue(&name,&value);
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! sets an attribute value
    \param name : name of the attribute
    \param value : [OUT] value of the attribute 
    \retval RET_NOERROR : Value readed
    \retval RET_REQUEST_FAILED : No persistent object.
*/
/***************************************************************************/
unsigned long TXObject::storeAttribute(TString name, long value)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pNode != NULL)
  {
    TString strValue = value;
    if (m_ulAttributeSchema == ATTSCHEMA_EXTENDED || m_ulAttributeSchema == ATTSCHEMA_COMPLEX)
    {
      ulRetval = writeAttAsNode(name, strValue);
    }
    else
    {
      ulRetval = m_pNode->setAttributeValue(&name,&strValue);
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! sets an attribute value
    \param name : name of the attribute
    \param value : [OUT] value of the attribute 
    \retval RET_NOERROR : Value readed
    \retval RET_REQUEST_FAILED : No persistent object.
*/
/***************************************************************************/
unsigned long TXObject::storeAttribute(TString name, unsigned long value)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pNode != NULL)
  {
    TString strValue = (long)value;
    if (m_ulAttributeSchema == ATTSCHEMA_EXTENDED || m_ulAttributeSchema == ATTSCHEMA_COMPLEX)
    {
      ulRetval = writeAttAsNode(name, strValue);
    }
    else
    {
      ulRetval = m_pNode->setAttributeValue(&name,&strValue);
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! sets an attribute value
    \param name : name of the attribute
    \param value : [OUT] value of the attribute 
    \retval RET_NOERROR : Value readed
    \retval RET_REQUEST_FAILED : No persistent object.
*/
/***************************************************************************/
unsigned long TXObject::storeAttribute(TString name, double value)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pNode != NULL)
  {
    TString strValue = value; 
    if (m_ulAttributeSchema == ATTSCHEMA_EXTENDED || m_ulAttributeSchema == ATTSCHEMA_COMPLEX)
    {
      ulRetval = writeAttAsNode(name, strValue);
    }
    else
    {
      ulRetval = m_pNode->setAttributeValue(&name,&strValue);
    }
  }
  return ulRetval;
}
 
/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL_LOCK(TXObject, m_pLock);
  INTERFACE_CAST(IXObject, INTERFACE_IXObject);
MEMORY_MANAGER_IMPL_END;

