/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : ZObjectFactory                                                *
 * Package : System                                                        *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:48:02 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: ZObjectFactory.cpp,v $
 *   Revision 1.3  2005/11/25 09:48:02  bhadorn
 *   Print detailed error message instead of Invalid Streamheader message only
 *
 *   Revision 1.2  2005/11/16 11:30:12  bhadorn
 *   All input parameter are [const]
 *
 *   Revision 1.1  2005/11/10 10:21:24  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 

#include <zeusbase/System/ZObjectFactory.h>
#include <zeusbase/System/LoggerManager.h>
#include <zeusbase/System/ByteArrayInputStream.h>
#include <zeusbase/System/ByteArrayOutputStream.h>
#include <zeusbase/System/LibraryManager.h>
#include <stdio.h>

USING_NAMESPACE_Zeus 

///Static object initialisation
TZObjectFactory TZObjectFactory::m_Factory;

/***************************************************************************/
/*! Returns the singleton object of the object factory
*/
/***************************************************************************/
TZObjectFactory& TZObjectFactory::getInstance()
{
  return m_Factory;
}


/***************************************************************************/
/*! Creates a object factory
*/
/***************************************************************************/
TZObjectFactory::TZObjectFactory()
{
  m_pDelegation = NULL;
}

/***************************************************************************/
/*! Destroys the object factory
*/
/***************************************************************************/
TZObjectFactory::~TZObjectFactory()
{
  releaseDelegationInterface();
}

 
/***************************************************************************/
/*! Sets a delegation interface. All calls are delegated to this interface
    object.
    \param factory : Delegation
*/
/***************************************************************************/
void TZObjectFactory::setDelegationInterface(IZObjectFactory* factory)
{
  releaseDelegationInterface();
  m_pDelegation = factory;
  if (m_pDelegation != NULL)
  {
    m_pDelegation->addRef();
  }
}

/***************************************************************************/
/*! Releases the delegation interface
*/
/***************************************************************************/
void TZObjectFactory::releaseDelegationInterface()
{
  if (m_pDelegation != NULL)
  {
    m_pDelegation->release();
    m_pDelegation = NULL;
  }
}

 
/***************************************************************************/
/*! See IZObjectFactory
*/
/***************************************************************************/
unsigned long MQUALIFIER TZObjectFactory::createObjectFromStream(const IByteArray* stream, ISerializable** obj)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pDelegation != NULL)
  {
    ulRetval = m_pDelegation->createObjectFromStream(stream, obj);
  }
  else if (stream == NULL || obj == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    ZLOGGER.indent();
    TByteArrayInputStream* pStreamer = new TByteArrayInputStream(stream);
    
    //Reading the serializable header information
    unsigned long ulHeaderID = pStreamer->readLong();
    if (ulHeaderID == SERIALOBJECT_HEADER)
    {
      unsigned long ulObjectID = pStreamer->readLong();
      TString strClassName;
      pStreamer->readString(&strClassName);
      if (strClassName != L"")
      {
        IZObjectFactory::FCreateZObject Function = m_mapClasses.getItem(strClassName);
        if (Function != NULL)
        {
          IZUnknown* pObject = NULL;
          ZLOGGER.printfln(LOGMODE_DEBUG,"Create object from class %s ...", strClassName.c_str());
          ulRetval = Function(ulObjectID, pStreamer, &pObject);
          if (ulRetval == RET_NOERROR)
          {
            ulRetval = pObject->askForInterface(INTERFACE_ISerializable, (IZUnknown**)obj);
            pObject->release();
            ZLOGGER.printfln(LOGMODE_DEBUG, "...done. [%s]", strClassName.c_str());
          }
          else
          {
            ZLOGGER.printfln(LOGMODE_DEBUG, "...failed. [%s]", strClassName.c_str());
          }
        }
        else
        {
          ZLOGGER.printfln(LOGMODE_ERROR, "Class %s is not registered.", strClassName.c_str());
        }
      }
      else
      {
        ZLOGGER.printfln(LOGMODE_ERROR, "Stream contains an empty class name");
        ulRetval = RET_INVALID_DATA;
      }
    }
    else if (stream->getCount() > 0)
    {
      ZLOGGER.printfln(LOGMODE_ERROR, "Invalid stream header.");
      ulRetval = RET_INVALID_DATA;
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_ERROR, "No data received to create object.");
      ulRetval = RET_REQUEST_FAILED;
    }
    pStreamer->release();
    ZLOGGER.unindent();
  }
  
  return ulRetval;
}

/***************************************************************************/
/*! See IZObjectFactory
*/
/***************************************************************************/
unsigned long MQUALIFIER TZObjectFactory::createObjectByClassName(const IString* classname, const IString* codemodule, IZUnknown** obj)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pDelegation != NULL)
  {
    ulRetval = m_pDelegation->createObjectByClassName(classname, codemodule, obj);
  }
  else if (classname == NULL || obj == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strClassName(classname);
    TString strCodeModule(codemodule);
    if (strClassName != L"")
    {
      //--------------------------------------------------------------------------
      //Check registry
      if (strCodeModule == L"")
      {
        IZObjectFactory::FCreateZObject Function = m_mapClasses.getItem(strClassName);
        if (Function != NULL)
        {
          ulRetval = Function(0, NULL, obj);
          ZLOGGER.printfln(LOGMODE_INFO, "Object %s created: F=%d::%d.", strClassName.c_str(), (int)Function, (int)ulRetval);
        }
        else
        {
          ZLOGGER.printfln(LOGMODE_ERROR, "Class %s is not registered.", strClassName.c_str());
        }
      }
      //--------------------------------------------------------------------------
      //Check code modules
      else
      {
        ulRetval = TLibraryManager::getInstance().getInstance().createObject(classname, codemodule, obj);
      }
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_ERROR, "Empty class name. Unable to create an object.");
      ulRetval = RET_INVALID_DATA;
    }
  }
  
  return ulRetval;
}


/***************************************************************************/
/*! See IZObjectFactory
*/
/***************************************************************************/
unsigned long MQUALIFIER TZObjectFactory::registerClass(const IString* name, IZObjectFactory::FCreateZObject function)
{
  unsigned long ulRetval = 0;
  if (m_pDelegation != NULL)
  {
    ulRetval = m_pDelegation->registerClass(name, function);
  }
  else if (name != NULL && function != NULL)
  {
    TString strClassName(name);
    if (!m_mapClasses.hasItem(strClassName))
    {
      m_mapClasses.setItem(strClassName, function);
      ulRetval = (unsigned long)function;
      ZLOGGER.printfln(LOGMODE_DEBUG, "Class registered: Name=%s; Handle=%d", strClassName.c_str(), (int)function);
    }
    else
    {
      ZLOGGER.printfln(LOGMODE_WARN, "Class already registered: %s:%d", strClassName.c_str(), (int)function);
    }
  }
  return ulRetval;
}

/***************************************************************************/
/*! See IZObjectFactory
*/
/***************************************************************************/
unsigned long MQUALIFIER TZObjectFactory::unregisterClass(const IString* name, unsigned long handle)
{
  unsigned long ulRetval = RET_REQUEST_FAILED;
  if (m_pDelegation != NULL)
  {
    ulRetval = m_pDelegation->unregisterClass(name, handle);
  }
  else if (name == NULL)
  {
    ulRetval = RET_INVALID_PARAMETER;
  }
  else
  {
    TString strClassName(name);
    IZObjectFactory::FCreateZObject Function = m_mapClasses.getItem(strClassName);
    if (m_mapClasses.hasItem(strClassName) && handle == (unsigned long)Function)
    {
      m_mapClasses.deleteItem(strClassName);
      ulRetval = RET_NOERROR;
    }
  }
  return ulRetval;
}

/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TZObjectFactory);
  INTERFACE_CAST(IZObjectFactory, INTERFACE_IZObjectFactory);
MEMORY_MANAGER_IMPL_END;

