/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : AbstractCell                                                  *
 * Package : Test Module for Cell Computing                                *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 09:59:57 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: AbstractCell.cpp,v $
 *   Revision 1.2  2005/11/25 09:59:57  bhadorn
 *   Eliminated compiler warnings
 *
 *   Revision 1.1  2005/10/28 12:22:23  bhadorn
 *   *** empty log message ***
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusdev/AbstractCell.h>
#include <zeusbase/System/String.h>
#include <zeusbase/Messaging/XMLMessage.h>
#include <zeusbase/Messaging/SystemMessage.h>

USING_NAMESPACE_Zeus

/***************************************************************************/
/*! Creates a new Abstract Cell
*/
/***************************************************************************/
TAbstractCell::TAbstractCell(IXMLNode* node) : TXObject(node)
{
  m_pLifeObj = NULL;
  m_lSleepTime = 10; //[ms]
  m_pCommunication = NULL;
}

/***************************************************************************/
/*! Destroyes the Abstract Cell
*/
/***************************************************************************/
TAbstractCell::~TAbstractCell()
{
  if (m_pCommunication != NULL)
  {
    m_pCommunication->release();
    m_pCommunication = NULL;
  }
}

/***************************************************************************/
/*! Checks if the life object has been killed or the flag to terminate is
    set.
    \retval true : Thread should terminate
    \retval false: Running
*/
/***************************************************************************/
bool TAbstractCell::isTerminated()
{
  return m_pLifeObj->isTerminated();
}

/***************************************************************************/
/*! see ICell
*/
/***************************************************************************/
void TAbstractCell::sendLifeSignal()
{
  if (m_pCommunication != NULL)
  {
    IMessage* pMessage = NULL;
    unsigned long long ulTimer = 0;
    if (m_pCommunication->getInputMessage((ICellKernel*)this, &pMessage, &ulTimer) == RET_NOERROR)
    {
      dispatchMessage(pMessage, ulTimer);
      pMessage->release();
    }
  }
}

/***************************************************************************/
/*! see ICell
*/
/***************************************************************************/
void TAbstractCell::dispatchMessage(IMessage* msg, unsigned long long timer)
{
  if (msg != NULL)
  {    
    switch(msg->getInterfaceID())
    {
      case INTERFACE_ISystemMessage :
        performSystemMessage((ISystemMessage*)msg, timer);  
      break;

      case INTERFACE_IBinaryMessage :
        performBinaryMessage((IBinaryMessage*)msg, timer);  
      break;
      
      case INTERFACE_IObjectMessage :
        performObjectMessage((IObjectMessage*)msg, timer);  
      break;
      
      case INTERFACE_IXMLMessage ://fall through
      default:
        performXMLMessage((IXMLMessage*)msg, timer);  
      break;
    }
  }
}


/***************************************************************************/
/*! Performs a system message object, such as ping etc.
    \param msg : Object to perform
    \param timer : Value of the message time
    \pre m_pCommunication is valid
*/
/***************************************************************************/
void TAbstractCell::performSystemMessage(ISystemMessage* msg, unsigned long long timer)
{
  if (msg != NULL)
  {
    TString strSenderCellName;
    TString strSenderPipeName;
    msg->getSenderCellName(&strSenderCellName);
    msg->getSenderPipeName(&strSenderPipeName);
    
    switch(msg->getMessageType())
    {
      case MSG_PING_REQUEST :
      {
        TSystemMessage* pMessage = NULL;
        if (TSystemMessage::createMessage(L"MessageObject", strSenderCellName, strSenderPipeName, &pMessage) == RET_NOERROR)
        {
          pMessage->setMessageType(MSG_PING_RESPONSE);
          m_pCommunication->sendMessage(pMessage, timer);
          pMessage->release();
        }
      }
      break;
      
      case MSG_PING_RESPONSE :
        //printf("MSG_PING_RESPONSE. Time=%Lu; Sender=%s; Pipe=%s\n", timer,strSenderCellName.c_str(),strSenderPipeName.c_str());
      break;
      
      default:
      ;
    }
  }
}

/***************************************************************************/
/*! Performs a xml message object
    \param msg : Object to perform
    \param timer : Value of the message time
    \pre m_pCommunication is valid
*/
/***************************************************************************/
void TAbstractCell::performXMLMessage(IXMLMessage* /*msg*/, unsigned long long /*timer*/)
{
}
    
/***************************************************************************/
/*! Performs a binary message object
    \param msg : Object to perform
    \param timer : Value of the message time
    \pre m_pCommunication is valid
*/
/***************************************************************************/
void TAbstractCell::performBinaryMessage(IBinaryMessage* /*msg*/, unsigned long long /*timer*/)
{}

/***************************************************************************/
/*! Performs a strean message object
    \param msg : Object to perform
    \param timer : Value of the message time
    \pre m_pCommunication is valid
*/
/***************************************************************************/
void TAbstractCell::performObjectMessage(IObjectMessage* /*msg*/, unsigned long long /*timer*/)
{}

/***************************************************************************/
/*! see ICellKernel
*/
/***************************************************************************/
void MQUALIFIER TAbstractCell::initialize(ICellComm* comm)
{
  if (m_pCommunication != NULL)
  {
    m_pCommunication->release();
    m_pCommunication = NULL;
  }
  m_pCommunication = comm;
  if (m_pCommunication != NULL)
  {
    m_pCommunication->addRef();
  }
  if (m_pLifeObj == NULL)
  {
    m_pLifeObj = new TAbstractCell::TCellLife(this, m_lSleepTime);
    m_pLifeObj->start();
  }
}


/***************************************************************************/
/*! see ICellKernel
*/
/***************************************************************************/
void MQUALIFIER TAbstractCell::uninitialize()
{
  if (m_pLifeObj != NULL)
  {
    m_pLifeObj->doTerminate();
    delete m_pLifeObj;
    m_pLifeObj = NULL;
  }
  if (m_pCommunication != NULL)
  {
    m_pCommunication->release();
    m_pCommunication = NULL;
  }
}

/***************************************************************************/
/*! see ICellKernel
*/
/***************************************************************************/
bool MQUALIFIER TAbstractCell::isInitialized()
{
  return (m_pLifeObj != NULL);
}


/**************************************************************************/
/*! Memory Management
*/
/**************************************************************************/
MEMORY_MANAGER_IMPL(TAbstractCell);
  INTERFACE_CAST(ICellKernel, INTERFACE_ICellKernel);
MEMORY_MANAGER_IMPL_PARENT_END(TXObject);

