/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : CommPipeTest                                                  *
 * Package : Test                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/25 10:01:44 $                                  *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: CommPipeTest.cpp,v $
 *   Revision 1.2  2005/11/25 10:01:44  bhadorn
 *   nameserver XML changed
 *
 *   Revision 1.1  2005/11/16 11:37:41  bhadorn
 *   Creates test classes for communication of cells
 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/  

#include <zeustest/CellModel/CommPipeTest.h>
#include <zeusbase/Messaging/XMLMessage.h>
#include <zeusbase/Remote/Naming.h>

USING_NAMESPACE_Zeus

CPPUNIT_TEST_SUITE_REGISTRATION( TCommPipeTest );


/***************************************************************************/
/*! Sets up a test
*/
/***************************************************************************/
void TCommPipeTest::setUp()
{
  Naming.useRemote();
  TString strXMLStream = TString(L"<?xml version=\"1.0\"?>\n\
    <zeus ClassName=\"TSystemManager\">\n\
      <NameServer ClassName=\"TNameServer\" CodeModule=\"NameServer\" Address=\"127.0.0.1\" Port=\"7278\"/>\n\
    </zeus>");
    
  IXObject* pObj = NULL;
  if (TXObjectFactory::getInstance().createObjectFromStream(&strXMLStream, (IXObject**)&pObj, false) == RET_NOERROR)
  {
    m_lstRoots.push(pObj);
  }
  
  //Try to create a stub and connect to the server
  TTime::sleep(0.75);
}

/***************************************************************************/
/*! Releases all instances
*/
/***************************************************************************/
void TCommPipeTest::tearDown()
{
  //Release all instances
  while(!m_lstStack.isEmpty())
  {
    m_lstStack.pop()->release();
  }
  while(!m_lstRoots.isEmpty())
  {
    IXObject* pObj = m_lstRoots.pop();
    pObj->freeze();
    pObj->release();
  }
  Naming.reset();
  Naming.useRemote(false);
}

/***************************************************************************/
/*! Tests the creation of the commpipes
*/
/***************************************************************************/
void TCommPipeTest::test_create()
{
  TCommPipe* pPipe1 = new TCommPipe(L"TestPipe");
  m_lstStack.push((ICommPipe*)pPipe1);

  TString strTestName;
  pPipe1->getPipeName(&strTestName);
  CPPUNIT_ASSERT(strTestName == L"TestPipe");
  CPPUNIT_ASSERT(pPipe1->getPriority() == PIPEPRIORITY_NORMAL);
  CPPUNIT_ASSERT(!pPipe1->hasRemoteAccess());

  TCommPipe* pPipe2 = new TCommPipe(PIPEPRIORITY_HIGH, L"TestPipe2", L"CellTest");
  m_lstStack.push((ICommPipe*)pPipe2);

  pPipe2->getPipeName(&strTestName);
  CPPUNIT_ASSERT(strTestName == L"TestPipe2");
  TString strCellName;
  pPipe2->getCellName(&strCellName);
  CPPUNIT_ASSERT(strCellName == L"CellTest");
  CPPUNIT_ASSERT(pPipe2->getPriority() == PIPEPRIORITY_HIGH);
  CPPUNIT_ASSERT(!pPipe2->hasRemoteAccess());  
}

/***************************************************************************/
/*! Tests locking of the pipe
*/
/***************************************************************************/
void TCommPipeTest::test_lock()
{
  TCommPipe* pPipe = new TCommPipe(L"TestPipe");
  m_lstStack.push((ICommPipe*)pPipe);
  IXMLMessage* pMessage = NULL;

  CPPUNIT_ASSERT(TXMLMessage::createMessage(L"MyMessage", L"", L"", &pMessage) == RET_NOERROR);
  m_lstStack.push((IMessage*)pMessage);

  CPPUNIT_ASSERT(pPipe->isLocked());
  CPPUNIT_ASSERT(pPipe->sendMessage(pMessage) != RET_NOERROR);
  CPPUNIT_ASSERT(pPipe->getCount() == 0);

  pPipe->unlock();
  CPPUNIT_ASSERT(!pPipe->isLocked());
  CPPUNIT_ASSERT(pPipe->sendMessage(pMessage) == RET_NOERROR);
  CPPUNIT_ASSERT(pPipe->getCount() == 1);

  pPipe->lock();
  IMessage* pTestMessage = NULL;
  CPPUNIT_ASSERT(pPipe->getMessage(&pTestMessage) != RET_NOERROR);
  CPPUNIT_ASSERT(pPipe->peekMessage(&pTestMessage) != RET_NOERROR);
  CPPUNIT_ASSERT(pPipe->getCount() == 1);
} 


/***************************************************************************/
/*! Tests sending and getting the messages in the right order
*/
/***************************************************************************/
void TCommPipeTest::test_sendMessage()
{
  TCommPipe* pPipe = new TCommPipe(L"TestPipe");
  pPipe->unlock();
  m_lstStack.push((ICommPipe*)pPipe);
  
  IXMLMessage* pMessage1 = NULL;  
  CPPUNIT_ASSERT(TXMLMessage::createMessage(L"MyMessage1", L"", L"", &pMessage1) == RET_NOERROR);
  m_lstStack.push(pMessage1);
  
  IXMLMessage* pMessage2 = NULL;  
  CPPUNIT_ASSERT(TXMLMessage::createMessage(L"MyMessage2", L"", L"", &pMessage2) == RET_NOERROR);
  m_lstStack.push(pMessage2);
  
  IXMLMessage* pMessage3 = NULL;  
  CPPUNIT_ASSERT(TXMLMessage::createMessage(L"MyMessage3", L"", L"", &pMessage3) == RET_NOERROR);
  m_lstStack.push((IMessage*)pMessage3);
  
  CPPUNIT_ASSERT(pPipe->sendMessage(pMessage1) == RET_NOERROR);
  CPPUNIT_ASSERT(pPipe->getCount() == 1);
  CPPUNIT_ASSERT(pPipe->sendMessage(pMessage2) == RET_NOERROR);
  CPPUNIT_ASSERT(pPipe->getCount() == 2);
  CPPUNIT_ASSERT(pPipe->sendMessage(pMessage3) == RET_NOERROR);
  CPPUNIT_ASSERT(pPipe->getCount() == 3);
    
  TString strName;
  IXMLMessage* pTestMessage1 = NULL;
  CPPUNIT_ASSERT(pPipe->getMessage((IMessage**)&pTestMessage1) == RET_NOERROR);
  m_lstStack.push(pTestMessage1);
  pTestMessage1->getMessageName(&strName);
  CPPUNIT_ASSERT(strName == L"MyMessage1");
  
  IXMLMessage* pTestMessage2 = NULL;
  CPPUNIT_ASSERT(pPipe->getMessage((IMessage**)&pTestMessage2) == RET_NOERROR);
  m_lstStack.push(pTestMessage2);
  pTestMessage2->getMessageName(&strName);
  CPPUNIT_ASSERT(strName == L"MyMessage2");

  IXMLMessage* pTestMessage3 = NULL;
  CPPUNIT_ASSERT(pPipe->getMessage((IMessage**)&pTestMessage3) == RET_NOERROR);
  m_lstStack.push(pTestMessage3);
  pTestMessage3->getMessageName(&strName);
  CPPUNIT_ASSERT(strName == L"MyMessage3");
}

/***************************************************************************/
/*! Tests the remote access and sending to a communication pipe
*/
/***************************************************************************/
void TCommPipeTest::test_sendremote()
{
  
  TString strPipeName = L"TestRemotePipe1";
  TCommPipe* pRemotePipe = new TCommPipe(PIPEPRIORITY_HIGH, strPipeName, L"CellTest");
  m_lstStack.push((ICommPipe*)pRemotePipe);
  CPPUNIT_ASSERT(pRemotePipe->connect() == RET_NOERROR);
  CPPUNIT_ASSERT(pRemotePipe->hasRemoteAccess());
  
  unsigned long ulRetval = RET_NOERROR;
  CPPUNIT_ASSERT(Naming.connect(L"127.0.0.1", 7278) == ulRetval);
  CPPUNIT_ASSERT(Naming.rebind(&strPipeName, pRemotePipe) == RET_NOERROR);
  
  //Get remote pipe from the naming service
  IRemoteObject* pRemoteObject = NULL;
  ICommPipe* pRemotePipeStub = NULL;
  CPPUNIT_ASSERT(Naming.lookup(&strPipeName, &pRemoteObject) == RET_NOERROR);
  m_lstStack.push(pRemoteObject);
  CPPUNIT_ASSERT(pRemoteObject->askForInterface(INTERFACE_ICommPipe, (IZUnknown**)&pRemotePipeStub) == RET_NOERROR);
  m_lstStack.push(pRemotePipeStub);
  
  //Try to send a message to the remote locked pipe (should fail);
  // Stub is not locked because connection was successful
  IXMLMessage* pMessage1 = NULL;  
  CPPUNIT_ASSERT(TXMLMessage::createMessage(L"MyMessage1", L"", L"", &pMessage1) == RET_NOERROR);
  m_lstStack.push(pMessage1);
  CPPUNIT_ASSERT(!pRemotePipeStub->isLocked());
  CPPUNIT_ASSERT(pRemotePipeStub->sendMessage(pMessage1) != RET_NOERROR);
  
  //No Message (server side)
  CPPUNIT_ASSERT(pRemotePipe->getCount() == 0);
  
  //Send a message to a unlocked pipe (should succeed)
  pRemotePipe->unlock();
  CPPUNIT_ASSERT(!pRemotePipeStub->isLocked());
  CPPUNIT_ASSERT(pRemotePipeStub->sendMessage(pMessage1) == RET_NOERROR);
  
  //One Message (server side)
  CPPUNIT_ASSERT(pRemotePipe->getCount() == 1);
  TString strName;
  IXMLMessage* pTestMessage1 = NULL;
  CPPUNIT_ASSERT(pRemotePipe->getMessage((IMessage**)&pTestMessage1) == RET_NOERROR);
  m_lstStack.push(pTestMessage1);
  pTestMessage1->getMessageName(&strName);
  CPPUNIT_ASSERT(strName == L"MyMessage1");
  
  
  CPPUNIT_ASSERT(Naming.unbind(&strPipeName) == RET_NOERROR);
  Naming.useRemote(false);
}


/***************************************************************************/
/*! Tests the remote informations of a communication pipe
*/
/***************************************************************************/
void TCommPipeTest::test_getInfoRemote()
{
  TString strPipeName = L"TestRemotePipe1";
  TCommPipe* pRemotePipe = new TCommPipe(PIPEPRIORITY_HIGH, strPipeName, L"CellTest");
  m_lstStack.push((ICommPipe*)pRemotePipe);
  CPPUNIT_ASSERT(pRemotePipe->connect() == RET_NOERROR);
  CPPUNIT_ASSERT(pRemotePipe->hasRemoteAccess());
  unsigned long ulRetval = RET_NOERROR;
  CPPUNIT_ASSERT(Naming.connect(L"127.0.0.1", 7278) == ulRetval);
  CPPUNIT_ASSERT(Naming.rebind(&strPipeName, pRemotePipe) == RET_NOERROR);

  //Get remote pipe from the naming service
  IRemoteObject* pRemoteObject = NULL;
  ICommPipe* pRemotePipeStub = NULL;
  CPPUNIT_ASSERT(Naming.lookup(&strPipeName, &pRemoteObject) == RET_NOERROR);
  m_lstStack.push(pRemoteObject);
  CPPUNIT_ASSERT(pRemoteObject->askForInterface(INTERFACE_ICommPipe, (IZUnknown**)&pRemotePipeStub) == RET_NOERROR);
  m_lstStack.push(pRemotePipeStub);
  
  TString strCellNameTest;
  TString strPipeNameTest;
  CPPUNIT_ASSERT(pRemotePipeStub->getCellName(&strCellNameTest) == RET_NOERROR);
  CPPUNIT_ASSERT(strCellNameTest == L"CellTest");
  CPPUNIT_ASSERT(pRemotePipeStub->getPipeName(&strPipeNameTest) == RET_NOERROR);
  CPPUNIT_ASSERT(strPipeNameTest == L"TestRemotePipe1");
  
  CPPUNIT_ASSERT(pRemotePipeStub->getPriority() == PIPEPRIORITY_HIGH);
  
  CPPUNIT_ASSERT(Naming.unbind(&strPipeName) == RET_NOERROR);
  Naming.useRemote(false);
}
