/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FuzzyLogic                                                    *
 * Package : Math                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:26:39 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: FuzzyLogic.cpp,v $
 *   Revision 1.1  2005/11/10 10:26:39  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
 
#include <zeusbase/Math/FuzzyLogic.h>   
#include <zeusbase/Math/Math.hpp>   
   
   
/***************************************************************************/
/*! Creates a fuzzy logic object
*/
/***************************************************************************/
TFuzzyLogic::TFuzzyLogic(TFuzzyInferenceMethod method /*= fimMamdani*/) : TZObject(), m_lstResults(TFuzzySet())
{
  m_Method = method;
}

/***************************************************************************/
/*! Destroys the fuzzy logic object
*/
/***************************************************************************/
TFuzzyLogic::~TFuzzyLogic()
{

}

/***************************************************************************/
/*! Resets the logic for a new conclusion
*/
/***************************************************************************/
void TFuzzyLogic::reset()
{
  m_lstResults.clear();
}

/***************************************************************************/
/*! adds an and-condition, 
    like if signal1 is set1 AND signal2 is set2 THEN ... <BR>
    The result will be stored in the result list
    \param signal1 : Input Signal 1
    \param set1    : Membership function of signal 1
    \param signal2 : Input Signal 2
    \param set2    : Membership function of signal 2
    \param output_membership : Membership function of the output
*/
/***************************************************************************/
void TFuzzyLogic::addAndCondition(double signal1, const TFuzzySet& set1, double signal2, const TFuzzySet& set2, TFuzzySet output_membership)
{
  double dAct1 = set1.getMemberShip(signal1);
  double dAct2 = set2.getMemberShip(signal2);
  double dA = TMath::minDouble(dAct1, dAct2);
  
  long lCount = output_membership.getResolution();
  switch(m_Method)
  {
    case fimLarson :
      for(int i = 0; i < lCount; i++)
      {
        output_membership.setValue(i, dA * output_membership.getValue(i));
      }
    break;
    
    case fimMamdani : //fall through
    default:
      for(int i = 0; i < lCount; i++)
      {
        double dValue = output_membership.getValue(i);
        if (dValue >= dA)
        {
          output_membership.setValue(i, dA);
        }
      }
    break;
  }
  m_lstResults.add(output_membership);  
}


/***************************************************************************/
/*! Gets the conclusion set of all added fuzzy conditions
*/
/***************************************************************************/
TFuzzySet TFuzzyLogic::getConclusionSet()
{
  TFuzzySet Conclusion(m_lstResults[0].getResolution());
  IListIterator<TFuzzySet>* pIt = m_lstResults.getIterator();
  while(pIt->hasNextItem())
  {
    Conclusion = Conclusion.intersectS(pIt->getNextItem());
  }
  m_lstResults.releaseIterator(pIt);
  
  return Conclusion;
}
