/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FuzzySet                                                      *
 * Package : Math                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:26:39 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: FuzzySet.cpp,v $
 *   Revision 1.1  2005/11/10 10:26:39  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <zeusbase/Math/FuzzySet.h>
#include <zeusbase/Math/Math.hpp>
#include <string.h>
#include <math.h>

USING_NAMESPACE_Zeus


/******************************************************************************/
/*! Creates a fuzzy set
    \param res : Resolution of the set.
*/
/******************************************************************************/
TFuzzySet::TFuzzySet(long res/*=100*/) : TZObject()//, m_lstValues(res)
{
  m_lResolution = res;
  double* pArray = new double[res];
  ::memset((void*)pArray, 0, sizeof(double)*res);
  
  m_lstValues.setArray(pArray, res);
  
  delete [] pArray;
}

/******************************************************************************/
/*! Copy constructor of the fuzzy set
    \param set : Object to copy
*/
/******************************************************************************/
TFuzzySet::TFuzzySet(const TFuzzySet& set)
{
  m_lResolution = set.m_lResolution;
  m_lstValues = set.m_lstValues;
}


/******************************************************************************/
/*! Destroys the fuzzy set
*/
/******************************************************************************/
TFuzzySet::~TFuzzySet()
{}


/******************************************************************************/
/*! Initialises the set using the trapezodial function.
    \param x1 : Beginning of the positive slope
    \param x2 : End of the positive slope 
    \param x3 : Beginning of the negative slope
    \param x4 : End of the negative slope
*/
/******************************************************************************/
void TFuzzySet::initTrapezodial(double x1, double x2, double x3, double x4)
{
  for(int i = 0; i < m_lResolution; i++)
  {
    double dZ = 0.0;
    if (i >= x2 && i <= x3)
    {
      dZ = 1.0;
    }
    else if (i >= x1 && i < x2)
    {
      dZ = (i-x1)/(x2-x1);
    }
    else if (i > x3 && i < x4)
    {
      dZ = (i-x4)/(x3-x4);
    }
    m_lstValues[i] = TMath::maxDouble(0,dZ);
  }
}

/******************************************************************************/
/*! Initialises the set using the synchron gaussian function
*/
/******************************************************************************/
void TFuzzySet::initGaussianSyn(double a, double whm)
{
  for(int i = 0; i < m_lResolution; i++)
  {
    m_lstValues[i] = exp( -log(2)/(whm*whm) * (i - a) * (i - a ));
  }
}

/******************************************************************************/
/*! Initialises the set using the asynchron gaussian function
*/
/******************************************************************************/
void TFuzzySet::initGaussianAsyn(double a, double whm1, double whm2)
{
  for(int i = 0; i < m_lResolution; i++)
  {
    if(i <= a)
    {
      m_lstValues[i] = exp( -log(2)/(whm1*whm1) * (i - a) * (i - a ));
    }
    else
    {
      m_lstValues[i] = exp( -log(2)/(whm2*whm2) * (i - a) * (i - a ));
    }
  }
}

/******************************************************************************/
/*! Initialises the set using the sigmodial function
*/
/******************************************************************************/
void TFuzzySet::initSigmodial(double a, double b)
{
  for(int i = 0; i < m_lResolution; i++)
  {
    m_lstValues[i] = 1.0/(1.0 + exp(-log(10)/b * (i-a )));
  }
}

/******************************************************************************/
/*! Returns the resolution of a fuzzy set
    \return Resolution
*/
/******************************************************************************/
long TFuzzySet::getResolution() const
{
  return m_lResolution;
}


/******************************************************************************/
/*! Gets the value of the fuzzy set at position index
    \param index : Value position
    \return value
*/
/******************************************************************************/
double TFuzzySet::getValue(long index) const
{
  return m_lstValues.getItemCopy(index);
}

/******************************************************************************/
/*! Gets the membership of a value. The variable x must be between 0 and 1.
   \param x : value
   \return membership
*/
/******************************************************************************/
double TFuzzySet::getMemberShip(double x) const
{
  double dA = x;
  dA = TMath::maxDouble(0,dA);
  dA = TMath::minDouble(1,dA);
  
  long lSlot = TMath::maxInt(1, (int)(dA*m_lResolution));
  return m_lstValues.getItemCopy(lSlot);
}


/******************************************************************************/
/*! Sets a new value to the fuzzy set
    \param index : Index of the value
    \param value : new value
*/
/******************************************************************************/
void TFuzzySet::setValue(long index, double value)
{
  m_lstValues[index] = value;
}

/******************************************************************************/
/*! Gets a defuzzificated value
    \param method : Method tu use to get the defuzzification value
    \param param : used for dmIndexedCenterOfGravity as a threshold
    \return defuzzificated value
*/
/******************************************************************************/
double TFuzzySet::getDefuzzificationValue(TDefuzzyMethod method /*= dmCenterOfGrapvity*/, double param /*= 1.0*/) const
{
  double dRetval = 0.0;
  
  switch(method)
  {
    // gets the center of gravity of those values above the level [param]
    case dmIndexedCenterOfGravity :
    {
      double dSum = 0.0;
      double dSum2 = 0;
      for(int i = 0; i < m_lResolution; i++)
      {
        double dValue = this->m_lstValues.getItemCopy(i);
        if (dValue > param)
        {
          dSum += dValue;
          dSum2 += dValue*i;
        }
      }
      if (dSum > 0)
      {
        dRetval = dSum2/dSum;
      }
    }
    break;  
    
    //gets the center of the sum
    case dmCenterOfSums : 
    
    break;
  
    // gets the first max value
    case dmFirstOfMaximum :
    {
      double dMaxVal = 0.0;
      double dMinIndex = 0;
      for(int i = 0; i < m_lResolution; i++)
      {
        double dValue = this->m_lstValues.getItemCopy(i);
        if (dValue > dMaxVal) // Only the first max value will be taken
        {
          dMaxVal = dValue;
          dMinIndex = i;
        }
      }
      dRetval = dMinIndex; 
    }
    break;
    
    // gets the middle max value
    case dmMiddleOfMaximum :
    {
      double dMaxVal = 0.0;
      for(int i = 0; i < m_lResolution; i++)
      {
        double dValue = this->m_lstValues.getItemCopy(i);
        if (dValue > dMaxVal)
        {
          dMaxVal = dValue;
        }
      }
      
      double dStart = 0.0;
      double dEnd = 0.0;
      for(int i = 0; i < m_lResolution; i++)
      {
        double dValue = this->m_lstValues.getItemCopy(i);
        if (dValue == dMaxVal && dStart == 0.0)
        {
          dStart = i;
        }
        else if (dValue == dMaxVal)
        {
          dEnd = i;
        }
      }
      if (dStart > dEnd)
      {
        dEnd = dStart;
      }
      dRetval = dStart + (dEnd - dStart)/2.0;
    }
    break;
    
    // gets the last max value
    case dmLastOfMaximum :
    {
      double dMaxVal = 0.0;
      double dMaxIndex = 0;
      for(int i = 0; i < m_lResolution; i++)
      {
        double dValue = this->m_lstValues.getItemCopy(i);
        if (dValue >= dMaxVal) // gets the last max value will be taken
        {
          dMaxVal = dValue;
          dMaxIndex = i;
        }
      }
      dRetval = dMaxIndex; 
    }
    break;
    
    // gets the center of gravity
    case dmCenterOfGrapvity : //fall through
    default:
    {
      double dSum = sum();
      if (dSum != 0)
      {
        double dSum2 = 0.0;
        for(int i = 0; i < m_lResolution; i++)
        {
          dSum2 += (this->m_lstValues.getItemCopy(i)*i);
        }
        dRetval = dSum2/dSum;
      }
    }
    break;
  }
  
  return dRetval;
}

/******************************************************************************/
/*! Gets the sum of the Fuzzy Set
    \return sum
*/
/******************************************************************************/
double TFuzzySet::sum() const
{
  double dRetval = 0.0;
  for(int i = 0; i < m_lResolution; i++)
  {
    dRetval += this->m_lstValues.getItemCopy(i);
  }
  return dRetval;
}

/******************************************************************************/
/*! Creates the union of two fuzzy sets (1st set is this, 2nd set is [set]
    \param set : 2nd set
    \param norm : Norm how to get the union
    \param param: used by the norm fnYager
    \return new Fuzzy set 
*/
/******************************************************************************/
TFuzzySet TFuzzySet::unionS(const TFuzzySet& set, TFuzzyNorm norm /*= fnZadeh*/, double param /*=1.0*/) const
{
  TFuzzySet NewSet(m_lResolution);
  
  switch(norm)
  {
    case fnProdConnect:
      for(int i = 0; i < m_lResolution; i++)
      {
        NewSet.m_lstValues[i] = this->m_lstValues.getItemCopy(i) * set.m_lstValues.getItemCopy(i);
      }
    break;
    
    case fnLukasiewiez:
      for(int i = 0; i < m_lResolution; i++)
      {
        NewSet.m_lstValues[i] = TMath::minDouble( 1.0,  this->m_lstValues.getItemCopy(i) + set.m_lstValues.getItemCopy(i) );
      }
    break;
    
    case fnYager:
      if (param != 0.0)
      {
        for(int i = 0; i < m_lResolution; i++)
        {
          NewSet.m_lstValues[i] = 1.0 - TMath::minDouble(1.0, 
                                        pow(pow(this->m_lstValues.getItemCopy(i), param) + 
                                         pow(set.m_lstValues.getItemCopy(i),param), 1.0/param));
        }
      }
    break;
     
    case fnZadeh : //fall through
    default:
      for(int i = 0; i < m_lResolution; i++)
      {
        NewSet.m_lstValues[i] = TMath::maxDouble(this->m_lstValues.getItemCopy(i), set.m_lstValues.getItemCopy(i));
      }
    break;
  }
  return NewSet;
}

/******************************************************************************/
/*! Creates the intersection of two fuzzy sets (1st set is this, 2nd set is [set]
    \param set : 2nd set
    \param norm : Norm how to get the intersection of two sets
    \param param : Used by the norm fnYager 
    \return new Fuzzy set 
*/
/******************************************************************************/
TFuzzySet TFuzzySet::intersectS(const TFuzzySet& set, TFuzzyNorm norm /*= fnZadeh*/, double param /*=1.0*/) const
{
  TFuzzySet NewSet(m_lResolution);
  switch(norm)
  {
    case fnProdConnect:
      for(int i = 0; i < m_lResolution; i++)
      {
        double dA = this->m_lstValues.getItemCopy(i);
        double dB = set.m_lstValues.getItemCopy(i);
        NewSet.m_lstValues[i] = dA + dB - (dA * dB);
      }
    break;
    
    case fnLukasiewiez:
      for(int i = 0; i < m_lResolution; i++)
      {
        NewSet.m_lstValues[i] = TMath::maxDouble(0, this->m_lstValues.getItemCopy(i)+ set.m_lstValues.getItemCopy(i)-1);
      }
    break;
    
    case fnYager:
      if (param != 0.0)
      {
        for(int i = 0; i < m_lResolution; i++)
        {
          NewSet.m_lstValues[i] = 1.0 - TMath::minDouble(1.0, 
                                        pow(pow(1.0 - this->m_lstValues.getItemCopy(i), param) + 
                                         pow(1.0-set.m_lstValues.getItemCopy(i),param), 1.0/param));
        }
      }
    break;
     
    case fnZadeh : //fall through
    default:
      for(int i = 0; i < m_lResolution; i++)
      {
        NewSet.m_lstValues[i] = TMath::minDouble(this->m_lstValues.getItemCopy(i), set.m_lstValues.getItemCopy(i));
      }
    break;
  }
  return NewSet;
}

/******************************************************************************/
/*! Operator to get the inverse of this set
    \return inverse set
*/
/******************************************************************************/
TFuzzySet TFuzzySet::notS() const
{
  TFuzzySet NewSet(m_lResolution);
  for(int i = 0; i < m_lResolution; i++)
  {
    NewSet.m_lstValues[i] = 1-this->m_lstValues.getItemCopy(i);
  }
  return NewSet;
}


/******************************************************************************/
/*! Operator to concentrate a fuzzy set
    \return concentrated set
*/
/******************************************************************************/
TFuzzySet TFuzzySet::concentrate() const
{
  TFuzzySet NewSet(m_lResolution);
  for(int i = 0; i < m_lResolution; i++)
  {
    NewSet.m_lstValues[i] = pow(this->m_lstValues.getItemCopy(i), 2);
  }
  return NewSet;
}

/******************************************************************************/
/*! Operator to dilate a fuzzy set
    \return dilated set
*/
/******************************************************************************/
TFuzzySet TFuzzySet::dilate() const
{
  TFuzzySet NewSet(m_lResolution);
  for(int i = 0; i < m_lResolution; i++)
  {
    NewSet.m_lstValues[i] = pow(this->m_lstValues.getItemCopy(i), 0.5);
  }
  return NewSet;
}

/******************************************************************************/
/*! Operator to intense a fuzzy set
    \return intensed set 
*/
/******************************************************************************/
TFuzzySet TFuzzySet::intense() const
{
  TFuzzySet NewSet(m_lResolution);
  for(int i = 0; i < m_lResolution; i++)
  {
    double dA = this->m_lstValues.getItemCopy(i);
    if (dA <= 0.5)
    {
      NewSet.m_lstValues[i] = 2*pow(dA, 2);
    }
    else
    {
      NewSet.m_lstValues[i] = 1.0 - 2*pow(1.0 - dA, 2);
    }
  }
  return NewSet;
}

/******************************************************************************/
/*! Linguistic modifier VERY of a fuzzy set
*/
/******************************************************************************/
TFuzzySet TFuzzySet::lingVery() const
{
  return this->concentrate();
}

/******************************************************************************/
/*! Linguistic modifier MOR OR LESS of a fuzzy set
*/
/******************************************************************************/
TFuzzySet TFuzzySet::lingMoreOrLess() const
{
  return this->dilate();
}

/******************************************************************************/
/*! Linguistic modifier VERY VERY of a fuzzy set
*/
/******************************************************************************/
TFuzzySet TFuzzySet::lingVeryVery() const
{
  return this->concentrate().concentrate();
}

/******************************************************************************/
/*! Linguistic modifier NOT VERY of a fuzzy set
*/
/******************************************************************************/
TFuzzySet TFuzzySet::lingNotVery() const
{
  return this->concentrate().notS();
}

/******************************************************************************/
/*! Linguistic modifier MORE of a fuzzy set
*/
/******************************************************************************/
TFuzzySet TFuzzySet::lingMore() const
{
  TFuzzySet NewSet(m_lResolution);
  for(int i = 0; i < m_lResolution; i++)
  {
    NewSet.m_lstValues[i] = pow(this->m_lstValues.getItemCopy(i), 1.25);
  }
  return NewSet;
}

/******************************************************************************/
/*! Linguistic modifier LESS of a fuzzy set
*/
/******************************************************************************/
TFuzzySet TFuzzySet::lingLess() const
{
  TFuzzySet NewSet(m_lResolution);
  for(int i = 0; i < m_lResolution; i++)
  {
    NewSet.m_lstValues[i] = pow(this->m_lstValues.getItemCopy(i), 0.75);
  }
  return NewSet;
}

        
/******************************************************************************/
/*! Copy operator
    \param set : Set to copy
    \return the copied set
*/
/******************************************************************************/
TFuzzySet& TFuzzySet::operator=(const TFuzzySet& set)
{
  this->m_lstValues = set.m_lstValues;
  this->m_lResolution = set.m_lResolution;
  return *this;
}

/******************************************************************************/
/*! Equals operator
    \param set : Set to check if equal
    \retval true : Set is equal
    \retval false : Set is not equal
*/
/******************************************************************************/
bool TFuzzySet::operator==(const TFuzzySet& set) const
{
  bool bRetval = false;
  if (this->m_lResolution == set.getResolution())
  {
    bRetval = true;
    for(int i = 0; i < m_lResolution && bRetval; i++)
    {
      bRetval &= (m_lstValues.getItemCopy(i) == set.m_lstValues.getItemCopy(i));
    }
  }
  return bRetval;
}

 
