/***************************************************************************
 * Copyright (C) 2005 by Benjamin Hadorn (bhadorn@swissinfo.org)           *
 *************************************************************************** 
 * Projekt : Zeus                                                          *
 * Module  : FuzzySet                                                      *
 * Package : Math                                                          *
 * Author  : Benjamin Hadorn                                               *
 * Datum   : $Date: 2005/11/10 10:26:39 $                                                        *
 * Ablage  : $File$                                                        *
 * System  : Cell Computing Model                                          *
 ***************************************************************************
 * Aenderungen :                                                           *
 *   $Log: FuzzySet.h,v $
 *   Revision 1.1  2005/11/10 10:26:39  bhadorn
 *   Created
 *                                                                 *
 ***************************************************************************
 * Lizenz  :                                                               *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef FuzzySetHPP 
#define FuzzySetHPP

#include <zeusbase/System/ZObject.h>

BEGIN_NAMESPACE_Zeus

/******************************************************************************/
/*! This class defines a fuzzy set and all operators used to modify and 
    transform the set. The set is used together with the TFuzzyLogic.
*/
/******************************************************************************/
class TFuzzySet : public TZObject
{
  public:
      
    /************************************************************************/
    /*! Enumeration to specify the way how the fuzzy set will be computed 
    */
    /************************************************************************/
    enum TFuzzyNorm 
    { 
      fnZadeh,       /*!< Uses the connectivities of Zadeh */
      fnProdConnect, /*!< Uses the product connectivities */
      fnLukasiewiez, /*!< Uses the multi value logic of Lukasiewiez */
      fnYager        /*!< Uses the connectivities of Yager */
    };
    
    /************************************************************************/
    /*! Enumeration to spezify the way how to calculate the defuzzification 
    */
    /************************************************************************/
    enum TDefuzzyMethod 
    { 
      dmCenterOfGrapvity,       /*!< Gets the defuzzification value as the center of gravity*/
      dmIndexedCenterOfGravity, /*!< Gets the defuzzification value as the center of gravity*/
      dmCenterOfSums,           /*!< Gets the defuzzification value as the center of the sum*/
      dmFirstOfMaximum,         /*!< Gets the defuzzification value as the first maximum value*/
      dmMiddleOfMaximum,        /*!< Gets the defuzzification value as the middle of all maximum values*/
      dmLastOfMaximum           /*!< Gets the defuzzification value as the last maximum value*/
    };
    
    
    TFuzzySet(long res=100);
    TFuzzySet(const TFuzzySet& set);
    virtual ~TFuzzySet();
    
    void initTrapezodial(double x1, double x2, double x3, double x4);
    void initGaussianSyn(double a, double whm);
    void initGaussianAsyn(double a, double whm1, double whm2);
    void initSigmodial(double a, double b);
    
    double getValue(long index) const;
    double getMemberShip(double x) const;
    long getResolution() const;
    void setValue(long index, double value);
    
    virtual double getDefuzzificationValue(TDefuzzyMethod method = dmCenterOfGrapvity, double param = 1.0) const;
    
    virtual TFuzzySet unionS(const TFuzzySet& set, TFuzzyNorm norm = fnZadeh, double param=1.0) const;
    virtual TFuzzySet intersectS(const TFuzzySet& set, TFuzzyNorm norm = fnZadeh, double param=1.0) const;
    virtual TFuzzySet notS() const;
    virtual TFuzzySet concentrate() const;
    virtual TFuzzySet dilate() const;
    virtual TFuzzySet intense() const;

    virtual TFuzzySet lingVery() const;
    virtual TFuzzySet lingMoreOrLess() const;
    virtual TFuzzySet lingVeryVery() const;
    virtual TFuzzySet lingNotVery() const;
    virtual TFuzzySet lingMore() const;
    virtual TFuzzySet lingLess() const;
    
    TFuzzySet& operator=(const TFuzzySet& set);
    bool operator==(const TFuzzySet& set) const;
//    virtual TFuzzySet operator||(const TFuzzySet& set);
  
  protected:
    ///Array of Values
    TArrayList<double> m_lstValues;
    ///Resolution of the set
    long m_lResolution;    
    
    double sum() const; 
};

END_NAMESPACE_Zeus

#endif 
